// ;-*-C++-*- *  Time-stamp: "2008-07-02 00:41:09 hmmr"


/***************************************************************************
 *            ach_data.cc
 *
 *  Copyright  2008  Andrei Zavada
 *  Email johnhommer@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#include "agh_data.hh"
#include "agh_expdesign.hh"
#include "agh_io.hh"









CHypnogram::CHypnogram( const gchar *_fname, guint _psize)
{
	_pagesize = _psize;
	_pages = g_array_new( FALSE, TRUE, sizeof(SPage));
	_filename = g_string_new( _fname);
	if ( g_str_has_suffix( _fname, ".edf") || g_str_has_suffix( _fname, ".EDF") )
		g_string_truncate( _filename, _filename->len-4);
	g_string_append_printf( _filename, ".hypnogram");

	load();
}










inline double
neigh_avg( double *vec, guint over2)
{
	double avg = 0;
	for ( guint i = 1; i <= over2; i++ )
		avg += *(vec-i) + *(vec+i);
	return avg / (over2 * 2);
}

inline double
full_avg( double *vec, guint over2)
{
	double avg = 0;
	for ( guint i = 1; i <= over2; i++ )
		avg += *(vec-i) + *(vec+i);
	return (avg + *vec) / (over2 * 2 + 1);
}


gdouble
CMeasurement::ship_power_course( GArray *power, GArray *track,
				 gint ipagesize, SFFTParamsSet &req_params,
				 gfloat from, gfloat upto,
				 gint smooth, gfloat glitch_mag,
				 gboolean filtered)
{
	if ( !(req_params == fft_params) || _pagesize != ipagesize ) {
		fft_params = req_params, _pagesize = ipagesize;
		status &= ~AGH_BP_INMEM;
	}

	const gchar *fname       = make_fname(),
		    *fname_track = make_fname_track();

	if ( !(status & AGH_BP_INMEM) ) {
		load( fname);
		if ( !filtered )
			load_track( fname_track);
	}

	if ( filtered || !(status & AGH_BP_OBTAINED) ) {
		obtain_power( _source, _source->sig_no_by_channel_name( channel_name()),
			      ipagesize, req_params,
			      filtered);
		save( fname);
		save_track( fname_track);
	}

	g_array_set_size( power, _pages);
	if ( !_pages )
		return 0.;

      // copy power
	guint	bin_start = from / fft_params.bin_size,
		bin_end   = upto / fft_params.bin_size,
		n_bins    = bin_end - bin_start;
	gdouble	max_value = 0.;
	gulong	p;
	for ( p = 0; p < _pages; p++ ) {
		double &v = g_array_index (power, double, p);
		v = 0.;
		for ( guint bin = bin_start; bin < bin_end; bin++ )
			v += ijth_bin( bin, p);
		v /= n_bins;
		if ( v > max_value && p > 1 ) // skip unexplained surge at start
			max_value = v;
	}

	if ( smooth > 1 ) {
		gdouble *shadow = (gdouble*) g_malloc( sizeof(double) * _pages);
		memcpy( shadow, power->data, sizeof(double) * _pages);

		gint	i, sm_2 = (smooth-1)/2;
		guint	_pages_min_sm2 = _pages - sm_2;

		if ( glitch_mag > 1.1 )
			for ( p = sm_2+1; p < _pages_min_sm2; p++ ) {
				gdouble mavg = neigh_avg( shadow+p, sm_2);
				if ( shadow[p] > mavg * glitch_mag ) {
					g_array_index (power, double, p) = shadow[p] = mavg;
//					printf( "glitch at page %d\n", p);
				}
				g_array_index (power, double, p) = mavg;
			}
		else
			for ( p = sm_2+1; p < _pages_min_sm2; p++ )
				g_array_index (power, double, p) = full_avg( shadow+p, sm_2);

		g_free( shadow);
	}

      // copy track
	if ( track ) { // CSCourse::add_measurement does not need track
		g_array_set_size( track, _pages * _pagesize);
		memcpy( track->data, _track, _pages * _pagesize * sizeof(gchar));
	}


	return max_value;
}






// EOF
