// ;-*-C++-*- *  Time-stamp: "2008-07-01 16:34:56 hmmr"

/***************************************************************************
 *            agh_data.hh
 *
 *  Copyright  2008  Andrei Zavada
 *  Email johnhommer@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _AGH_DATA_H
#define _AGH_DATA_H

//#include <math.h>

#include <string.h>
#include <glib.h>

#include "z_lists.hh"
#include "agh_enums.h"


class CMeasurement;




extern gchar agh_score_codes[];

struct SPage {
	gfloat	NREM, REM, Wake;
	gchar	p2score()  { return (NREM >  3./4) ?agh_score_codes[AGH_SCORE_NREM4]
				   :(NREM >  1./2) ?agh_score_codes[AGH_SCORE_NREM3]
				   :(REM  >= 1./3) ?agh_score_codes[AGH_SCORE_REM  ]
				   :(Wake >= 1./3) ?agh_score_codes[AGH_SCORE_WAKE ]
				   :(NREM >  1./4) ?agh_score_codes[AGH_SCORE_NREM2]
				   :(NREM >   .1 ) ?agh_score_codes[AGH_SCORE_NREM1]
				   :(Wake == AGH_MVT_WAKE_VALUE) ?agh_score_codes[AGH_SCORE_MVT]
						   :agh_score_codes[AGH_SCORE_NONE];  }

	gboolean has_swa()  { return (NREM + REM > .2); }  // excludes NREM1, in fact
};

struct SPageWithSWA : public SPage {
	gdouble	SWA;
};

struct SPageSimulated : public SPageWithSWA {
	gdouble	S, SWA_sim;
};




class CHypnogram {
    friend class CEDFFile;

	GArray	*_pages;
	guint _pagesize;

	GString	*_filename;
    public:
	CHypnogram()	{ _pagesize = 30;
			  _pages = g_array_new( FALSE, TRUE, sizeof(SPage));
			  _filename = g_string_sized_new( 80);                }
	CHypnogram( const gchar *_fname,
		    guint _psize = 30);

       ~CHypnogram()    { save();
			  g_array_free( _pages, TRUE);
			  g_string_free( _filename, TRUE); }

	guint pagesize()            { return _pagesize; }
	void set_pagesize( gint p)  { _pagesize = p; }
	guint length()              { return _pages->len; }
	SPage &nth_page( guint i)   { return g_array_index (_pages, SPage, i); }

	void set_size( guint _sz)   { g_array_set_size( _pages, _sz); }

	gint save();
	gint save_canonical( const gchar*);
	gint load();
	gint load_canonical( const gchar*);
};









struct SFFTParamsSet {
	guint	window_size_2power;
	gfloat	bin_size;
	TFFTWinType window_type;

	void defaults()  { window_size_2power = 2, bin_size = .5;
			   window_type = AGH_WT_WELCH; }
	gint operator == ( SFFTParamsSet &o)  { return window_type == o.window_type &&
						       window_size_2power == o.window_size_2power &&
						       bin_size == o.bin_size; }
	void dump();
};




#define AGH_BP_OBTAINED		(1 << 0)
#define AGH_BP_HAS_TRACK	(1 << 1)
#define AGH_BP_INMEM		(1 << 2)

class CEDFFile;
class CSimulation;

class CBinnedPower {

	gdouble	*_data;
	gchar	*_track;
	gulong	 _pages;

    friend class CMeasurement;
    friend class CSimulation;
    friend class CExpDesign; // enable CExpDesign::load/save to fill these out
	gint	_pagesize,
		_samplerate;
	SFFTParamsSet fft_params;

	gint save( const gchar*, gboolean binary = TRUE);
	gint save_track( const gchar*, gboolean binary = TRUE);
	gint load( const gchar*, gboolean binary = TRUE);
	gint load_track( const gchar*, gboolean binary = TRUE);

	void set_size( guint _sz)       { guint n_bins = _samplerate/2/fft_params.bin_size;
					  _data = (double*)
						  g_realloc( _data,
							     ((_pages = _sz)+1) * n_bins * sizeof(double));
					  _track = (gchar*)
						  g_realloc( _track, (_pages+1) * _pagesize * sizeof(gchar));  }
    public:
	gint	status;

	CBinnedPower()  { _pages = 0, _data = NULL, _track = NULL, _samplerate = 0, _pagesize = 30;
			  status = 0;      } // don't care about rubbish in fields as long as status is 0
	// we don't know the _filename at creation time, so don't bother to load(), either
	// (would do load on demand, anyway)

	CBinnedPower( guint ipagesize,
		      const SFFTParamsSet &ifft_params)  { _pages = 0, _data = NULL, _track = NULL;
							   _samplerate = 0, _pagesize = ipagesize, fft_params = ifft_params;
							   status = 0; }
       ~CBinnedPower()  { g_free( _data), g_free( _track); }

	gint pagesize()             { return _pagesize; };
	gulong pages()              { return _pages; };

	gchar &nth_second( guint i)            { return *(_track + i); }
	gdouble &ijth_bin( guint n, gulong p)  { guint n_bins = _samplerate/2/fft_params.bin_size;
						 return _data[p * n_bins + n]; }

	gint obtain_power( CEDFFile *source, gint signal_no,
			   gint ipagesize, SFFTParamsSet&,
			   gboolean repeat = FALSE);
	void update_track( const gchar *track) { memcpy( _track, track, _pages * _pagesize * sizeof(gchar));
						 status |= AGH_BP_HAS_TRACK; }

	gint import_from_octave( const gchar *filename);
	gint import_from_ascii( const gchar *filename);
};






#endif

// EOF
