// ;-*-C++-*- *  Time-stamp: "2008-06-26 18:19:16 hmmr"


/***************************************************************************
 *            ach_exp-design.cc
 *
 *  Copyright  2008  Andrei Zavada
 *  Email johnhommer@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <glib.h>
#include <glib/gstdio.h>

#include "agh_io.hh"
#include "agh_expdesign.hh"
#include "agh_model.hh"









CSubject::CSubject( const gchar *iname,
		    TGender igender,  gint iage,
		    CRefList<CSubject> *to,
		    const gchar *icomment)
{
//	printf( " welcome %s (%p) to %s\n", iname, this, status, to ?to->name() :"---");
	_name = g_string_new( iname);
	_age = iage;
	_comment = g_string_new( icomment);
	_gender = igender;
	status = 0;
	if ( (belongs = to) )
		to->checkin( this);
}


CSubject::~CSubject()
{
	g_string_free( _name, TRUE);
	g_string_free( _comment, TRUE);
	if ( belongs )
		belongs->checkout( this);
}










CMeasurement::CMeasurement( COrderedName *iepisode,
			    CSubject *iowner,
			    COrderedName *ichannel,
			    CRefList<CMeasurement> *session,
			    CEDFFile *isource,
			    guint pagesize,
			    SFFTParamsSet &fft_params)
    : CBinnedPower( pagesize, fft_params)
{
	_episode = iepisode, _owner = iowner, _channel = ichannel, _source = isource;
	status = 0;
	if ( (belongs = session) )
		session->checkin( this);

	_basename = g_string_sized_new( 80);

	g_string_printf( _basename,
			 "%s/%s/%s/%s",
			 _owner->group()->name(),
			 _owner->name(),
			 belongs ?belongs->name() :"(loose)",
			 _episode->name());
	g_mkdir_with_parents( _basename->str, 0755);
}



CMeasurement::~CMeasurement()
{
	if ( !(status & Z_OBJECT_PHONY) ) {
		if ( belongs )
			belongs->checkout( this);
		g_string_free( _basename, TRUE);
		// don't do save explicitly here: saving is done right after obtain
	}
}









void
CMeasurement::dump()
{
	printf( "%p\tE:%s\tJ:%s\tH:%s\tD:%s\tF:%s%s%s%s\n",
		this,
		_episode->name(), _owner->name(), _channel->name(),
		belongs ?belongs->name() :"(loose)",
		_source ?_source->filename() :"(no source)",
		(status & AGH_BP_OBTAINED) ?"\tobtained" :"",
		(status & AGH_BP_INMEM) ?"\tin-mem" :"",
		(status & Z_OBJECT_PHONY) ?"\tPHONY" :"");
}







// ------------ CExpDesign

CExpDesign::CExpDesign( const gchar *_dir)
{
	printf( "Initialize ExpDesign: %s\n", _dir);

	channel_names  =
	episode_names  = NULL;
	groups	       = NULL;
	sessions       = NULL;
	subjects       = NULL;
	sources	       = NULL;
	measurements   = NULL;
	simulations    = NULL;

	_session_dir = g_string_new( _dir);

	if ( chdir( _dir) ) {
		printf( "Could not cd to %s: Trying to create a new directory there...", _dir);
		if ( !g_mkdir_with_parents( _dir, 0755) )
			printf( "done\n");
		else {
			printf( "failed\n");
			state = 0 | AGH_SESSION_STATE_INITFAIL;
			return;
		}
	}

	state = 0 | AGH_SESSION_STATE_INITFAIL;

	if ( !(channel_names = new CList<COrderedName>) )			goto fail;
	if ( !(episode_names = new CList<COrderedName>) )			goto fail;
	if ( !(groups        = new CAggregatorList< CRefList<CSubject> >) )	goto fail;
	if ( !(subjects      = new CList<CSubject>) )				goto fail;
	if ( !(sessions      = new CAggregatorList< CRefList<CMeasurement> >) )	goto fail;
	if ( !(measurements  = new CList<CMeasurement>) )			goto fail;
	if ( !(sources       = new CList<CEDFFile>) )				goto fail;
	if ( !(simulations   = new CList<CSimulation>) )			goto fail;

	printf( "Creating Primary session\n");
	sessions->add( "Primary");

	state = AGH_SESSION_STATE_OK;

	fft_params.defaults();
	tunables.assign_defaults();
	control_params.assign_defaults();

	return;
fail:
	printf( "ExpDesign Init: Failed to initialize\n");
	delete channel_names;	episode_names = NULL;
	delete episode_names;	channel_names = NULL;
	delete sessions;	sessions = NULL;
	delete groups;		groups = NULL;
	delete subjects;	subjects = NULL;
	delete sources;		sources = NULL;
	delete measurements;	measurements = NULL;
	delete simulations;	simulations = NULL;
}





CExpDesign::~CExpDesign()
{
	printf( "=== Delete simulations:\n");
	delete simulations;
	printf( "=== Delete measurements:\n");
	delete measurements;
	printf( "=== Delete subjects:\n");
	delete subjects;
	printf( "=== Delete sources:\n");
	delete sources;
	printf( "=== Delete groups:\n");
	delete groups;
	printf( "=== Delete sessions:\n");
	delete sessions;
	printf( "=== Delete channel_names:\n");
	delete channel_names;
	printf( "=== Delete episode_names:\n");
	delete episode_names;

	g_string_free( _session_dir, TRUE);
}







gint
CExpDesign::add_source( CEDFFile *F, TSourceAttachMode option)
{
	CEDFFile *Fy = new CEDFFile();
	Fy->make_phony( F->filename());
	CEDFFile *F_ = sources->exist( Fy);
	delete Fy;
	if ( F_ && !strcmp( F_->filename(), F->filename()) )
		delete_source( F_);

	sources->add( F);

	CSubject *Jy = new CSubject();
	Jy->make_phony( F->PatientID_raw);
	CSubject *J = subjects->exist( Jy);
	if ( !J ) {  // don't know what group to add to, so barf
		delete Jy;
		return -1;
	}

      // otherwise, go create channels, sessions and episodes as necessary
	COrderedName *E = episode_names->o_by_name( F->Episode);
	if ( !E )
		episode_names->add( E = new COrderedName( F->Episode));
	CRefList<CMeasurement> *D = sessions->o_by_name( F->Session);
	if ( !D ) {
		sessions->add( F->Session);
		D = sessions->o_by_name( F->Session);
	}

	CMeasurement	*My = new CMeasurement(),
			*M;  // would keep M around for taking its basename
	for ( guint h = 0; h < F->NSignals; h++ ) {
		if ( !F->H(h)->Channel )
			continue;

		COrderedName *H = channel_names->o_by_name( F->H(h)->Channel);
		if ( !H )
			channel_names->add( H = new COrderedName( F->H(h)->Channel));
		My->make_phony( E, J, H, D);
		M = measurements->exist( My);
		if ( !M )
			measurements->add( M = new CMeasurement( E, J, H, D, F, pagesize, fft_params));
		else
			M->_source = F;
//		M->dump();
	}
	delete My;

	GString *assimilated_edf = g_string_new( M->basename());
	g_string_append_printf( assimilated_edf, ".edf");
	F->assimilate( assimilated_edf->str, option);
	g_string_free( assimilated_edf, TRUE);

	return 0;
}





void
CExpDesign::delete_source( CEDFFile* F)
{
	sources->del( F);
	for ( guint m = 0; m < measurements->n(); m++ ) {
		CMeasurement *M = measurements->nth(m);
		if ( M->_source == F )
			M->_source = NULL;
	}
}





void
CExpDesign::delete_subject( CSubject *J)
{
	for ( guint m = 0; m < measurements->n(); m++ ) {
		CMeasurement *M = measurements->nth(m);
		if ( M->_owner == J )
			M->detach();
	}
	measurements->clean_unowned();
	J->detach();
	subjects->clean_unowned();
}



void
CExpDesign::delete_group( CRefList<CSubject> *G)
{
	for ( guint j = 0; j < G->n(); j++ )
		delete_subject( G->nth(j));
	groups->del( G->name());
}

















gint
CExpDesign::load()
{
	printf( "--- load session from %s\n", _session_dir->str);

	if ( state )
		return -1;

	GKeyFile *kf = g_key_file_new();

	GError *kf_error = NULL;
	GString *ext_msg = g_string_sized_new( 120);

	gint	intval;
	gdouble	dblval;

	guint	i;

	if ( !g_key_file_load_from_file( kf, "ExpDesign", G_KEY_FILE_KEEP_COMMENTS, &kf_error) ) {
		printf( "%s\n", kf_error->message);

		GString *ext_msg = g_string_sized_new( 120);
		switch ( kf_error->code ) {
		case G_KEY_FILE_ERROR_NOT_FOUND:
			g_string_printf( ext_msg,
					 "The file ExpDesign could not be read from location %s (%s)\n\n"
					 "This file should contain a description of your experiment design; "
					 "it will be created and your experimental setup will be saved in it ",
					 "at the end of your current session.",
					 _session_dir->str, kf_error->message);
			break;
		case G_KEY_FILE_ERROR_PARSE:
			g_string_printf( ext_msg,
					 "The file ExpDesign in %s could not be parsed.\n"
					 "It will be overwritten once you are done designing your experiment.",
					 _session_dir->str);
			break;
		default:
			g_string_printf( ext_msg,
					 "There was an error reading and parsing the ExpDesign file in %s: %s.\n"
					 "I will attempt to save your changes again at the end of the current session",
					 _session_dir->str, kf_error->message);
		}
		agh_pop_ok_message( ext_msg->str);

		return -1;
	}

	g_string_assign( ext_msg, "");

	if ( !g_key_file_has_group( kf, "Design") )
		g_string_printf( ext_msg, "No [Design] group found.\n");

	gchar	**key_value_list, *p;

	key_value_list = g_key_file_get_string_list( kf, "Design", "Sessions", NULL, NULL);
	i = 0;
	if ( key_value_list ) {
		while ( p = key_value_list[i++] )
			sessions->add( p);
		g_strfreev( key_value_list);
	}

	key_value_list = g_key_file_get_string_list( kf, "Design", "Channels", NULL, NULL);
	i = 0;
	if ( key_value_list ) {
		while ( p = key_value_list[i++] )
			channel_names->add( new COrderedName( p));
		g_strfreev( key_value_list);
	}

	key_value_list = g_key_file_get_string_list( kf, "Design", "Episodes", NULL, NULL);
	i = 0;
	if ( key_value_list ) {
		while ( p = key_value_list[i++] )
			episode_names->add( new COrderedName( p));
		g_strfreev( key_value_list);
	}

	key_value_list = g_key_file_get_string_list( kf, "Design", "Groups", NULL, NULL);
	i = 0;
	if ( key_value_list ) {
		while ( p = key_value_list[i++] )
			groups->add( p);
		g_strfreev( key_value_list);
	}

      // load subjects before sources, or else CExpDesign::add_source will barf loud
	gchar **group_list = g_key_file_get_groups( kf, NULL);
	if ( !group_list )
		g_string_append_printf( ext_msg, "No groups in file???.\n");
	else {
	      // go through a list of key groups [Subject jjj]
		gchar *kg;
		guint i = 0;
		while ( kg = group_list[i++] ) {
			if ( !strncmp( kg, "Subject ", 8) ) {
				gchar *x = g_key_file_get_value( kf, kg, "gender", NULL);
				gchar *y = g_key_file_get_value( kf, kg, "group", NULL);
				CRefList<CSubject> *G = groups->o_by_name( y);
				if ( !G )
					g_string_append_printf( ext_msg, "Subject %s claims to be in an unknown group: %s.\n", kg+8, y);
				else
					subjects->add( new CSubject( kg+8,
								     g_ascii_toupper( x[0]) == 'M' ?AGH_SUBJECT_GENDER_MALE :AGH_SUBJECT_GENDER_FEMALE,
								     g_key_file_get_integer( kf, kg, "age", NULL),
								     G,
								     g_key_file_get_value( kf, kg, "comment", NULL)));
				g_free( x);
				g_free( y);
			}
		}
	}

	intval = g_key_file_get_integer( kf, "PageSize", "Value", NULL);
	if ( intval == 0 || intval % 5 != 0 ) {
		g_string_append_printf( ext_msg, "Expecting PageSize to be a non-zero multiple of 5.\n");
		pagesize = 30;
	} else
		agh_pagesize_set_current( intval);
		//pagesize = intval;

	intval = g_key_file_get_integer( kf, "FFT", "WindowType", NULL);
	if ( intval > AGH_WT_N_TYPES || intval < 0 ) {
		g_string_append_printf( ext_msg, "Bad WindowType.\n");
		fft_params.window_type = AGH_WT_WELCH;
	} else
		fft_params.window_type = (TFFTWinType)intval;

	intval = g_key_file_get_integer( kf, "FFT", "WindowSize2Power", NULL);
	if ( intval < 0 || 2 << intval > pagesize ) {
		g_string_append_printf( ext_msg, "WindowSize is less than 0 or exceeds PageSize.\n");
		fft_params.window_size_2power = 2;
	} else
		fft_params.window_size_2power = intval;

	dblval = g_key_file_get_double( kf, "FFT", "BinSize", NULL);
	if ( dblval < 1. / (2 << fft_params.window_size_2power) ||
	     dblval * (2 << fft_params.window_size_2power) != (gint) (dblval * (2 << fft_params.window_size_2power)) ) {
		g_string_append_printf( ext_msg, "BinSize must be a multiple of, and greater than, 1/WindowSize.\n");
		fft_params.bin_size = 1;
	} else
		fft_params.bin_size = dblval;

	intval = g_key_file_get_integer( kf, "Artifacts", "SmoothOver", NULL);
	if ( intval < 1 || intval > 11 || (intval-1)/2*2 != (intval-1) ) {
		g_string_append_printf( ext_msg, "SmoothOver must be an odd integer >1 and <11.\n");
		agh_set_smoothover( 1);
	} else
		agh_set_smoothover( intval);

	dblval = g_key_file_get_double( kf, "Artifacts", "GlitchMagnitude", NULL);
	if ( dblval <= 0 ) {
		g_string_append_printf( ext_msg, "GlitchMagnitude must be >0.\n");
		agh_set_glitch_mag( 1.);
	} else
		agh_set_glitch_mag( dblval);

	intval = g_key_file_get_integer( kf, "View", "PixelsPeruV2", NULL);
	if ( intval < 1 ) {
		g_string_append_printf( ext_msg, "Bad value for PixelsPeruV2.\n");
		agh_set_ppuV2( 100);
	} else
		agh_set_ppuV2( intval);

	dblval = g_key_file_get_double( kf, "Simulations", "OperatingRangeFrom", NULL);
	if ( dblval <= 0 ) {
		g_string_append_printf( ext_msg, "OperatingRangeFrom must be >0.\n");
		agh_sim_set_operating_range_from( 2.);
	} else
		agh_sim_set_operating_range_from( dblval);
	dblval = g_key_file_get_double( kf, "Simulations", "OperatingRangeUpto", NULL);
	if ( dblval <= agh_sim_get_operating_range_from() ) {
		g_string_append_printf( ext_msg, "OperatingRangeUpto must be > OperatingRangeFrom.\n");
		agh_sim_set_operating_range_upto( 3.);
	} else
		agh_sim_set_operating_range_upto( dblval);




	key_value_list = g_key_file_get_string_list( kf, "Design", "Sources", NULL, NULL);
	i = 0;
	if ( key_value_list ) {
		while ( p = key_value_list[i++] )
			add_source( new CEDFFile( p), AGH_SRCATTACH_LEAVE);
		g_strfreev( key_value_list);
	}


	agh_current_session = g_key_file_get_integer( kf, "Design", "CurrentSession", NULL);
	agh_current_channel = g_key_file_get_integer( kf, "Design", "CurrentChannel", NULL);


	if ( g_key_file_has_group( kf, "Batch Run") ) {
		agh_sim_runbatch_include_all_channels	= g_key_file_get_boolean( kf, "Batch Run", "IncludeAllChannels",  NULL);
		agh_sim_runbatch_include_all_sessions	= g_key_file_get_boolean( kf, "Batch Run", "IncludeAllSessions",  NULL);
		agh_sim_runbatch_iterate_ranges		= g_key_file_get_boolean( kf, "Batch Run", "IterateRanges",  NULL);
		agh_sim_runbatch_redo_option		= g_key_file_get_integer( kf, "Batch Run", "RedoOption",  NULL);
	}

	if ( g_key_file_has_group( kf, "Control parameters") ) {
		control_params.InitRand               = g_key_file_get_double ( kf, "Control parameters", "InitRand", NULL);
		control_params.MinSuffFitImprov       = g_key_file_get_double ( kf, "Control parameters", "MinSuffFitImprov", NULL);
		control_params.MaxAllowedParamDrift   = g_key_file_get_integer( kf, "Control parameters", "MaxAllowedParamDrift", NULL);
		control_params.StrideLength           = g_key_file_get_integer( kf, "Control parameters", "StrideLength", NULL);
		control_params.MaxStrides             = g_key_file_get_integer( kf, "Control parameters", "MaxStrides", NULL);
		control_params.RetriesoOnSimCycleFail = g_key_file_get_integer( kf, "Control parameters", "RetriesoOnSimCycleFail", NULL);
		control_params.TeleportDistance	      = g_key_file_get_integer( kf, "Control parameters", "TeleportDistance", NULL);
		control_params.MaxHalvingTimes	      = g_key_file_get_integer( kf, "Control parameters", "MaxHalvingTimes", NULL);
		control_params.DBAmendment1           = g_key_file_get_boolean( kf, "Control parameters", "DBAmendment1", NULL);
		control_params.DBAmendment2           = g_key_file_get_boolean( kf, "Control parameters", "DBAmendment2", NULL);
		control_params.AZAmendment            = g_key_file_get_boolean( kf, "Control parameters", "AZAmendment", NULL);
		control_params.ScoreMVTAsWake         = g_key_file_get_boolean( kf, "Control parameters", "ScoreMVTAsWake", NULL);
		control_params.ScoreUnscoredAsWake    = g_key_file_get_boolean( kf, "Control parameters", "ScoreUnscoredAsWake", NULL);

		if ( !control_params.check_sane() ) {
			g_string_append_printf( ext_msg,
						"Failed to read all control parameters, or some values were missing or "
						"those supplied invalid. Sane values assigned to all.\n");
			control_params.assign_defaults();
		}
	}


	if ( !g_key_file_has_group( kf, "Tunables") )
		g_string_append_printf( ext_msg, "No [Tunables] group found.\n");
	else {
		for ( guint t = 0; t < AGH_VNUM; t++ ) {
			gdouble *x;
			gsize n;
			x = g_key_file_get_double_list( kf, "Tunables", agh_v_names[t], &n, NULL);
			if ( n == 5 ) {
				tunables.value.pidx[t]	= x[0];
				tunables.min.pidx[t]	= x[1];
				tunables.max.pidx[t]	= x[2];
				tunables.step.pidx[t]	= x[3];
				tunables.flags[t]	= ((gboolean)x[4] ?AGH_T_REQUIRED :0);
			} else // no matter which assignment failed
				g_string_append_printf( ext_msg,
							"Failed to read all fields for tunable %s.\n",
							agh_v_names[t]);
			g_free( x);
		}
		if ( !tunables.check_consisitent() ) {
			g_string_append_printf( ext_msg,
						"Tunable definitions inconsistent; defaults will be assigned.\n");
			tunables.assign_defaults();
		}
	}



	if ( ext_msg->len )
		agh_pop_ok_message( ext_msg->str);

	g_string_free( ext_msg, TRUE);
	g_key_file_free( kf);
	if ( kf_error)
		g_error_free( kf_error);

//	agh_dump_all();

	return 0;
}






gint
CExpDesign::save()
{
	GKeyFile *kf = g_key_file_new();

	GString *agg = g_string_sized_new( 200);
	guint i;
	gchar **key_list;

	g_string_assign( agg, "");
	if ( sessions->n() ) {
		for ( i = 0; i < sessions->n(); i++ )
			g_string_append_printf( agg, "%s;", sessions->nth(i)->name());
		key_list = g_strsplit( agg->str, ";", -1);
		g_key_file_set_string_list( kf, "Design", "Sessions", key_list, sessions->n());
		g_strfreev( key_list);
	}

	g_string_assign( agg, "");
	if ( channel_names->n() ) {
		for ( i = 0; i < channel_names->n(); i++ )
			g_string_append_printf( agg, "%s;", channel_names->o_by_seq_no(i)->name());
		key_list = g_strsplit( agg->str, ";", -1);
		g_key_file_set_string_list( kf, "Design", "Channels", key_list, channel_names->n());
		g_strfreev( key_list);
	}

	g_string_assign( agg, "");
	if ( episode_names->n() ) {
		for ( i = 0; i < episode_names->n(); i++ )
			g_string_append_printf( agg, "%s;", episode_names->o_by_seq_no(i)->name());
		key_list = g_strsplit( agg->str, ";", -1);
		g_key_file_set_string_list( kf, "Design", "Episodes", key_list, episode_names->n());
		g_strfreev( key_list);
	}

	g_string_assign( agg, "");
	if ( groups->n() ) {
		for ( i = 0; i < groups->n(); i++ )
			g_string_append_printf( agg, "%s;", groups->nth(i)->name());
		key_list = g_strsplit( agg->str, ";", -1);
		g_key_file_set_string_list( kf, "Design", "Groups", key_list, groups->n());
		g_strfreev( key_list);
	}


	g_string_assign( agg, "");
	if ( sources->n() ) {
		for ( i = 0; i < sources->n(); i++ )
			if ( sources->nth(i)->attach_mode == AGH_SRCATTACH_LEAVE )
				g_string_append_printf( agg, "%s;", sources->nth(i)->filename());
		key_list = g_strsplit( agg->str, ";", -1);
		g_key_file_set_string_list( kf, "Design", "Sources", key_list, sources->n());
		g_strfreev( key_list);
	}

	GString *kg = g_string_sized_new( 80);
	for ( i = 0; i < subjects->n(); i++ ) {
		CSubject *J = subjects->nth(i);
		g_string_printf( kg, "Subject %s", J->name());
		gchar gender_string[2] = { (gchar)J->gender(), 0 };
		g_key_file_set_integer( kf, kg->str, "age", J->age());
		g_key_file_set_value( kf, kg->str, "gender", gender_string);
		g_key_file_set_value( kf, kg->str, "group", J->belongs->name());
		g_key_file_set_string( kf, kg->str, "comment", J->comment());
	}


	g_key_file_set_integer( kf, "Design", "CurrentSession", agh_current_session);
	g_key_file_set_integer( kf, "Design", "CurrentChannel", agh_current_channel);

	g_key_file_set_boolean( kf, "Batch Run", "IncludeAllChannels",	agh_sim_runbatch_include_all_channels);
	g_key_file_set_boolean( kf, "Batch Run", "IncludeAllSessions",	agh_sim_runbatch_include_all_sessions);
	g_key_file_set_boolean( kf, "Batch Run", "IterateRanges",	agh_sim_runbatch_iterate_ranges);
	g_key_file_set_integer( kf, "Batch Run", "RedoOption",		agh_sim_runbatch_redo_option);

	g_key_file_set_double ( kf, "Control parameters", "InitRand",			control_params.InitRand);
	g_key_file_set_double ( kf, "Control parameters", "MinSuffFitImprov",		control_params.MinSuffFitImprov);
	g_key_file_set_integer( kf, "Control parameters", "MaxAllowedParamDrift",	control_params.MaxAllowedParamDrift);
	g_key_file_set_integer( kf, "Control parameters", "StrideLength",		control_params.StrideLength);
	g_key_file_set_integer( kf, "Control parameters", "MaxStrides",			control_params.MaxStrides);
	g_key_file_set_integer( kf, "Control parameters", "RetriesoOnSimCycleFail",	control_params.RetriesoOnSimCycleFail);
	g_key_file_set_integer( kf, "Control parameters", "TeleportDistance",		control_params.TeleportDistance);
	g_key_file_set_integer( kf, "Control parameters", "MaxHalvingTimes",		control_params.MaxHalvingTimes);
	g_key_file_set_boolean( kf, "Control parameters", "DBAmendment1",		control_params.DBAmendment1);
	g_key_file_set_boolean( kf, "Control parameters", "DBAmendment2",		control_params.DBAmendment2);
	g_key_file_set_boolean( kf, "Control parameters", "AZAmendment",		control_params.AZAmendment);
	g_key_file_set_boolean( kf, "Control parameters", "ScoreMVTAsWake",		control_params.ScoreMVTAsWake);
	g_key_file_set_boolean( kf, "Control parameters", "ScoreUnscoredAsWake",	control_params.ScoreUnscoredAsWake);


	for ( guint t = 0; t < AGH_VNUM; t++ ) {
		gdouble x[5] = { tunables.value.pidx[t],
				 tunables.min.pidx[t],
				 tunables.max.pidx[t],
				 tunables.step.pidx[t],
		       (gdouble)(tunables.flags[t] & AGH_T_REQUIRED) };
		g_key_file_set_double_list( kf, "Tunables", agh_v_names[t], x, 5);
	}


	g_key_file_set_integer( kf, "PageSize", "Value", pagesize);

	g_key_file_set_integer( kf, "FFT", "WindowSize2Power", fft_params.window_size_2power);
	g_key_file_set_integer( kf, "FFT", "WindowType",       fft_params.window_type);
	g_key_file_set_double(  kf, "FFT", "BinSize",          fft_params.bin_size);

	g_key_file_set_integer( kf, "Artifacts", "SmoothOver", agh_get_smoothover());
	g_key_file_set_double(  kf, "Artifacts", "GlitchMagnitude", agh_get_glitch_mag());
	g_key_file_set_integer( kf, "View", "PixelsPeruV2", agh_get_ppuV2());

	g_key_file_set_double(  kf, "Simulations", "OperatingRangeFrom", agh_sim_get_operating_range_from());
	g_key_file_set_double(  kf, "Simulations", "OperatingRangeUpto", agh_sim_get_operating_range_upto());


	g_string_free( kg, TRUE);

	g_string_free( agg, TRUE);

	gchar *towrite = g_key_file_to_data( kf, NULL, NULL);
	g_file_set_contents( "ExpDesign", towrite, -1, NULL);
	g_free( towrite);

	g_key_file_free( kf);

	return 0;
}

















const gchar*
CExpDesign::make_fname_edf( CSubject *J,
			    CRefList<CMeasurement> *D,
			    COrderedName *E)
{
	static GString *fname = NULL;
	if ( !fname )
		fname = g_string_sized_new( 120);
	g_string_printf( fname, "%s/%s/%s/%s.edf", J->belongs->name(), J->name(), D->name(), E->name());
	return fname->str;
}


const gchar*
CExpDesign::make_fname_power( CSubject *J,
			      CRefList<CMeasurement> *D,
			      COrderedName *E,
			      COrderedName *H)
{
	static GString *fname = NULL;
	if ( !fname )
		fname = g_string_sized_new( 180);
	g_string_printf( fname, "%s/%s/%s/%s/H:%s G:%d B:%g W:%c%d.power",
			 J->belongs->name(), J->name(), D->name(), E->name(),
			 H->name(),
			 pagesize, fft_params.bin_size,
			 'a' + fft_params.window_type, 2 << fft_params.window_size_2power);
	return fname->str;
}



const gchar*
CExpDesign::make_fname_simulation( CSubject *J,
				   CRefList<CMeasurement> *D,
				   COrderedName *H,
				   gfloat from, gfloat upto)
{
	static GString *fname = NULL;
	if ( !fname )
		fname = g_string_sized_new( 180);

	g_string_printf( fname,
			 "%s/%s/%s/SIMULATIONS/"
			 "B:%g W:%c%d H:%s F:%g-%g G:%d DB1:%s DB2:%s AZ:%s "
			 "R:%s%s%s%s%s%s%s%s%s.S",
			 J->belongs->name(), J->name(), D->name(),
			 fft_params.bin_size,
			 'a' + fft_params.window_type, 2 << fft_params.window_size_2power,
			 H->name(),
			 from, upto, pagesize,
			 yesno( control_params.DBAmendment1),
			 yesno( control_params.DBAmendment2),
			 yesno( control_params.AZAmendment),
			 (tunables.step.pidx[AGH_Vgc  ] > 0.)?"g" :"",
			 (tunables.step.pidx[AGH_Vrs  ] > 0.)?"r" :"",
			 (tunables.step.pidx[AGH_Vrc  ] > 0.)?"c" :"",
			 (tunables.step.pidx[AGH_Vfc_R] > 0.)?"R" :"",
			 (tunables.step.pidx[AGH_Vfc_W] > 0.)?"W" :"",
			 (tunables.step.pidx[AGH_VS_0 ] > 0.)?"0" :"",
			 (tunables.step.pidx[AGH_VS_U ] > 0.)?"U" :"",
			 (tunables.step.pidx[AGH_Vt_a ] > 0.)?"a" :"",
			 (tunables.step.pidx[AGH_Vt_p ] > 0.)?"p" :"");
	return fname->str;
}







void
CExpDesign::collect_msmts_from_tree()
{
	CMeasurement *My = new CMeasurement;

	struct stat f_stat;

	for ( guint g = 0; g < groups->n(); g++ ) {
		CRefList<CSubject> *G = groups->nth(g);

		for ( guint j = 0; j < G->n(); j++ ) {
			CSubject *J = G->nth(j);

			for ( guint d = 0; d < sessions->n(); d++ ) {
				CRefList<CMeasurement> *D = sessions->nth(d);

				for ( guint e = 0; e < episode_names->n(); e++) {
					COrderedName *E = episode_names->o_by_seq_no(e);

					CEDFFile *F = NULL;
					const gchar *edf_fname = make_fname_edf( J, D, E);
					if ( stat( edf_fname, &f_stat) == 0 )
						add_source( F = new CEDFFile( edf_fname),
							    AGH_SRCATTACH_MOVE);

					for ( guint h = 0; h < channel_names->n(); h++ ) {
						COrderedName *H = channel_names->o_by_seq_no(h);

						My->make_phony( E, J, H, D);
						CMeasurement *M = measurements->exist( My);
						// at this point, add_source will have created some
						// measurements for channels whose signals were contained
						// in the edf that was found; here, we further look for
						// power files, which may well occur with their
						// sources lost; also the other way round

						gint	pow_exist = (stat( make_fname_power( J, D, E, H), &f_stat) == 0),
							edf_has_channel = F && (F->sig_no_by_channel_name( H->name()) != -1);

						if ( edf_has_channel || pow_exist ) {
							if ( !M )
								measurements->add( M = new CMeasurement( E, J, H, D,
													 edf_has_channel ?F :NULL,
													 pagesize, fft_params));
							if ( pow_exist ) M->status |=  AGH_BP_OBTAINED;
						} else
							if ( M )
								M->status &= ~AGH_BP_OBTAINED;
					}
				}
			}
		}
	}

	delete My;
}





void
CExpDesign::collect_simulations_from_tree()
{
	struct stat f_stat;

	for ( guint g = 0; g < groups->n(); g++ ) {
		CRefList<CSubject> *G = groups->nth(g);

		for ( guint j = 0; j < G->n(); j++ ) {
			CSubject *J = G->nth(j);

			for ( guint d = 0; d < sessions->n(); d++ ) {
				CRefList<CMeasurement> *D = sessions->nth(d);

				for ( guint h = 0; h < channel_names->n(); h++ ) {
					COrderedName *H = channel_names->o_by_seq_no(h);

					gint sim_exist = (stat( make_fname_simulation( J, D, H,
										       agh_sim_operating_range_from,
										       agh_sim_operating_range_upto),
								&f_stat) == 0);
					if ( sim_exist )
						setup_modrun( J, D, H,
								agh_sim_operating_range_from,
								agh_sim_operating_range_upto, NULL);

				}
			}
		}
	}
}


// EOF
