// ;-*-C++-*- *  Time-stamp: "2008-06-22 15:49:24 hmmr"

/***************************************************************************
 *            agh_expdesign.h
 *
 *  Copyright  2008  Andrei Zavada
 *  Email johnhommer@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _AGH_EXPDESIGN_H
#define _AGH_EXPDESIGN_H


#if HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glib.h>
#include <string.h>
#include <stdio.h>

#include "z_lists.hh"

#include "agh_enums.h"
#include "agh_model.hh"   // contains base classes for CMeasurement
#include "agh_iface.hh"  // for various agh_* functions exposing
			 // controls to C ui yet needing to touch
			 // private fields of core, which happen to be
			 // declared extern "C"





// Two classes, CSubject and CMeasurement, are primary objects.  These
// are dynamically collected for treeviews, grouped by Group per
// Session.
//
// CSubject objects reference a group to which they uniquely belong,
// and in turn, are referenced from CGroup objects.  We take care to
// unlink references from CGroup when destroying a Subject, namely
// when deleting its reference from a CSubjectList.
//
// Likewise, CSubject objects are orpaned on deletion of CGroup

// The same goes for Measurement-Session relationship


class CExpDesign;
class CSubject {

    friend class CList<CSubject>;
    friend class CRefList<CSubject>;
    friend class CExpDesign;
    friend gint agh_rename_subject( guint, guint, const gchar*);

    private:
	GString *_name;
	gint     _age;
	TGender	 _gender;
	GString *_comment;

    public:
	CRefList<CSubject> *belongs;
	int status;
      // no-arg ctor will be used to create dummies, then to be fed to 'exist' functions,
      // to avoid them having side-effects like checking in/out
	CSubject()	    { belongs = NULL, status = 0;
			      _name = g_string_new( "<anon>");
			      _age = -1;
			      _gender = AGH_SUBJECT_GENDER_NEUTER;
			      _comment = g_string_new( "");	       }
	CSubject( const gchar *iname,
		  TGender igender,  gint iage,
		  CRefList<CSubject> *to,
		  const gchar *icomment = NULL);
       ~CSubject();

	void make_phony( const gchar *ph_name)      { g_string_assign( _name, ph_name);  }
	virtual gboolean operator == ( CSubject &o) { return !strcmp( name(), o.name()); }
	void detach()  { belongs->checkout( this);  belongs = NULL; }

	gchar  *name()              { return _name->str; };
	gint    age()               { return _age; }
	TGender gender()            { return _gender; }
	gchar  *comment()           { return _comment->str; }
	CRefList<CSubject> *group() { return  belongs; }

	void rename( const gchar *new_name)	{ g_string_assign( _name, new_name);   }
	void set_age( gint new_age)		{ _age = new_age;                      }
	void set_gender( TGender new_gender)	{ _gender = new_gender;                }
	void set_comment( const gchar *new_cmt)	{ g_string_assign( _comment, new_cmt); }

      // can be NULL to detach from an existing group
//	void join( CRefList<CSubject> *to);
      // appear superfluous: objects are not to switch groups

	void dump()    { printf( "name:%s\tgender:%c\tage:%d\tin:%s\n", _name->str, _gender, _age, belongs->name()); }
};







class CMeasurement
    : virtual public CBinnedPower {

    friend class CList<CMeasurement>;
    friend class CRefList<CMeasurement>;
    friend class CExpDesign;
    friend class CSCourse;
    friend gint agh_populate_mDesignMeasurements();
    friend gint agh_msmt_has_edf( const gchar*, const gchar*, const gchar*, const gchar*);
    friend gint agh_refresh_msmts_from_tree();
    friend gint agh_add_or_replace_edf( const gchar*);
    friend gint agh_get_signal_vector( const gchar*, const gchar*, const gchar*,
				       GArray**, guint*, gdouble*);
    private:
      // identification
	COrderedName  *_episode,
		      *_channel;
	CSubject *_owner;

      // the source of EEG data
	CEDFFile *_source;

	GString	*_basename;

    public:
	CRefList<CMeasurement> *belongs;
//	int status; // use CBinnedPower::status

	CMeasurement()
	  : CBinnedPower()  { belongs = NULL, status = Z_OBJECT_PHONY;
			      _episode = _channel = NULL,  _owner = NULL;
			      _source = NULL;
			      _basename = NULL;                  }
	CMeasurement( COrderedName *episode,
		      CSubject *owner,
		      COrderedName *channel,
		      CRefList<CMeasurement> *session,
		      CEDFFile *source,
		      guint pagesize,
		      SFFTParamsSet &fft_params);
       ~CMeasurement();

	gboolean operator == ( CMeasurement &o)  { return _episode == o._episode &&
							  _owner   == o._owner   &&
							  _channel == o._channel &&
							  belongs  == o.belongs;    }
	void make_phony( COrderedName *iepisode,
			 CSubject *iowner,
			 COrderedName *ichannel,
			 CRefList<CMeasurement> *to)  {	belongs = to, status = Z_OBJECT_PHONY;
							_episode = iepisode, _owner = iowner, _channel = ichannel; }
	void detach()  { belongs->checkout( this);  belongs = NULL; }

	const gchar *episode_name()  { return _episode->name();     }
	guint      episode_seq_no()  { return _episode->seq_no();   }
	const gchar *channel_name()  { return _channel->name();     }
	guint      channel_seq_no()  { return _channel->seq_no();   }
	CSubject           *owner()  { return _owner;               }
	CEDFFile          *source()  { return _source;              }
	gchar           *basename()  { return _basename->str;       }

	gint try_attach_edf( CEDFFile*);

	const gchar *make_fname();
	const gchar *make_fname_track();

      // this methods belong here because of the load hook, which needs
      // to know subject, and other particulars to construct a proper filename;
      //
      // note: pagesize and fft_params are matched against those stored in CBinnedPower fields,
      // and will just load the power if matching, or further call obtain_power otherwise
      // (and save then)
	gdouble ship_power_course( GArray *power, GArray *track,
				   gint ipagesize, SFFTParamsSet&,
				   gfloat from, gfloat upto,
				   gint smooth, gfloat glitch_mag,
				   gboolean filtered);

	void dump();
};







template<class> class CAggregatorList;
class CAggregatorList<CSubject>;
class CAggregatorList<CMeasurement>;







#define AGH_SESSION_STATE_OK       0
#define AGH_SESSION_STATE_INITFAIL 1

class CExpDesign {
	GString *_session_dir;
	gint state;

    public:
	bool is_ok()  { return !state; }
	GString *name;

	CList<CSubject>       *subjects;
	CList<CMeasurement>   *measurements;
	CList<CEDFFile>       *sources;
	CList<CSimulation>    *simulations;
	CAggregatorList< CRefList<CSubject> >     *groups;
	CAggregatorList< CRefList<CMeasurement> > *sessions;

	CList<COrderedName>	*channel_names;
	CList<COrderedName>	*episode_names;

	guint		pagesize;
	SFFTParamsSet	fft_params;

	STunableSetFull	 tunables;
	SControlParamSet control_params;

	CExpDesign( const gchar *sessiondir);
       ~CExpDesign();

	gint load();
	gint save();

      // convenience access functions
	CSubject *J_by_name( const gchar *j_name)            { CSubject *Jy = new CSubject();
							       Jy->make_phony( j_name);
							       CSubject *J = subjects->exist( Jy);
							       delete Jy;
							       return J;                           }
	CMeasurement *M_by_EJHD( COrderedName           *E,
				 CSubject               *J,
				 COrderedName           *H,
				 CRefList<CMeasurement> *D)  { CMeasurement *My = new CMeasurement();
							       My->make_phony( E, J, H, D);
							       CMeasurement *M = measurements->exist( My);
							       delete My;
							       return M;                                          }

      // various member functions providing for nontrivial post-removal cleanup
	gint add_source( CEDFFile*, TSourceAttachMode);
	void delete_source( CEDFFile*);
	void delete_subject( CSubject*);
	void delete_group( CRefList<CSubject>*);

	gint setup_modrun( CSubject*, CRefList<CMeasurement>*, COrderedName*,
			   gfloat, gfloat,
			   CSimulation**);
	void reset_modrun( CModelRun*);

	const gchar *make_fname_edf  ( CSubject*, CRefList<CMeasurement>*, COrderedName*);
	const gchar *make_fname_power( CSubject*, CRefList<CMeasurement>*, COrderedName*, COrderedName*);
	const gchar *make_fname_simulation( CSubject *J,
					    CRefList<CMeasurement> *D,
					    COrderedName *H,
					    gfloat from, gfloat upto);

	void collect_msmts_from_tree();
	void collect_simulations_from_tree();
};



extern CExpDesign *AghCC;


#endif

// EOF
