// ;-*-C++-*- *  Time-stamp: "2008-07-01 22:46:32 hmmr"

/***************************************************************************
 *            agh_iface-expdesign.cc
 *
 *  Copyright  2008  Andrei Zavada
 *  Email johnhommer@gmail.com

 *  Parts of expdesign/data having ui controls.  We populate the
 *  GtkTreeStore objects, both those populated once at startup and
 *  other ones that change on every redisplay.
 *
 *  Subsequently, other functions defined here manage synchronisation of treemodels
 *  with the core structures.  Callbacks in agh-ui are simply wrappers for these.
 *
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <sys/stat.h>
#include <gtk/gtk.h>
#include "agh_iface.hh"
#include "agh_io.hh"
#include "agh_expdesign.hh"






// all the following functions can't be made members of CExpDesigns
// since they are called from callbacks, which are plain C

gint
agh_add_subject_to_group( guint g, const gchar *name)
{
	gint retval = AghCC->subjects->add( new CSubject( name,
							  AGH_SUBJECT_GENDER_MALE,
							  22,
							  AghCC->groups->nth(g)));
	return retval;
}

gint
agh_add_group( const gchar *g)
{
	return AghCC->groups->add( g);
}

void
agh_delete_group_with_subjects( guint g)
{
	AghCC->delete_group( AghCC->groups->nth( g));
}

void
agh_delete_subject( guint g, guint j)
{
	AghCC->delete_subject( AghCC->groups->nth(g)->nth(j));
}

gint
agh_rename_group( guint g, gchar *new_name)
{
	g_strcanon( g_strstrip( new_name), "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890_-.,()$~!", '-');
	if ( !AghCC->groups->o_by_name( new_name) ) {
		AghCC->groups->nth(g)->rename( new_name);
		return 1;
	}
	return 0;
}

gint
agh_rename_subject( guint g, guint j, gchar *new_name)
{
	gint retval = 0;

	g_strcanon( g_strstrip( new_name), "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890_-.,()$~!", '-');
	CSubject *Jy = new CSubject;
	Jy->make_phony( new_name);

	if ( !AghCC->subjects->exist( Jy) ) {
		AghCC->groups->nth(g)->nth(j)->rename( new_name);
		retval = 1;
	}
	delete Jy;
	return retval;
}

void
agh_set_subject_age( guint g, guint j, guint new_age)
{
	AghCC->groups->nth(g)->nth(j)->set_age( new_age);
}

void
agh_switch_subject_gender( guint g, guint j)
{
	CSubject *J = AghCC->groups->nth(g)->nth(j);
	TGender x = J->gender();
	J->set_gender( (x == AGH_SUBJECT_GENDER_MALE)
		       ?AGH_SUBJECT_GENDER_FEMALE
		       :AGH_SUBJECT_GENDER_MALE);
}

void
agh_set_subject_comment( guint g, guint j, const gchar *new_comment)
{
	AghCC->groups->nth(g)->nth(j)->set_comment( new_comment);
}



const gchar*
agh_get_subject_group( const gchar *j_name)
{
	return AghCC->J_by_name( j_name)->group()->name();
}


gint
agh_get_n_groups()
{
	return AghCC->groups->n();
}

gint
agh_get_n_subjects_in_group( gint g)
{
	return AghCC->groups->nth(g)->n();
}

const gchar*
agh_get_nth_subject_name_in_group( gint g, gint n)
{
	return AghCC->groups->nth(g)->nth(n)->name();
}




const gchar*
agh_get_current_session_name()
{
	return AghCC->sessions->nth(agh_current_session)->name();
}

const gchar*
agh_get_nth_session_name( gint n)
{
	return AghCC->sessions->nth(n)->name();
}


gint
agh_add_session( const gchar *name)
{
	return AghCC->sessions->add( name);
}

void
agh_delete_session_at( guint at)
{
	AghCC->sessions->del_nth(at);
	AghCC->measurements->clean_unowned();
}

gint
agh_rename_session_at( const gchar *new_name, guint at)
{
	AghCC->sessions->nth(at)->rename( new_name);
	return 1;
}


gint
agh_get_n_sessions()
{
	return AghCC->sessions->n();
}







gint
agh_append_episodename( const gchar *name)
{
	COrderedName *o = new COrderedName( name);
	return AghCC->episode_names->add( o);
}

gint
agh_insert_episodename_at( const gchar *name, gint at)
{
	COrderedName *o = new COrderedName( name);
	return (at == G_MAXINT) ?AghCC->episode_names->add( o)
				:AghCC->episode_names->ins( o, at);
}

void
agh_delete_episodename_at( guint at)
{
	AghCC->episode_names->del( AghCC->episode_names->o_by_seq_no( at));
}

gint
agh_rename_episodename_at( guint at, const gchar *new_name)
{
	gint retval;
	COrderedName *o = new COrderedName( new_name);
	if ( !AghCC->episode_names->exist( o) ) {
		AghCC->episode_names->o_by_seq_no(at)->rename( new_name);
		retval = 1;
	} else
		retval = 0;
	delete o;
	return retval;
}

gint
agh_move_episodename_up( guint at)
{
	if ( at > 0 ) {
		AghCC->episode_names->swap( at, at-1);
		return 1;
	} else
		return 0;
}

gint
agh_move_episodename_down( guint at)
{
	if ( at < AghCC->episode_names->n() - 1 ) {
		AghCC->episode_names->swap( at, at+1);
		return 1;
	} else
		return 0;
}

const gchar*
agh_get_nth_episode_name( gint n)
{
	return AghCC->episode_names->o_by_seq_no(n)->name();
}






const gchar*
agh_get_current_channel_name()
{
	return AghCC->channel_names->o_by_seq_no(agh_current_channel)->name();
}

const gchar*
agh_get_nth_channel_name( gint n)
{
	return AghCC->channel_names->o_by_seq_no(n)->name();
}


gint
agh_append_channelname( const gchar *name)
{
	COrderedName *o = new COrderedName( name);
	return AghCC->channel_names->add( o);
}

gint
agh_insert_channelname_at( const gchar *name, guint at)
{
	COrderedName *o = new COrderedName( name);
	if ( at <= agh_current_channel )
		agh_current_channel++;
	return (at == G_MAXINT) ?AghCC->channel_names->add( o)
				 :AghCC->channel_names->ins( o, at);
}

void
agh_delete_channelname_at( guint at)
{
	AghCC->channel_names->del( AghCC->channel_names->o_by_seq_no( at));
	if ( at < agh_current_channel )
		agh_current_channel--;
	else if ( at == agh_current_channel )
		agh_current_channel = 0;
}

gint
agh_rename_channelname_at( guint at, const gchar *new_name)
{
	gint retval;
	COrderedName *o = new COrderedName( new_name);
	if ( !AghCC->channel_names->exist( o) ) {
		AghCC->channel_names->o_by_seq_no(at)->rename( new_name);
		retval = 1;
	} else
		retval = 0;
	delete o;
	return retval;
}

gint
agh_move_channelname_up( guint at)
{
	if ( at > 0 ) {
		AghCC->channel_names->swap( at, at-1);
		if ( at == agh_current_channel )
			agh_current_channel--;
		return 1;
	} else
		return 0;
}

gint
agh_move_channelname_down( guint at)
{
	if ( at < AghCC->channel_names->n() - 1 ) {
		AghCC->channel_names->swap( at, at+1);
		if ( at == agh_current_channel )
			agh_current_channel++;
		return 1;
	} else
		return 0;
}

gint
agh_get_channel_name_list( GPtrArray *list)
{
	g_ptr_array_set_size( list, AghCC->channel_names->n());
	guint i;
	for ( i = 0; i < AghCC->channel_names->n(); i++ )
		g_ptr_array_index (list, i) = (void*) AghCC->channel_names->o_by_seq_no(i)->name();
	return i;
}

gint
agh_get_n_channels()
{
	return AghCC->channel_names->n();
}





// EOF
