// ;-*-C++-*- *  Time-stamp: "2008-07-02 03:44:02 hmmr"

/***************************************************************************
 *            agh_iface-msmt.cc
 *
 *  Copyright  2008  Andrei Zavada
 *  Email johnhommer@gmail.com

 *  Parts of expdesign/data having ui controls.  We populate the
 *  GtkTreeStore objects, both those populated once at startup and
 *  other ones that change on every redisplay.
 *
 *  Subsequently, other functions defined here manage synchronisation of treemodels
 *  with the core structures.  Callbacks in agh-ui are simply wrappers for these.
 *
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <gtk/gtk.h>

#include "agh_iface.hh"
#include "agh_io.hh"
#include "agh_expdesign.hh"
#include "agh_model.hh"




guint	agh_pagesize_values[] = { 5, 10, 15, 20, 30, 60 },
	agh_pagesize_item = 4;

const guint agh_pagesize_total_items = 6;


guint
agh_pagesize_get_current()
{
	return AghCC->pagesize;
}


guint
agh_pagesize_set_current_by_item( guint item)
{
	if ( item < agh_pagesize_total_items )
		agh_pagesize_item = item;
	return AghCC->pagesize = agh_pagesize_values[agh_pagesize_item];
}

guint
agh_pagesize_set_current( guint psize)
{
	for ( guint i = 0; i < agh_pagesize_total_items; i++ ) {
		if ( psize == agh_pagesize_values[i] )
			return AghCC->pagesize = agh_pagesize_values[agh_pagesize_item = i];
	}
	return AghCC->pagesize;
}





guint agh_ppuV2 = 10;
guint agh_smoothover = 1;
gfloat agh_glitch_mag = 1;



const gchar*
agh_edf_get_subject( const gchar *filename)
{
	static gchar subject[81];

	CEDFFile *Fy = new CEDFFile();
	Fy->make_phony( filename);
	strncpy( subject, Fy->PatientID_raw, 80);

	delete Fy;

	return subject;
}




gint
agh_edf_add_or_replace( const gchar *f_name, TSourceAttachMode option)
{
	CEDFFile *F = new CEDFFile( f_name);
	if ( !F->Session || !F->Episode || !F->NSignals ) {
		delete F;
		return -2; // barf louder
	}

	return AghCC->add_source( F, option);
}



void
agh_edf_delete( const gchar *fname)
{
	CEDFFile *Fy = new CEDFFile();
	Fy->make_phony( fname);
	AghCC->delete_source( AghCC->sources->exist(Fy));
	delete Fy;
}



void
agh_edf_get_stats( const gchar *fname, GString *out)
{
	CEDFFile *Fy = new CEDFFile();
	Fy->make_phony( fname);
	Fy->dump( out);
	delete Fy;
}


gint
agh_edf_is_good( const gchar *filename, GString *out)
{
	CEDFFile *Fy = new CEDFFile();
	Fy->make_phony( filename);

	g_string_assign( out, "");
	if ( Fy->status & AGH_EDFCHK_BAD_HEADER )
		g_string_append_printf( out, "* Ill-formed header\n");
	if ( Fy->status & AGH_EDFCHK_BAD_VERSION )
		g_string_append_printf( out, "* Bad Version signature (i.e., not an EDF file)\n");
	if ( Fy->status & AGH_EDFCHK_BAD_NUMFLD )
		g_string_append_printf( out, "* Garbage in numerical fields\n");
	if ( Fy->status & AGH_EDFCHK_DATE_UNPARSABLE )
		g_string_append_printf( out, "* Date field ill-formed\n");
	if ( Fy->status & AGH_EDFCHK_TIME_UNPARSABLE )
		g_string_append_printf( out, "* Time field ill-formed\n");
	if ( Fy->status & AGH_EDFCHK_NOSESSION )
		g_string_append_printf( out, "* No session information in field RecordingID "
					     "(expecting this to appear after "
					     "episode designation followed by a comma)\n");
	if ( Fy->status & AGH_EDFCHK_NOCHANNEL )
		g_string_append_printf( out, "* Channel not specified (after SignalType)\n");
	if ( Fy->status & AGH_EDFCHK_NON1020_CHANNEL )
		g_string_append_printf( out, "* Channel designation not following 10-20 system\n");
	if ( Fy->status & AGH_EDFCHK_NONKEMP_SIGNALTYPE )
		g_string_append_printf( out, "* Signal type not approved by Kemp\n");
	if ( Fy->status & AGH_EDFCHK_DUP_CHANNELS )
		g_string_append_printf( out, "* Duplicate channel names\n");
	if ( Fy->status & AGH_EDFCHK_NOGAIN )
		g_string_append_printf( out, "* Physical or Digital Min not greater than Max\n");
	delete Fy;
}






const gchar*
agh_msmt_get_signal_type( const gchar *j_name, const gchar *e_name, const gchar *h_name)
{
	CMeasurement *M = AghCC->M_by_EJHD( AghCC->episode_names->o_by_name( e_name),
					    AghCC->J_by_name( j_name),
					    AghCC->channel_names->o_by_name( h_name),
					    AghCC->sessions->nth( agh_current_session));
	if ( !M || !M->source() )
		return "No such signal";

	CEDFFile *F = M->source();
	gint sig_no = F->sig_no_by_channel_name( h_name);
	return F->H(sig_no)->SignalType;
}





gint
agh_msmt_get_signal_vector( const gchar *j_name, const gchar *e_name, const gchar *h_name,
			    GArray *buffer, guint *samplerate, gfloat *signal_scale)
{
	CMeasurement *M = AghCC->M_by_EJHD( AghCC->episode_names->o_by_name( e_name),
					    AghCC->J_by_name( j_name),
					    AghCC->channel_names->o_by_name( h_name),
					    AghCC->sessions->nth( agh_current_session));
	if ( !M || !M->source() ) {
		g_array_set_size( buffer, 0);
		return -5;
	}

	CEDFFile *F = M->source();
	gint sig_no = F->sig_no_by_channel_name( h_name);
//	printf( "get vector [%d]%s\n", sig_no, H->name());

	gint retval =  F->ship_data_raw( buffer, sig_no,
					 0, F->DataRecordSize * F->NDataRecords); // page resized now
	if ( retval) {
		printf( "failed to retrieve signal data (%d)\n", retval);
		return retval;
	} else {
		*samplerate = F->H(sig_no)->SamplesPerRecord / F->DataRecordSize;
		*signal_scale = F->H(sig_no)->Scale;
		return 0;
	}
}







guint
agh_scores_get( const gchar *j_name, const gchar *e_name,
		GArray *scores, guint *pagesize_p)
{
	CMeasurement *M = AghCC->M_by_EJHD( AghCC->episode_names->o_by_name( e_name),
					    AghCC->J_by_name( j_name),
					    AghCC->channel_names->o_by_seq_no( agh_current_channel),
					    AghCC->sessions->nth( agh_current_session));
	guint p;
	if ( M && M->source() ) {
		CEDFFile *F = M->source();
		if ( scores->len < F->CHypnogram::length() )
			g_array_set_size( scores, F->CHypnogram::length()); // why does this not preserve previous content?

		for ( p = 0; p < F->CHypnogram::length(); p++ )
			g_array_index (scores, gchar, p) = F->nth_page(p).p2score();

		if ( pagesize_p)
			*pagesize_p = F->pagesize();
	}
	return p;
}





void
agh_scores_put( const gchar *j_name, const gchar *e_name,
		GArray *scores, guint pagesize)
{
	// we find the right source and its hypnogram regardless of h_name,
	// so don't ask the caller for h_name; using the current channel will
	// lead us to itjust the same
	CMeasurement *M = AghCC->M_by_EJHD( AghCC->episode_names->o_by_name( e_name),
					    AghCC->J_by_name( j_name),
					    AghCC->channel_names->o_by_seq_no( agh_current_channel),
					    AghCC->sessions->nth( agh_current_session));
	guint p = 0;
	if ( M && M->source() ) {
		CEDFFile *F = M->source();
		F->CHypnogram::set_size( 0);
		F->CHypnogram::set_size( scores->len);
		for ( ; p < scores->len; p++ )
			switch ( g_array_index (scores, gchar, p) ) {
		      // these codes don't correspond to agh_score_codes
		      // (but should they not?)
			case '1': F->CHypnogram::nth_page(p).NREM =  .25; break;
			case '2': F->CHypnogram::nth_page(p).NREM =  .50; break;
			case '3': F->CHypnogram::nth_page(p).NREM =  .75; break;
			case '4': F->CHypnogram::nth_page(p).NREM = 1.  ; break;
			case '5': F->CHypnogram::nth_page(p).REM  = 1.  ; break;
			case '6': F->CHypnogram::nth_page(p).Wake = 1.  ; break;
			case '7': F->CHypnogram::nth_page(p).Wake = AGH_MVT_WAKE_VALUE; break; // .01 is specially reserved for MVT
			}
		F->set_pagesize( pagesize);

	}
}



gint
agh_scores_import( const gchar *j_name, const gchar *e_name,
		   const gchar *fname)
{
	CMeasurement *M = AghCC->M_by_EJHD( AghCC->episode_names->o_by_name( e_name),
					    AghCC->J_by_name( j_name),
					    AghCC->channel_names->o_by_seq_no( agh_current_channel),
					    AghCC->sessions->nth( agh_current_session));
	if ( M && M->source() )
		M->source()->load_canonical( fname);
	return 0;
}


void
agh_scores_export( const gchar *j_name, const gchar *e_name,
		   const gchar *fname)
{
	CMeasurement *M = AghCC->M_by_EJHD( AghCC->episode_names->o_by_name( e_name),
					    AghCC->J_by_name( j_name),
					    AghCC->channel_names->o_by_seq_no( agh_current_channel),
					    AghCC->sessions->nth( agh_current_session));
	if ( M && M->source() )
		M->source()->save_canonical( fname);
}








gboolean
agh_msmt_has_edf( const gchar *j_name,
		  const gchar *e_name,
		  const gchar *h_name)
{
	CMeasurement *M = AghCC->M_by_EJHD( AghCC->episode_names->o_by_name( e_name),
					    AghCC->J_by_name( j_name),
					    AghCC->channel_names->o_by_name( h_name),
					    AghCC->sessions->nth( agh_current_session));
	return M && M->source();
}

gboolean
agh_msmt_has_powercourse( const gchar *j_name,
			  const gchar *e_name,
			  const gchar *h_name)
{
	CMeasurement *M = AghCC->M_by_EJHD( AghCC->episode_names->o_by_name( e_name),
					    AghCC->J_by_name( j_name),
					    AghCC->channel_names->o_by_name( h_name),
					    AghCC->sessions->nth( agh_current_session));
	return M && M->status & AGH_BP_OBTAINED;
}





gint
agh_msmt_get_power( const gchar *j_name, const gchar *e_name, const gchar *h_name,
		    GArray *power, GArray *track,
		    gfloat from, gfloat upto, double *max_value,
		    gboolean filtered)
{
	CMeasurement *M = AghCC->M_by_EJHD( AghCC->episode_names->o_by_name( e_name),
					    AghCC->J_by_name( j_name),
					    AghCC->channel_names->o_by_name( h_name),
					    AghCC->sessions->nth( agh_current_session));
	if ( M == NULL ) {
		g_array_set_size( power, 0);
		g_array_set_size( track, 0);
		return -5;
	}

	double _max_value = M->ship_power_course( power, track,
						  AghCC->pagesize, AghCC->fft_params,
						  from, upto,
						  agh_smoothover, agh_glitch_mag,
						  filtered);
	if ( max_value )
		*max_value = _max_value;

	if ( _max_value )
		return 0;
	else
		return -1;
}





void
agh_msmt_update_track( const gchar *j_name, const gchar *e_name, const gchar *h_name,
		       GArray *track)
{
	CMeasurement *M = AghCC->M_by_EJHD( AghCC->episode_names->o_by_name( e_name),
					    AghCC->J_by_name( j_name),
					    AghCC->channel_names->o_by_name( h_name),
					    AghCC->sessions->nth( agh_current_session));

	if ( M )
		M->update_track( track->data);
}



void
agh_msmt_set_pagesize( const gchar *j_name, const gchar *e_name,
		       guint psize)
{
	// see comment to agh_scores_get() on why there is no h_name parameter here
	CMeasurement *M = AghCC->M_by_EJHD( AghCC->episode_names->o_by_name( e_name),
					    AghCC->J_by_name( j_name),
					    AghCC->channel_names->o_by_seq_no( agh_current_channel),
					    AghCC->sessions->nth( agh_current_session));
	if ( M && M->source() )
		M->source()->set_pagesize( psize);
}











gfloat
agh_fft_get_binsize()
{
	return AghCC->fft_params.bin_size;
}

gint
agh_fft_set_binsize( gdouble val, double *corrected)
{
	if ( val < 1. / (2 << AghCC->fft_params.window_size_2power) ) {
		if ( corrected )
			*corrected = 1. / (2 << AghCC->fft_params.window_size_2power);
		return 0;
	} else
		AghCC->fft_params.bin_size = val;
	return 1;
}


TFFTWinType
agh_fft_get_window_type()
{
	return AghCC->fft_params.window_type;
}

void
agh_fft_set_window_type( TFFTWinType value)
{
	AghCC->fft_params.window_type = value;
}


gint
agh_fft_get_window_size_2power()
{
	return AghCC->fft_params.window_size_2power;
}

void
agh_fft_set_window_size_2power( gint value)
{
	AghCC->fft_params.window_size_2power = value;
}


guint
agh_get_ppuV2()			{ return agh_ppuV2; }

void
agh_set_ppuV2( guint val)	{ agh_ppuV2 = val; }


guint
agh_get_smoothover()		{ return agh_smoothover; }

void
agh_set_smoothover( guint val)	{ agh_smoothover = val; }


gfloat
agh_get_glitch_mag()		{ return agh_glitch_mag; }

void
agh_set_glitch_mag( gfloat val)	{ agh_glitch_mag = val; }




void
agh_collect_msmts_from_tree()
{
	AghCC->collect_msmts_from_tree();

	agh_populate_mDesignSetupChannels();
	agh_populate_mDesignSetupEpisodes();
}


// EOF
