// ;-*-C++-*- *  Time-stamp: "2008-06-24 16:05:31 hmmr"

/***************************************************************************
 *            agh_iface-populate.cc
 *
 *  Copyright  2008  Andrei Zavada
 *  Email johnhommer@gmail.com

 *  agh_populate-* functions used to stock initially at startup, and
 *  also to restock the GtkTreeStore objects.  They are quite bulky so
 *  isolated in this file.
 *
 *  Subsequently, other functions defined throughout elsewhere manage
 *  synchronisation of treemodels with the core structures.  Callbacks
 *  in agh-ui are simply wrappers for these.
 *
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <sys/stat.h>

#include <glib.h>
#include <gtk/gtk.h>

#include "agh_expdesign.hh"
#include "agh_io.hh"
#include "agh_model.hh"

#include "agh_iface.hh"
#include "agh_ui.h"





void
agh_populate_models()
{
	agh_populate_mDesignSetupGroupsSubjects();
	agh_populate_mDesignSetupGroups();
	agh_populate_mDesignSetupChannels();
	agh_populate_mDesignSetupSessions();
	agh_populate_mDesignSetupEpisodes();
	agh_populate_mDesignSetupSources();
	agh_populate_mDesignMeasurements();
	agh_populate_mSimulations();
}





gint
agh_populate_mDesignSetupGroupsSubjects()
{
	gtk_tree_store_clear( agh_mDesignSetupGroupsSubjects);

	GtkTreeIter iter, iter2;
	for ( guint g = 0; g < AghCC->groups->n(); g++ ) {
		CRefList<CSubject> *G = AghCC->groups->nth( g);
		gtk_tree_store_append( agh_mDesignSetupGroupsSubjects, &iter, NULL);
		gtk_tree_store_set( agh_mDesignSetupGroupsSubjects, &iter,
				    0, G->name(),
				    1, "x",
				    2, 0,
				    4, FALSE,
				    -1);
		for ( guint j = 0; j < G->n(); j++ ) {
			CSubject *J = G->nth(j);
//			guint symbol[2] = { 0x2640};
			gtk_tree_store_append( agh_mDesignSetupGroupsSubjects, &iter2, &iter);
			gtk_tree_store_set( agh_mDesignSetupGroupsSubjects, &iter2,
					    0, J->name(),
					    1, (J->gender() == AGH_SUBJECT_GENDER_MALE) ?"M" :"F",
					    2, J->age(),
					    3, J->comment(),
					    4, TRUE,
					    -1);
		}
	}

	return 0;
}







//static GMutex *x_mDesignSession = NULL;



gint
agh_populate_mDesignSetupSessions()
{
//	g_mutex_lock( x_mDesignSession);
	__agh__disconnect_sessions_combo();
	gtk_list_store_clear( agh_mDesignSetupSessions);

	GtkTreeIter iter;
	for ( guint i = 0; i < AghCC->sessions->n(); i++ ) {
		CRefList<CMeasurement> *C = AghCC->sessions->nth( i);
		gtk_list_store_append( agh_mDesignSetupSessions, &iter);
		gtk_list_store_set( agh_mDesignSetupSessions, &iter,
				    0, C->name(),
				    -1);
	}
//	g_mutex_unlock( x_mDesignSession);
	__agh__reconnect_sessions_combo();
	return 0;
}






gint
agh_populate_mDesignSetupEpisodes()
{
	gtk_list_store_clear( agh_mDesignSetupEpisodes);

      // populate in order
	GtkTreeIter iter;
	for ( guint i = 0; i < AghCC->episode_names->n(); i++ ) {
		COrderedName *o = AghCC->episode_names->o_by_seq_no( i);
		if ( o ) {
			gtk_list_store_append( agh_mDesignSetupEpisodes, &iter);
			gtk_list_store_set( agh_mDesignSetupEpisodes, &iter,
					    0, o->name(),
					    -1);
		}
	}
	return 0;
}











gint
agh_populate_mDesignSetupChannels()
{
	__agh__disconnect_channels_combo();
	gtk_list_store_clear( agh_mDesignSetupChannels);

//	AghCC->channel_names->dump();
      // populate in order
	GtkTreeIter iter;
	for ( guint i = 0; i < AghCC->channel_names->n(); i++ ) {
		COrderedName *o = AghCC->channel_names->o_by_seq_no( i);
		if ( o ) {
			gtk_list_store_append( agh_mDesignSetupChannels, &iter);
			gtk_list_store_set( agh_mDesignSetupChannels, &iter,
					    0, o->name(),
					    -1);
		}
	}
	__agh__reconnect_channels_combo();
	return 0;
}






gint
agh_populate_mDesignSetupGroups()
{
	gtk_list_store_clear( agh_mDesignSetupGroups);

	GtkTreeIter iter;
	for ( guint g = 0; g < AghCC->groups->n(); g++ ) {
		CRefList<CSubject> *G = AghCC->groups->nth( g);
		gtk_list_store_append( agh_mDesignSetupGroups, &iter);
		gtk_list_store_set( agh_mDesignSetupGroups, &iter,
				    0, G->name(),
				    -1);
	}

	return 0;
}



gint
agh_populate_mDesignSetupSources()
{
	gtk_list_store_clear( agh_mDesignSetupSources);

	static GString *ch_list_s = NULL;
	if ( !ch_list_s )
		ch_list_s = g_string_sized_new( 50);

	char len_buf[20], time_buf[28];

	GtkTreeIter iter;
	for ( guint f = 0; f < AghCC->sources->n(); f++ ) {
		CEDFFile *F = AghCC->sources->nth( f);

		if ( F->status & AGH_EDFCHK_BAD_HEADER )
			continue;

		g_string_assign( ch_list_s, "");
		for ( guint h = 0; h < F->NSignals; h++ ) {
			g_string_append_printf( ch_list_s, F->H(h)->Channel);
			if ( h < F->NSignals-1 )
				g_string_append_printf( ch_list_s, ", ");
		}

		guint	_hr  =  F->CEDFFile::length() / 3600,
			_min = (F->CEDFFile::length() - _hr * 3600) / 60,
			_sec =  F->CEDFFile::length() % 60;
		snprintf( len_buf, 19,
			  "%2d:%02d:%02d (%dp)",
			  _hr, _min, _sec,
			  F->CEDFFile::length() / AghCC->pagesize);

		strftime( time_buf, 27, "%y-%m-%d %T", &F->timestamp_struct);

		gtk_list_store_append( agh_mDesignSetupSources, &iter);
		gtk_list_store_set( agh_mDesignSetupSources, &iter,
				    0, F->PatientID_raw,
				    1, ch_list_s->str,
				    2, F->Session,
				    3, F->Episode,
				    4, len_buf,
				    5, time_buf,
				    6, F->filename(),
				    -1);
	}

	return 0;
}






// the following populate_* functions are special because we have to
// deal with variable number of episodes, and so need to add/remove
// columns to tvDesignEpisodes in addition to restocking the connected
// model

gint
agh_populate_mDesignMeasurements()
{
	// agh! Let's just cap the total no. of episodes to some sane 20.
	// I ain't know how to add/remove colimns to an existing gtk_tree_store,
	//
	// Hope there's a way to hide 'em unnecessary from gtk_tree_view.

	CMeasurement *My = new CMeasurement();
	static GString *statestring = NULL;
	if ( !statestring )  statestring = g_string_sized_new( 50);

	gtk_tree_store_clear( GTK_TREE_STORE (agh_mDesignMeasurements));

	guint e = 0;  // need it here outside all possibly-non-enterable loops,
		      // to know how many unused episode columns to hide

	GtkTreeIter iter, iter2;
	for ( guint g = 0; g < AghCC->groups->n(); g++ ) {
		CRefList<CSubject> *G = AghCC->groups->nth( g);
		gtk_tree_store_append( agh_mDesignMeasurements, &iter, NULL);
		gtk_tree_store_set( agh_mDesignMeasurements, &iter,
				    0, G->name(),
				    17, FALSE,
				    -1);

		for ( guint j = 0; j < G->n(); j++ ) {
			CSubject *J = G->nth(j);
			gtk_tree_store_append( agh_mDesignMeasurements, &iter2, &iter);
			gtk_tree_store_set( agh_mDesignMeasurements, &iter2,
					    0, J->name(),
					    17, TRUE,
					    -1);

			for ( e = 0; e < AghCC->episode_names->n(); e++ ) {
				COrderedName *E = AghCC->episode_names->o_by_seq_no(e);
				GtkTreeViewColumn *col = gtk_tree_view_get_column( GTK_TREE_VIEW (tvDesignMeasurements), e+1);
				gtk_tree_view_column_set_visible( col, TRUE);
				gtk_tree_view_column_set_title( gtk_tree_view_get_column( GTK_TREE_VIEW (tvDesignMeasurements),
											  e+1),
								E->name());
			}

			for ( e = 0; e < AghCC->episode_names->n(); e++ ) {
				COrderedName *E = AghCC->episode_names->o_by_seq_no(e);
				My->make_phony( E, J,
						AghCC->channel_names->o_by_seq_no(agh_current_channel),
						AghCC->sessions->nth(agh_current_session));
				CMeasurement *M = AghCC->measurements->exist( My);

				if ( M ) {
					g_string_assign( statestring, "");
					if ( M->source() ) {
						CEDFFile *F = M->source();
						guint	_hr  = F->CEDFFile::length() / 3600,
							_min = (F->CEDFFile::length() - _hr * 3600) / 60,
							_sec = F->CEDFFile::length() % 60;
						g_string_append_printf( statestring,
									"%2d:%02d:%02d",
									_hr, _min, _sec);
					}
					if ( M->status & AGH_BP_OBTAINED ) {
						g_string_append_printf( statestring, "\342\227\217");
						if ( M->CBinnedPower::pages() )
							g_string_append_printf( statestring, "%dp",
										M->pages() * M->pagesize() / AghCC->pagesize);
					}
					gtk_tree_store_set( agh_mDesignMeasurements, &iter2,
							    e+1, statestring->str,
							    -1);
				}
			}
		}
	}

      // and also render remaining comlumns invisible, please
	for ( ; e < 16; e++ )
		gtk_tree_view_column_set_visible( gtk_tree_view_get_column( GTK_TREE_VIEW (tvDesignMeasurements),
									    e+1),
						  FALSE);

	delete My;

	gtk_tree_view_expand_all( GTK_TREE_VIEW (tvDesignMeasurements));

	return 0;
}









gint
agh_populate_mSimulations()
{
	gtk_tree_store_clear( agh_mSimulations);

	guint last_t = AghCC->control_params.AZAmendment ?AghCC->episode_names->n() :1;

	gchar valuestr[AGH_VNUM + AGH_EGC_MAX - 1][12];

	CRefList<CMeasurement> *D = AghCC->sessions->nth(agh_current_session);
	COrderedName           *H = AghCC->channel_names->o_by_seq_no(agh_current_channel);

	CSimulation *Ry = new CSimulation,
		    *R;

	guint m = 0;
	GtkTreeIter iter, iter2;
	for ( guint g = 0; g < AghCC->groups->n(); g++ ) {
		CRefList<CSubject> *G = AghCC->groups->nth( g);
		gtk_tree_store_append( agh_mSimulations, &iter, NULL);
		gtk_tree_store_set( agh_mSimulations, &iter,
				    0, G->name(),
				    10+AGH_EGC_MAX-1, FALSE,
				    -1);

		for ( guint j = 0; j < G->n(); j++ ) {
			CSubject *J = G->nth(j);
			gtk_tree_store_append( agh_mSimulations, &iter2, &iter);
			gtk_tree_store_set( agh_mSimulations, &iter2,
					    0, J->name(),
					    10+AGH_EGC_MAX-1, TRUE,
					    -1);

			Ry->make_phony( J, D, H,
					agh_sim_get_operating_range_from(), agh_sim_get_operating_range_upto(),
					agh_pagesize_get_current(),
					agh_fft_get_window_size_2power(), agh_fft_get_binsize(), agh_fft_get_window_type());
			if ( (R = AghCC->simulations->exist( Ry)) ) {
				guint t;
				for ( t = 0; t < AGH_VNUM; t++ )
					snprintf( valuestr[t], 11, agh_v_formats[t], R->value.pidx[t] * agh_v_dispf[t]);
				for ( t = 0; t < last_t - 1; t++ )
					snprintf( valuestr[AGH_VNUM+t], 11, agh_v_formats[AGH_Vgc], R->value.pname.egc[t] * agh_v_dispf[AGH_Vgc]);

				gtk_tree_store_set( agh_mSimulations, &iter2,
						    1, valuestr[AGH_VS_0],  2, valuestr[AGH_VS_U],
						    3, valuestr[AGH_Vfc_R], 4, valuestr[AGH_Vfc_W],
						    5, valuestr[AGH_Vt_a],  6, valuestr[AGH_Vt_p],
						    7, valuestr[AGH_Vrc],   8, valuestr[AGH_Vrs],
						    9, valuestr[AGH_Vgc],
						    -1);
				guint m = 1;
				if ( AghCC->control_params.AZAmendment )
					for ( ; m < R->measurements->len; m++ )
						gtk_tree_store_set( agh_mSimulations, &iter2,
								    9 + m, valuestr[AGH_VNUM+m-1],
								    -1);
			} else
				gtk_tree_store_set( agh_mSimulations, &iter2,
						    1,  "",  2, "",  3, "",
						    4,  "",  5, "",  6, "",
						    7,  "",  8, "",  9, "",
						    10, "", 11, "", 12, "",
						    13, "", 14, "", 15, "",
						    -1);

		}
	}

	gtk_tree_view_column_set_title( gtk_tree_view_get_column( GTK_TREE_VIEW (tvSimulations), 9),
					AghCC->control_params.AZAmendment ?"gc1" :"gain const.");

	for ( m = 1; m < AGH_EGC_MAX; m++ )
		gtk_tree_view_column_set_visible( gtk_tree_view_get_column( GTK_TREE_VIEW (tvSimulations), 9 + m),
						  m < last_t);

	delete Ry;

	gtk_tree_view_expand_all( GTK_TREE_VIEW (tvSimulations));

	return 0;
}















void
agh_populate_models_static()
{
	GtkTreeIter iter;
	gint i;

	__agh__disconnect_pagesize_combo();
	const gchar* const pagesize_values[] = { "5 sec", "10 sec", "15 sec", "20 sec", "30 sec", "1 min", NULL };
	i = 0;
	while ( pagesize_values[i] ) {
		gtk_list_store_append( agh_mPageSize, &iter);
		gtk_list_store_set( agh_mPageSize, &iter, 0, pagesize_values[i++], -1);
	}
	__agh__reconnect_pagesize_combo();

	const gchar* const windowsize_values[] = { "2 sec", "4 sec", "8 sec", NULL };
	i = 0;
	while ( windowsize_values[i] ) {
		gtk_list_store_append( agh_mFFTParamsWindowSize, &iter);
		gtk_list_store_set( agh_mFFTParamsWindowSize, &iter, 0, windowsize_values[i++], -1);
	}

	const gchar* const windowtype_values[] = { "Bartlett", "Blackman", "Blackman-Harris",
						   "Hamming",  "Hanning",  "Parzen",
						   "Square",   "Welch", NULL };
	i = 0;
	while ( windowtype_values[i] ) {
		gtk_list_store_append( agh_mFFTParamsWindowType, &iter);
		gtk_list_store_set( agh_mFFTParamsWindowType, &iter, 0, windowtype_values[i++], -1);
	}
}



// EOF
