// ;-*-C++-*- *  Time-stamp: "2008-06-29 00:57:23 hmmr"

/***************************************************************************
 *            agh_iface-simulations.cc
 *
 *  Copyright  2008  Andrei Zavada
 *  Email johnhommer@gmail.com

 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <sys/stat.h>
#include <gtk/gtk.h>
#include "agh_iface.hh"
#include "agh_expdesign.hh"


gfloat	agh_sim_operating_range_from,
	agh_sim_operating_range_upto;



void
agh_collect_simulations_from_tree()
{
	AghCC->collect_simulations_from_tree();
	agh_populate_mSimulations();
}













gfloat
agh_sim_get_operating_range_low_bound()		    { return agh_sim_operating_range_from; }

gfloat
agh_sim_get_operating_range_width()		    { return agh_sim_operating_range_upto
							   - agh_sim_operating_range_from; }

gfloat
agh_sim_get_operating_range_from()		    { return agh_sim_operating_range_from; }
void
agh_sim_set_operating_range_from( gfloat val)	    { agh_sim_operating_range_from = val;  }
gfloat
agh_sim_get_operating_range_upto()		    { return agh_sim_operating_range_upto; }
void
agh_sim_set_operating_range_upto( gfloat val)	    { agh_sim_operating_range_upto = val;  }









gint
agh_sim_modelrun_setup( const gchar *j_name, const gchar *h_name,
			GArray *scores, GArray *swa_course,
			gdouble *swa_avg, void **R_ptr)
{
	gint retval;
	CSimulation *S;
	if ( retval = AghCC->setup_modrun( AghCC->J_by_name( j_name),
					   AghCC->sessions->nth( agh_current_session),
					   AghCC->channel_names->o_by_name( h_name),
					   agh_sim_operating_range_from,
					   agh_sim_operating_range_upto,
					   &S) ) {
		if ( R_ptr )	*R_ptr = NULL;
		return retval;
	}

	if ( swa_course && scores ) {
		g_array_set_size( swa_course,	S->vector->len);
		g_array_set_size( scores,	S->vector->len);
		memset( scores->data, 0, sizeof(char) * S->vector->len);

		for ( guint p = 0; p < S->vector->len; p++ ) {
			SPageSimulated &P = g_array_index (S->vector, SPageSimulated, p);
			g_array_index (swa_course, double, p) = P.SWA;
			g_array_index (scores, gchar, p) = P.p2score();
		}
	}

	if ( R_ptr )	*R_ptr = S;
	if ( swa_avg )	*swa_avg = S->_SWA_100;

	return 0;
}




guint
agh_sim_modelrun_get_n_episodes( void *R_ptr)
{
	CSimulation& R = *(CSimulation*) R_ptr;
	return R.measurements->len;
}



guint
agh_sim_modelrun_get_nth_episode_start_p( void *R_ptr, guint ep)
{
	CSimulation& R = *(CSimulation*) R_ptr;
	return R.nth_msmt_start_p(ep);
}

guint
agh_sim_modelrun_get_nth_episode_end_p( void *R_ptr, guint ep)
{
	CSimulation& R = *(CSimulation*) R_ptr;
	return R.nth_msmt_end_p(ep);
}



void
agh_sim_modelrun_reset( void *R_ptr)
{
	CSimulation& R = *(CSimulation*) R_ptr;
	AghCC->reset_modrun( &R);
}


gint
agh_sim_modelrun_get_snapshot( void *R_ptr, gint stride,
			       GArray *swa_sim_course, GArray *s_course,
			       SConsumerTunableSet *t_set, gint *stride_p)
{
	CSimulation& R = *(CSimulation*) R_ptr;
	guint t;
	gint n_unstable = AGH_VNUM + (R.AZAmendment ?R.measurements->len - 1 :0);

	if ( stride == -1 || stride >= R.stride() ) {  // get the current (last)
		if ( stride_p )
			*stride_p = R.stride();

		if ( swa_sim_course && s_course ) {
			g_array_set_size( swa_sim_course, R.vector->len);
			g_array_set_size( s_course,	  R.vector->len);

			guint p;
			for ( p = 0; p < R._sim_start; p++ )
				g_array_index (swa_sim_course, double, p) = g_array_index (s_course, double, p) = 0.;
			for (      ; p < R.vector->len; p++ ) {
				SPageSimulated &P = g_array_index (R.vector,  SPageSimulated, p);
				g_array_index (swa_sim_course, double, p) = P.SWA_sim;
				g_array_index (s_course,       double, p) = P.S;
			}
		}

		for ( t = 0; t < AGH_VNUM; t++ ) {
			if ( t_set )
				t_set->value[t]	= R.value.pidx[t];
			n_unstable -= !!(R.flags[t] & AGH_T_STABLE);
		}
		if ( R.AZAmendment )
			for ( t = 0; t < R.measurements->len-1; t++ ) {
				if ( t_set )
					t_set->egc[t] = R.value.pname.egc[t];
				n_unstable -= !!(R.flags[t] & AGH_T_STABLE);
			}
		if ( t_set ) {
			t_set->fit	= R.fit();
			t_set->dfit	= R.dfit();
			t_set->status	= R.status;
		}

		return n_unstable;

	} else {  // get the nth snapshot from the stack
		if ( swa_sim_course && s_course )
			if ( R.nth_swa_sim_course(stride) ) {
				memcpy( swa_sim_course->data,	R.nth_swa_sim_course(stride)->data, sizeof(double) * R.vector->len);
				memcpy( s_course->data,		R.nth_s_course      (stride)->data, sizeof(double) * R.vector->len);
			} else {
				g_array_set_size( swa_sim_course, 0);
				g_array_set_size( s_course,	  0);
			}

		if ( t_set ) {
			for ( t = 0; t < AGH_VNUM; t++ )
				t_set->value[t]	= R.nth_t_set(stride).pidx[t];
			t_set->fit = t_set->dfit = 0.;
			t_set->status = 0;
		}

		return -1;
	}
}



void
agh_sim_modelrun_save( void *R_ptr)
{
	CSimulation& R = *(CSimulation*) R_ptr;
	R.save( R.make_fname());
}





void
agh_sim_modelrun_update_tunables( void *R_ptr, SConsumerTunableSet *t_set)
{
	CSimulation& R = *(CSimulation*) R_ptr;
	guint t;
	for ( t = 0; t < AGH_VNUM; t++ )
		R.value.pidx[t] = t_set->value[t];
	for ( t = 0; t < R.measurements->len-1; t++ )
		R.value.pname.egc[t] = t_set->egc[t];
}


void
agh_sim_modelrun_do_cycle( void *R_p)
{
	return ((CSimulation*)R_p) -> do_cycle();
}


gint
agh_sim_modelrun_do_stride( void *R_p, gint from_stride, GString *log)
{
	return ((CSimulation*) R_p) -> do_stride( from_stride, log);
}





gboolean
	agh_sim_runbatch_include_all_channels,
	agh_sim_runbatch_include_all_sessions,
	agh_sim_runbatch_iterate_ranges;
gint
	agh_sim_runbatch_redo_option;





void
agh_sim_export_tsv( const gchar* fname)
{
	FILE *f = fopen( fname, "w");
	if ( !f )
		return;

	fprintf( f, "#Subject\tS0\tSU\tfcR\tfcW\tta\ttp\trc\trs\tgc\n");

	CSimulation *R, *Ry = new CSimulation;

	CRefList<CMeasurement>	*D = AghCC->sessions->nth(agh_current_session);
	COrderedName		*H = AghCC->channel_names->o_by_seq_no(agh_current_channel);
	for ( guint g = 0; g < AghCC->groups->n(); g++ ) {
		CRefList<CSubject> *G = AghCC->groups->nth(g);
		fprintf( f, "#Group %s\n", G->name());

		for ( guint j = 0; j < G->n(); j++ ) {
			CSubject *J = G->nth(j);
			fprintf( f, "%s", J->name());

			Ry->make_phony( J, D, H,
					agh_sim_operating_range_from, agh_sim_operating_range_upto,
					AghCC->pagesize,
					AghCC->fft_params.window_size_2power, AghCC->fft_params.bin_size, AghCC->fft_params.window_type);
			if ( (R = AghCC->simulations->exist( Ry)) ) {
				fprintf( f, "\t%g\t%g\t%g\t%g\t%g\t%g\t%g\t%g\t%g",
					 R->value.pname.S_0  * agh_v_dispf[AGH_VS_0 ], R->value.pname.S_U  * agh_v_dispf[AGH_VS_0 ],
					 R->value.pname.fc_R * agh_v_dispf[AGH_Vfc_R], R->value.pname.fc_W * agh_v_dispf[AGH_Vfc_W],
					 R->value.pname.t_a  * agh_v_dispf[AGH_Vt_a ], R->value.pname.t_p  * agh_v_dispf[AGH_Vt_p ],
					 R->value.pname.rc   * agh_v_dispf[AGH_Vrc  ], R->value.pname.rs   * agh_v_dispf[AGH_Vrs  ],
					 R->value.pname.gc   * agh_v_dispf[AGH_Vgc  ]);
				if ( R->AZAmendment )
					for ( guint m = 0; m < R->measurements->len; m++ )
						fprintf( f, "\t%g", R->value.pname.egc[m] * agh_v_dispf[AGH_Vgc]);
			}
			fprintf( f, "\n");
		}
	}

	fclose( f);
}



// EOF
