// ;-*-C-*-   Time-stamp: "2008-07-01 22:47:51 hmmr"

/***************************************************************************
 *            agh_iface.h
 *
 *  Copyright  2008  Andrei Zavada
 *  Email johnhommer@gmail.com

 *  Structures and functions accessed both from ui (plain C) and
 *  interface (C++)
 *
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _AGH_IFACE_H
#define _AGH_IFACE_H


#include <gtk/gtk.h>

#include "agh_enums.h"

#if HAVE_CONFIG_H
#  include <config.h>
#endif

#define FAFA printf( __FILE__ " %d: fafa\n", __LINE__);





extern int agh_core_init();
extern int agh_core_shutdown();



G_BEGIN_DECLS

void agh_pop_ok_message( const gchar*);


extern GtkListStore *agh_mExpDesignList;

extern gchar	*agh_last_expdesign_dir;
extern gint	 agh_last_expdesign;

gint agh_expdesign_init( void);
gint agh_expdesign_shutdown( void);


extern gint	agh_current_session,
		agh_current_channel,
		agh_n_current_episodes;

extern GString	*agh_j_name,
		*agh_e_name;


// tree/list models

//  (one-time populatable at startup)
extern GtkTreeStore *agh_mDesignSetupGroupsSubjects;
gint        agh_populate_mDesignSetupGroupsSubjects();

extern GtkListStore *agh_mDesignSetupGroups;
gint        agh_populate_mDesignSetupGroups();

extern GtkListStore *agh_mDesignSetupEpisodes;
gint        agh_populate_mDesignSetupEpisodes();

extern GtkListStore *agh_mDesignSetupSessions;
gint        agh_populate_mDesignSetupSessions();

extern GtkListStore *agh_mDesignSetupChannels;
gint        agh_populate_mDesignSetupChannels();

extern GtkListStore *agh_mDesignSetupSources;
gint        agh_populate_mDesignSetupSources();

extern GtkTreeStore *agh_mDesignMeasurements;
gint        agh_populate_mDesignMeasurements();

extern GtkTreeStore *agh_mSimulations;
gint        agh_populate_mSimulations();

void	    agh_populate_models();


extern GtkListStore *agh_mPageSize;
extern GtkListStore *agh_mFFTParamsWindowType;
extern GtkListStore *agh_mFFTParamsWindowSize;

void	    agh_populate_models_static();




// various core controls
gint agh_add_subject_to_group( guint, const gchar*);
gint agh_add_group( const gchar*);
gint agh_rename_group( guint, gchar*);
gint agh_rename_subject( guint, guint, gchar*);
void agh_set_subject_age( guint, guint, guint);
void agh_set_subject_comment( guint, guint, const gchar*);
void agh_switch_subject_gender( guint, guint);
void agh_delete_group_with_subjects( guint);
void agh_delete_subject( guint, guint);
const gchar *agh_get_subject_group( const gchar*);
gint agh_get_n_groups();
gint agh_get_n_subjects_in_group( gint);
const gchar *agh_get_nth_subject_name_in_group( gint, gint);


gint agh_add_session( const gchar*);
void agh_delete_session_at( guint);
gint agh_rename_session_at( const gchar*, guint);
gint agh_get_n_sessions();
const gchar *agh_get_current_session_name();
const gchar *agh_get_nth_session_name( gint);

gint agh_append_episodename( const gchar*);
gint agh_insert_episodename_at( const gchar*, gint);
void agh_delete_episodename_at( guint);
gint agh_rename_episodename_at( guint, const gchar*);
gint agh_move_episodename_up( guint);
gint agh_move_episodename_down( guint);
const gchar *agh_get_nth_episode_name( gint);

const gchar *agh_get_current_channel_name();
const gchar *agh_get_nth_channel_name( gint);
gint agh_append_channelname( const gchar*);
gint agh_insert_channelname_at( const gchar*, guint);
void agh_delete_channelname_at( guint);
gint agh_rename_channelname_at( guint, const gchar*);
gint agh_move_channelname_up( guint);
gint agh_move_channelname_down( guint);
gint agh_get_channel_name_list( GPtrArray*);
gint agh_get_n_channels();




extern guint	agh_pagesize_values[],
		agh_pagesize_item;
extern const guint
		agh_pagesize_total_items;

guint agh_pagesize_get_current();
guint agh_pagesize_set_current( guint);
guint agh_pagesize_set_current_by_item( guint);




#define AGH_KNOWN_CHANNELS_TOTAL 78
extern const char *__agh_System1020_channels[];

#define AGH_KNOWN_SIGNAL_TYPES 16
extern const char *__agh_SignalTypeByKemp[];

const gchar* agh_edf_get_subject( const gchar*);
gint agh_edf_add_or_replace( const gchar*, TSourceAttachMode);
void agh_edf_delete( const gchar*);
void agh_edf_get_stats( const gchar*, GString*);
gint agh_edf_is_good( const gchar*, GString*);
gint agh_msmt_get_signal_vector( const gchar*, const gchar*, const gchar*,
				GArray*, guint*, gfloat*);
const gchar *agh_msmt_get_signal_type( const gchar*, const gchar*, const gchar*);


guint agh_scores_get( const gchar*, const gchar*, GArray*, guint *);
void  agh_scores_put( const gchar*, const gchar*, GArray*, guint);
gint  agh_scores_import( const gchar*, const gchar*, const gchar*);
void  agh_scores_export( const gchar*, const gchar*, const gchar*);





gfloat agh_fft_get_binsize();
gint agh_fft_set_binsize( gdouble, gdouble*);
TFFTWinType agh_fft_get_window_type();
void agh_fft_set_window_type( TFFTWinType);
gint agh_fft_get_window_size_2power();
void agh_fft_set_window_size_2power( gint);


extern guint	agh_ppuV2,
		agh_smoothover;
extern gfloat	agh_glitch_mag;

guint  agh_get_ppuV2();
void   agh_set_ppuV2( guint);
guint  agh_get_smoothover();
void   agh_set_smoothover( guint);
gfloat agh_get_glitch_mag();
void   agh_set_glitch_mag( gfloat);




gboolean agh_msmt_has_edf( const gchar*, const gchar*, const gchar*);
gboolean agh_msmt_has_powercourse( const gchar*, const gchar*, const gchar*);
gint agh_msmt_get_power( const gchar*, const gchar*, const gchar*,
			 GArray*, GArray*,
			 gfloat, gfloat, gdouble*,
			 gboolean);
void agh_msmt_update_track( const gchar*, const gchar*, const gchar*,
			    GArray*);
void agh_msmt_set_pagesize( const gchar*, const gchar*,
			    guint);





extern gfloat	agh_sim_operating_range_from,
		agh_sim_operating_range_upto;

gfloat agh_sim_get_operating_range_low_bound();
gfloat agh_sim_get_operating_range_width();

gfloat agh_sim_get_operating_range_from();
void   agh_sim_set_operating_range_from( gfloat);
gfloat agh_sim_get_operating_range_upto();
void   agh_sim_set_operating_range_upto( gfloat);




extern const gchar* const agh_v_names[];
extern const gchar* const agh_v_formats[];
extern const guint   agh_v_dispf[];
extern const gdouble agh_v_defaults[];
extern const gdouble agh_v_min[];
extern const gdouble agh_v_max[];
extern const gdouble agh_v_step[];

typedef struct {
	gdouble	value[AGH_VNUM];
	gdouble	egc[AGH_EGC_MAX-1];
	gdouble	fit, dfit;
	gint	status;
} SConsumerTunableSet;


gint agh_sim_modelrun_setup( const gchar*, const gchar*,
			     GArray*, GArray*, gdouble*, void**);
guint agh_sim_modelrun_get_n_episodes( void*);
guint agh_sim_modelrun_get_nth_episode_start_p( void*, guint);
guint agh_sim_modelrun_get_nth_episode_end_p( void*, guint);
void agh_sim_modelrun_reset( void*);

gint agh_sim_modelrun_get_snapshot( void*, gint,
				    GArray *swa_sim, GArray *s,
				    SConsumerTunableSet*, gint*);
void agh_sim_modelrun_update_tunables( void*, SConsumerTunableSet*);

void agh_sim_modelrun_do_cycle( void*);
gint agh_sim_modelrun_do_stride( void*, gint, GString*);
void agh_sim_modelrun_save( void*);

extern gboolean
	agh_sim_runbatch_include_all_channels,
	agh_sim_runbatch_include_all_sessions,
	agh_sim_runbatch_iterate_ranges;
extern gint
	agh_sim_runbatch_redo_option;


void agh_sim_export_tsv( const gchar*);


void agh_collect_msmts_from_tree();

void agh_collect_simulations_from_tree();




void agh_dump_all();

G_END_DECLS

#endif

// EOF
