// ;-*-C++-*- *  Time-stamp: "2008-07-01 18:17:11 hmmr"

/***************************************************************************
 *            agh_io-edf.cc
 *
 *  Copyright  2008  Andrei Zavada
 *  Email johnhommer@gmail.com

 *
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <stdio.h>

#include <glib.h>
#include <glib/gstdio.h>

#include "agh_io.hh"




CEDFFile::~CEDFFile()
{
	for ( guint i = 0; i < NSignals; i++ )
		g_free( H(i)->SignalType), g_free( H(i)->Channel);
	if ( _signal_data )
		g_array_free( _signal_data, TRUE);
	if ( CEDFFile::_filename )
		g_string_free( CEDFFile::_filename, TRUE);
}




gint
CEDFFile::sig_no_by_channel_name( const gchar *ch)
{
	for ( guint i = 0; i < NSignals; i++ )
		if ( !strcmp( ch, H(i)->Channel) )
			return i;
	return -1;
}




gint
CEDFFile::assimilate( const gchar *where_to, TSourceAttachMode option)
{
	struct stat stat_data_in,
		    stat_data_out;
	switch ( option ) {
	case AGH_SRCATTACH_COPY:
	{
		printf( "copying %s to %s\n", CEDFFile::_filename->str, where_to);
		int fd_in = open( CEDFFile::_filename->str, O_RDONLY);
		if ( fd_in == -1 ) {
			printf( "failed opening %s: %s\n", CEDFFile::_filename->str, strerror( errno));
			return fd_in;
		}
		int fd_out = open( where_to, O_RDWR | O_CREAT | O_TRUNC, 0644);
		if ( fd_out == -1 ) {
			printf( "failed opening %s: %s\n", where_to, strerror( errno));
			close( fd_in);
			return fd_out;
		}

		gsize buf_size = 20 * 1024 * 1024;
		gpointer buffer = g_malloc( buf_size);
		gssize bytes_read;
		while ( (bytes_read = read( fd_in, buffer, buf_size)) > 0 )
			write( fd_out, buffer, bytes_read);

		g_free( buffer);
		close( fd_out);
		close( fd_in);

		if ( stat( CEDFFile::_filename->str, &stat_data_in) || stat( where_to, &stat_data_out) ) {
			printf( "stat failed: %s\n", strerror( errno));
			return -2;
		}

		if ( stat_data_in.st_size != stat_data_out.st_size ) {
			printf( "Copied file differs in size from source\n");
			return -3;
		}
	}
	break;

	case AGH_SRCATTACH_MOVE:
//		printf( "moving %s to %s\n", CEDFFile::_filename->str, where_to);
		if ( g_rename( CEDFFile::_filename->str, where_to) == -1 ) {
			printf( "failed moving : %s\n", strerror( errno));
			return -4;
		}
		// stat
	    break;

	case AGH_SRCATTACH_LINK:
		printf( "linking %s as %s\n", CEDFFile::_filename->str, where_to);
		symlink( CEDFFile::_filename->str, where_to);
		if ( stat( CEDFFile::_filename->str, &stat_data_in) || stat( where_to, &stat_data_out) ) {
			printf( "symlink failed: %s\n", strerror( errno));
			return -2;
		}
		if ( stat_data_in.st_size != stat_data_out.st_size )
			return -3;
	    break;

	case AGH_SRCATTACH_LEAVE:
	    break;
	}

	if ( option != AGH_SRCATTACH_LEAVE )
		g_string_assign( CEDFFile::_filename, where_to);

      // assimilate girlfriend, too
	g_string_assign( CHypnogram::_filename, CEDFFile::_filename->str);
	if ( g_str_has_suffix( CHypnogram::_filename->str, ".edf") || g_str_has_suffix( CHypnogram::_filename->str, ".EDF") )
		g_string_truncate( CHypnogram::_filename, CHypnogram::_filename->len-4);
	g_string_append_printf( CHypnogram::_filename, ".hypnogram");

	attach_mode = option;

	CHypnogram::load();

	return 0;
}







#define AGH_KNOWN_CHANNELS_TOTAL 78

const gchar *__agh_System1020_channels[] = {  // counted 'em all!
	"Nz",
	"Fp1", "Fpz", "Fp2",
	"AF7", "AF3", "AFz", "AF4", "AF8",
	"F9",  "F7", "F5", "F3", "F1", "Fz", "F2", "F4", "F6", "F8", "F10",
	"FT9", "FT7", "FC5", "FC3", "FC1", "FCz", "FC2", "FC4", "FC6", "FCT8", "FT10",
	"A1", "T9", "T7", "C5", "C3", "C1", "Cz", "C2", "C4", "C6", "T8", "T10", "A2",
	"TP9", "TP7", "CP5", "CP3", "CP1", "CPz", "CP2", "CP4", "CP6", "TP8", "TP10",
	"P9", "P7", "P5", "P3", "P1", "Pz", "P2", "P4", "P6", "P8", "P10",
	"PO7", "PO3", "POz", "PO4", "PO8",
	"O1", "Oz", "O2",
	"Iz",
	// plus channels of other signal types
	"Left", "Right",
	"Chin"
};

#define AGH_KNOWN_SIGNAL_TYPES 16

const gchar *__agh_SignalTypeByKemp[] = {
	"EEG", "ECG", "EOG", "ERG", "EMG",
	"NC",  "MEG", "MCG", "EP",
	"Temp", "Resp", "SaO2",
	"Light", "Sound", "Event", "Freq",
};

static gssize read_field( gint fd, gchar* field, guint fld_size)
{
	memset( (void*)field, '\0', fld_size+1);
	gssize retval = read( fd, field, fld_size);
	guint c = fld_size-1;
	while ( field[c] == ' '  && c )
		field[c--] = '\0';
	return retval;
}


static const gchar *follows_Kemp( const gchar *signal)
{
	for ( guint h = 0; h < AGH_KNOWN_SIGNAL_TYPES; h++ )
		if ( !strcmp( signal, __agh_SignalTypeByKemp[h]) )
			return __agh_SignalTypeByKemp[h];
	return NULL;
}

static const gchar *follows_1020( const gchar *channel)
{
	for ( guint h = 0; h < AGH_KNOWN_CHANNELS_TOTAL; h++ )
		if ( !strcmp( channel, __agh_System1020_channels[h]) )
			return __agh_System1020_channels[h];
	return NULL;
}


gint
CEDFFile::read_header()
{
	gint fd = open( _filename->str, O_RDONLY);
	if ( fd == -1 ) {
		status |= AGH_EDFCHK_BAD_HEADER;
		NSignals = 0;
		return -1;
	}

	gint retval = 0;
	gchar *p;

	status = 0;
	if ( read_field( fd, VersionNumber_raw,  8) !=  8 ||
	     read_field( fd, PatientID_raw,     80) != 80 ||
	     read_field( fd, RecordingID_raw,   80) != 80 ||
	     read_field( fd, RecordingDate_raw,  8) !=  8 ||
	     read_field( fd, RecordingTime_raw,  8) !=  8 ||
	     read_field( fd, HeaderLength_raw,   8) !=  8 ||
	     read_field( fd, Reserved_raw,      44) != 44 ||
	     read_field( fd, NDataRecords_raw,   8) !=  8 ||
	     read_field( fd, DataRecordSize_raw, 8) !=  8 ||
	     read_field( fd, NSignals_raw,       4) !=  4 )  {
		status |= AGH_EDFCHK_BAD_HEADER;
		retval = -2;
		goto skip_signal_hdr_checks;
	}

	if ( !strcmp( VersionNumber_raw, "0       ") ) {
		status |= AGH_EDFCHK_BAD_VERSION;
		retval = -2;
		goto skip_signal_hdr_checks;
	}

	HeaderLength = NDataRecords = DataRecordSize = NSignals = 0;
	sscanf( HeaderLength_raw,   "%8u", &HeaderLength);
	sscanf( NDataRecords_raw,   "%8u", &NDataRecords);
	sscanf( DataRecordSize_raw, "%8u", &DataRecordSize);
	sscanf( NSignals_raw,       "%4u", &NSignals);

	if ( !HeaderLength || !NDataRecords || !DataRecordSize || !NSignals ) {
		status |= AGH_EDFCHK_BAD_NUMFLD;
		retval -2;
		goto skip_signal_hdr_checks;
	}


	Episode = RecordingID_raw;
	p = strpbrk( RecordingID_raw, ",;:");
	if ( p ) {
		*p = '\0';
		p += strspn( p+1, " ") + 1;
		Session = p;
	} else {
		Session = NULL;
		status |= AGH_EDFCHK_NOSESSION;
		// this is a non-critical omission, so we go on reading;
		// but we know this edf will not end up being checked it
	}


	if ( sscanf( RecordingDate_raw, "%2d.%2d.%2d", &timestamp_struct.tm_mday,
						       &timestamp_struct.tm_mon,
						       &timestamp_struct.tm_year) != 3 )
		status |= AGH_EDFCHK_DATE_UNPARSABLE;
	if ( sscanf( RecordingTime_raw, "%2d.%2d.%2d", &timestamp_struct.tm_hour,
						       &timestamp_struct.tm_min,
						       &timestamp_struct.tm_sec) != 3 )
		status |= AGH_EDFCHK_TIME_UNPARSABLE;
	if ( !(status & (AGH_EDFCHK_TIME_UNPARSABLE | AGH_EDFCHK_DATE_UNPARSABLE)) ) {
		timestamp_struct.tm_mon--;
		if ( timestamp_struct.tm_year < 50 )
			timestamp_struct.tm_year += 100;
		timestamp = mktime( &timestamp_struct);
	}

	guint i;
	_signal_data = g_array_set_size( _signal_data, NSignals);

	for ( i = 0; i < NSignals; i++ )
		if ( read_field( fd, H(i)->Label, 16) != 16 ) {
			status |= AGH_EDFCHK_BAD_HEADER;
			retval = -2;
			break;
		}

	for ( i = 0; i < NSignals; i++ )
		if ( read_field( fd, H(i)->TransducerType, 80) != 80 ) {
			status |= AGH_EDFCHK_BAD_HEADER;
			retval = -2;
			break;
		}

	for ( i = 0; i < NSignals; i++ )
		if ( read_field( fd, H(i)->PhysicalDim, 8) != 8 ) {
			status |= AGH_EDFCHK_BAD_HEADER;
			retval = -2;
			break;
		}

	for ( i = 0; i < NSignals; i++ ) {
		H(i)->PhysicalMin = 0;
		if ( read_field( fd, H(i)->PhysicalMin_raw, 8) != 8 ||
		     !sscanf( H(i)->PhysicalMin_raw, "%8g",
			      &H(i)->PhysicalMin) ) {
			status |= AGH_EDFCHK_BAD_NUMFLD;
			retval = -2;
			break;
		}
	}
	for ( i = 0; i < NSignals; i++ ) {
		H(i)->PhysicalMax = 0;
		if ( read_field( fd, H(i)->PhysicalMax_raw, 8) != 8 ||
		     !sscanf( H(i)->PhysicalMax_raw, "%8g",
			      &H(i)->PhysicalMax) ) {
			status |= AGH_EDFCHK_BAD_NUMFLD;
			retval = -2;
			break;
		}
	}
	for ( i = 0; i < NSignals; i++ ) {
		H(i)->DigitalMin = 0;
		if ( read_field( fd, H(i)->DigitalMin_raw, 8) != 8 ||
		     !sscanf( H(i)->DigitalMin_raw, "%8d",
			     &H(i)->DigitalMin) ) {
			status |= AGH_EDFCHK_BAD_NUMFLD;
			retval = -2;
			break;
		}
	}
	for ( i = 0; i < NSignals; i++ ) {
		H(i)->DigitalMax = 0;
		if ( read_field( fd, H(i)->DigitalMax_raw, 8) != 8 ||
		     !sscanf( H(i)->DigitalMax_raw, "%8d",
			      &H(i)->DigitalMax) ) {
			status |= AGH_EDFCHK_BAD_NUMFLD;
			retval = -2;
			break;
		}
	}

	for ( i = 0; i < NSignals; i++ )
		if ( read_field( fd, H(i)->FilteringInfo, 80) != 80 ) {
			status |= AGH_EDFCHK_BAD_NUMFLD;
			retval = -2;
			break;
		}

	for ( i = 0; i < NSignals; i++ ) {
		H(i)->SamplesPerRecord = 0;
		if ( read_field( fd, H(i)->SamplesPerRecord_raw, 8) != 8 ||
		     !sscanf( H(i)->SamplesPerRecord_raw, "%8u",
			      &H(i)->SamplesPerRecord) ) {
			status |= AGH_EDFCHK_BAD_NUMFLD;
			retval = -2;
			break;
		}
	}

	for ( i = 0; i < NSignals; i++ )
		if ( read_field( fd, H(i)->Reserved, 32) != 32 )  {
			status |= AGH_EDFCHK_BAD_HEADER;
			retval = -2;
			break;
		}

      // calculate gain
	for ( i = 0; i < NSignals; i++ )
		if ( H(i)->PhysicalMax <= H(i)->PhysicalMin ||
		     H(i)->DigitalMax  <= H(i)->DigitalMin  ) {
			status |= AGH_EDFCHK_NOGAIN;
			retval = -2;
		} else
			H(i)->Scale =
				(H(i)->PhysicalMax - H(i)->PhysicalMin) /
				(H(i)->DigitalMax  - H(i)->DigitalMin );


      // parse label as 'SignalType Channel'
	for ( i = 0; i < NSignals; i++ ) {
		p = strpbrk( H(i)->Label, " ");
		if ( p ) {
			*p = '\0';
			if ( follows_Kemp( H(i)->Label) )
				H(i)->SignalType = g_strdup( H(i)->Label);
			else {
				H(i)->SignalType = g_strdup( "unknown-type");
				status |= AGH_EDFCHK_NONKEMP_SIGNALTYPE;
			}

			gchar *pp = p + strspn( p+1, " ") + 1;
			if ( follows_1020( pp) )
				H(i)->Channel = g_strdup( pp);
			else {
				H(i)->Channel = g_strdup_printf( "%d:<%s>", i, pp);
				status |= AGH_EDFCHK_NON1020_CHANNEL;
			}
		} else {
			H(i)->SignalType = g_strdup( "no-type");
			status |= AGH_EDFCHK_NONKEMP_SIGNALTYPE;
			H(i)->Channel = g_strdup_printf( "%d:[%s]", i, H(i)->Label);
			status |= AGH_EDFCHK_NOCHANNEL;
		}
	}

      // convenience field
	TotalRecSamples = 0;
	for ( guint i = 0; i < NSignals; i++ )
		TotalRecSamples += H(i)->SamplesPerRecord;


      // do Channels follow 1020?
	for ( i = 0; i < NSignals; i++ ) {
		for ( guint h = 0; h < AGH_KNOWN_CHANNELS_TOTAL; h++ )
			if ( H(i)->Channel &&
			     !strcmp( H(i)->Channel,
				      __agh_System1020_channels[h]) ) // found
				goto found_channel;
		status |= AGH_EDFCHK_NON1020_CHANNEL;
	found_channel:
		;
	}

      // are channels unique?
	for ( i = 0; i < NSignals; i++ )
		for ( guint j = 0; j < NSignals; j++ )
			if ( j != i && H(i)->Channel && H(j)->Channel &&
			     !strcmp( H(j)->Channel,
				      H(i)->Channel) ) {
				status |= AGH_EDFCHK_DUP_CHANNELS;
				break;
			}

skip_signal_hdr_checks:
	close( fd);

	return retval;
}







void
CEDFFile::dump( GString *out)
{
	gboolean output_requested = (out != NULL);
	if ( !output_requested )
		out = g_string_sized_new( 400);

	if ( status & AGH_EDFCHK_BAD_HEADER )
		g_string_printf( out, "Bad header, or no file\n");
	else {
		g_string_printf( out,
				 "File\t: %s\n"
				 "PatientID\t: %s\n"
				 "Session\t: %s,   Episode: %s\n"
				 "Timestamp\t: %s\n"
				 "# of signals\t: %d\n"
				 "# of records\t: %d\n"
				 "Record length\t: %d sec\n",
				 _filename->str,
				 PatientID_raw,
				 Session ?Session :"(unspecified)", Episode,
				 asctime( &timestamp_struct),
				 NSignals,
				 NDataRecords,
				 DataRecordSize);
		for ( guint i = 0; i < NSignals; i++ )
			g_string_append_printf( out,
						"Signal %d: Type: %s Channel: %s\n"
						"  Transducer type\t: %s\n"
						"  Physical dimension\t: %s\n"
						"  Physical min\t: %g\n"
						"  Physical max\t: %g\n"
						"  Digital min\t: %d\n"
						"  Digital max\t: %d\n"
						"  Filtering info\t: %s\n"
						"  Samples/rec\t: %d\n"
						"  (reserved)\t: %s\n",
						i, H(i)->SignalType, H(i)->Channel? H(i)->Channel :"(not specified)",
						H(i)->TransducerType,
						H(i)->PhysicalDim,
						H(i)->PhysicalMin,
						H(i)->PhysicalMax,
						H(i)->DigitalMin,
						H(i)->DigitalMax,
						H(i)->FilteringInfo,
						H(i)->SamplesPerRecord,
						H(i)->Reserved);
		printf( out->str);
	}

	if ( !output_requested )
		g_string_free( out, TRUE);
}








gint
CEDFFile::ship_data_raw( GArray *buffer, gint sig_no, gulong from, gulong length, gboolean swapbytes)
{
	gint retval = -1;

	if ( status & (AGH_EDFCHK_BAD_HEADER | AGH_EDFCHK_BAD_VERSION) ||
	     sig_no >= NSignals )
		return -2;

	gint fd = open( _filename->str, O_RDONLY);
	if ( fd == -1 )
		return -1;
	if ( lseek( fd, 256 + (NSignals * 256), SEEK_SET) == -1) {
		close( fd);
		return -3;
	}

	gulong n_samples = (length / DataRecordSize + 1) * H(sig_no)->SamplesPerRecord;
	if ( buffer->len < n_samples )
		g_array_set_size( buffer, n_samples);


	gulong	rec = from / DataRecordSize,
		rec_end = (from + length) / DataRecordSize;

	lseek( fd, rec * TotalRecSamples * sizeof(gint16), SEEK_CUR);

	gint16* p = (gint16*)buffer->data;
	for ( ; rec < rec_end; rec++ )
		for ( guint i = 0; i < NSignals; i++ ) {
			guint chunk = H(i)->SamplesPerRecord * sizeof(gint16);
			if ( i == sig_no ) {
				if ( read( fd, p, chunk) != chunk ) { retval = -1; printf( "short read at rec %u\n", rec); goto out; }
				p += chunk/sizeof(gint16);
			} else
				if ( lseek( fd, chunk, SEEK_CUR) == -1) { retval = -1; goto out; }
		}


	if ( swapbytes )
		for ( gulong i = 0; i < n_samples; i += 2 ) {
			guint8 b = *(guint8*)(buffer->data);
			*(guint8*)(buffer->data) =
				*(guint8*)(buffer->data+1);
			*(guint8*)(buffer->data+1) = b;

		}
		retval = 0;

out:
	close( fd);
	return retval;
}





gint
CEDFFile::ship_data( GArray *buffer_double, gint sig_no, gulong from, gulong length, gboolean swapbytes)
{
	static GArray *buffer_int16 = NULL;
	if ( !buffer_int16 )
		buffer_int16 = g_array_new( FALSE, FALSE, sizeof(gint16));

	gint retval = ship_data_raw( buffer_int16, sig_no, from, length, swapbytes);
	if ( retval ) {
		g_array_free( buffer_int16, TRUE);
		g_array_set_size( buffer_double, 0);
		return retval;
	}

	if ( buffer_double->len < buffer_int16->len )  // avoid shrinking custom-size buffer size
						       // set by caller CBinnedPower::obtain_power
		g_array_set_size( buffer_double, buffer_int16->len);

	gdouble scale = H(sig_no)->Scale;
	for ( gulong i = 0; i < buffer_int16->len; i++ )
		g_array_index (buffer_double, gdouble, i) =
			g_array_index (buffer_int16, gint16, i) * scale;

	return 0;
}

// EOF
