// ;-*-C++-*- *  Time-stamp: "2008-07-02 01:02:57 hmmr"

/***************************************************************************
 *            agh_io.cc
 *
 *  Copyright  2008  Andrei Zavada
 *  Email johnhommer@gmail.com

 *
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <stdio.h>

#include <glib.h>

#include "agh_expdesign.hh"
#include "agh_model.hh"




const gchar*
CMeasurement::make_fname() // provisionally useful (saves a call to
			   // make_phony); still, higly contentious
			   // against its namesake in CExpDesign
{
	static GString *fname = NULL;
	if ( !fname )
		fname = g_string_sized_new( 180);
	g_string_printf( fname, "%s/H:%s G:%d B:%g W:%c%d.power",
			 _basename->str,
			 _channel->name(),
			 _pagesize, fft_params.bin_size,
			 'a' + fft_params.window_type, 2 << fft_params.window_size_2power);
	return fname->str;
}


const gchar*
CMeasurement::make_fname_track()
{
	static GString *fname = NULL;
	if ( !fname )
		fname = g_string_sized_new( 180);
	g_string_printf( fname, "%s/H:%s.track",
			 _basename->str,
			 _channel->name() /*,
					    _pagesize */ );
	return fname->str;
}





const gchar*
CSimulation::make_fname()
{
	static GString *fname = NULL;
	if ( !fname )
		fname = g_string_sized_new( 180);

	g_string_printf( fname,
			 "%s/"
			 "B:%g W:%c%d H:%s F:%g-%g G:%d DB1:%s DB2:%s AZ:%s "
			 "R:%s%s%s%s%s%s%s%s%s.S",
			 _basename->str,

			 fft_params.bin_size,
			 'a' + fft_params.window_type, 2 << fft_params.window_size_2power,
			 _channel->name(),
			 freq_from, freq_upto, pagesize,
			 yesno( DBAmendment1), yesno( DBAmendment2), yesno( AZAmendment),
			 (step.pidx[AGH_Vgc  ] > 0.)?"g" :"",
			 (step.pidx[AGH_Vrs  ] > 0.)?"r" :"",
			 (step.pidx[AGH_Vrc  ] > 0.)?"c" :"",
			 (step.pidx[AGH_Vfc_R] > 0.)?"R" :"",
			 (step.pidx[AGH_Vfc_W] > 0.)?"W" :"",
			 (step.pidx[AGH_VS_0 ] > 0.)?"0" :"",
			 (step.pidx[AGH_VS_U ] > 0.)?"U" :"",
			 (step.pidx[AGH_Vt_a ] > 0.)?"a" :"",
			 (step.pidx[AGH_Vt_p ] > 0.)?"p" :"");

//	printf( "%s\n", fname->str);
	return fname->str;
}








gint
CBinnedPower::save( const gchar *fname, gboolean binary)
{
	guint n_bins = _samplerate/2/fft_params.bin_size + 1;

	if ( binary ) {
		int fd = open( fname, O_RDWR | O_CREAT | O_TRUNC, 0644);
		if ( fd == -1 )
			return -1;
		write( fd, &_samplerate, sizeof(gint));
		write( fd, _data, sizeof(double) * _pages * n_bins);
		close( fd);
	} else {
		FILE *f = fopen( fname, "w");
		if ( !f )
			return -1;
		fprintf( f, "%d\n", _samplerate);
		for ( guint p = 0; p < _pages; p++ )
			for ( guint bin = 0; bin < n_bins; bin++ )
				fprintf( f, "%g%s", ijth_bin( bin, p), (bin == n_bins-1) ?"\n" :"\t");
		fclose( f);
	}

	return 0;
}


gint
CBinnedPower::save_track( const gchar *fname, gboolean binary)
{
	if ( binary ) {
		int fd = open( fname, O_RDWR | O_CREAT | O_TRUNC, 0644);
		if ( fd == -1 )
			return -1;
		write( fd, _track, sizeof(gchar) * _pages * _pagesize);
		close( fd);
	} else {
		FILE *f = fopen( fname, "w");
		if ( !f )
			return -1;
		guint len_sec = _pagesize * _pages;
		for ( guint s = 0; s < len_sec; s++ )
			fprintf( f, "%c%s", nth_second(s)+'0', (s && s % _pagesize == 0) ?"\n" :"");
		fclose( f);
	}

	return 0;
}




gint
CBinnedPower::load( const gchar *fname, gboolean binary)
{
	if ( binary ) {
		int fd = open( fname, O_RDONLY);
		if ( fd != -1 ) {
			read( fd, &_samplerate, sizeof(gint));
			guint n_bins = _samplerate/2/fft_params.bin_size;
			struct stat f_stat;
			stat( fname, &f_stat);
			off_t len = f_stat.st_size - sizeof(gint);
			set_size( len/n_bins/sizeof(double));
			read( fd, _data, len);
			close( fd);

			status |= (AGH_BP_OBTAINED | AGH_BP_INMEM);
		} else
			status &= ~(AGH_BP_OBTAINED | AGH_BP_INMEM);
	} else {
		FILE *f = fopen( fname, "r");
		if ( !f )
			return -1;
		fscanf( f, "%d", &_samplerate);
		guint n_bins = _samplerate/2/fft_params.bin_size + 1;
		guint p = 0;
		while ( !feof( f) ) {
			if ( p >= _pages )
				set_size( _pages+20);
			for ( guint bin = 0; bin < n_bins; bin++ )
				fscanf( f, "%g ", &ijth_bin( bin, p));
			p++;
		}
		set_size( p+1);
		fclose( f);
	}
	return 0;
}





gint
CBinnedPower::load_track( const gchar *fname, gboolean binary)
{
	if ( binary ) {
		int fd = open( fname, O_RDONLY);
		if ( fd != -1 ) {
			read( fd, _track, _pages * _pagesize * sizeof(gchar));
			close( fd);
			status |= AGH_BP_HAS_TRACK;
		} else {
			status &= ~AGH_BP_HAS_TRACK;
			return -1;
		}
	} else {
		FILE *f = fopen( fname, "r");
		if ( f ) {
			gchar c;
			for ( guint s = 0; s < _pages * _pagesize && !feof( f); s++ ) {
				fscanf( f, "%c", &c);
				nth_second(s) = c-'0';
			}
			fclose( f);
			status |= AGH_BP_HAS_TRACK;
		} else {
			status &= ~AGH_BP_HAS_TRACK;
			return -1;
		}
	}
	return 0;
}











gint
CHypnogram::save()
{
	if ( !length() )
		return 0;

	FILE *f = fopen( _filename->str, "w");
	if ( !f )
		return -1;

	fprintf( f, "%d\n", _pagesize);
	for ( guint p = 0; p < length(); p++ )
		fprintf( f, "%g\t%g\t%g\n", nth_page(p).NREM, nth_page(p).REM, nth_page(p).Wake);

	fclose( f);

	return 0;
}


gint
CHypnogram::load()
{
	FILE *f = fopen( _filename->str, "r");
	if ( !f )
		return -1;

	fscanf( f, "%d\n", &_pagesize);
	guint p = 0;
	SPage	P;
	while ( fscanf( f, "%g %g %g\n", &P.NREM, &P.REM, &P.Wake) == 3 ) {
		if ( p >= length() )
			set_size( p+20);
		nth_page(p++) = P;
	}
	set_size( p);

	fclose( f);

	return 0;
}






gint
CHypnogram::save_canonical( const gchar *fname)
{
	if ( !length() )
		return 0;

	FILE *f = fopen( fname, "w");
	if ( !f )
		return -1;

	for ( guint p = 0; p < length(); p++ ) {
		gfloat	N = nth_page(p).NREM,
			R = nth_page(p).REM,
			W = nth_page(p).Wake;
		fprintf( f, "%s\n",
			 N > .7 ?"NREM4" :N > .4 ?"NREM3"
			 :R > .5 ?"REM"
			 :W > .5 ?"Wake"
			 :N > .2 ?"NREM2"
			 :N > .01 ?"NREM1"
			 :"(unscored)");
	}
	fclose( f);

	return 0;
}






gchar agh_score_codes[] = {
	'\0', '1', '2', '3', '4', '0', '5', '6',
};



gint
CHypnogram::load_canonical( const gchar *fname)
{
	FILE *f = fopen( fname, "r");
	if ( !f )
		return -1;

	guint p = 0;
	gchar token[19];
	while ( !feof (f) ) {
		SPage	P = { 0., 0., 0. };
		fscanf( f, "%18s", token);
		if ( !strcasecmp( token, "Wake") ||
		     !strcasecmp( token, "W") ||
		     (strlen( token) == 1 && *token == agh_score_codes[AGH_SCORE_WAKE]) )
			P.Wake = 1.;
		else
		if ( !strcasecmp( token, "NREM1") ||
		     !strcasecmp( token, "N1") ||
		     (strlen( token) == 1 && *token == agh_score_codes[AGH_SCORE_NREM1]) )
			P.NREM = .25;
		else
		if ( !strcasecmp( token, "NREM2") ||
		     !strcasecmp( token, "N2") ||
		     (strlen( token) == 1 && *token == agh_score_codes[AGH_SCORE_NREM2]) )
			P.NREM = .5;
		else
		if ( !strcasecmp( token, "NREM3") ||
		     !strcasecmp( token, "N3") ||
		     (strlen( token) == 1 && *token == agh_score_codes[AGH_SCORE_NREM3]) )
			P.NREM = .75;
		else
		if ( !strcasecmp( token, "NREM4") ||
		     !strcasecmp( token, "N4") ||
		     (strlen( token) == 1 && *token == agh_score_codes[AGH_SCORE_NREM4]) )
			P.NREM = 1.;
		else
		if ( !strcasecmp( token, "REM") ||
		     !strcasecmp( token, "R") ||
		     (strlen( token) == 1 && *token == agh_score_codes[AGH_SCORE_REM]) )
			P.REM = 1.;
		else
		if ( !strcasecmp( token, "MVT") ||
		     !strcasecmp( token, "M") ||
		     (strlen( token) == 1 && *token == agh_score_codes[AGH_SCORE_MVT]) )
			;
		else
		if ( !strcasecmp( token, "-") ||
		     !strcasecmp( token, "9") ||
		     !strcasecmp( token, "unscored") ||
		     (strlen( token) == 1 && *token == agh_score_codes[AGH_SCORE_NONE]) )
			;
		else {
			continue;
		}

		if ( p >= length() )
			set_size( p+20);

		nth_page(p++) = P;
	}
	fclose( f);
	set_size( p+1);

	return 0;
}








gint
CModelRun::save( const gchar *fname, gboolean binary)
{
	if ( !(status & AGH_MODRUN_INMEM) )  // prevent rubbish written and read
		return 0;

	if ( binary ) {
		int fd = open( fname, O_RDWR | O_CREAT | O_TRUNC, 0644);
		if ( fd == -1 )
			return -1;

		write( fd, &vector->len, sizeof(guint));
		write( fd, vector->data, sizeof(SPageSimulated) * vector->len);  // always unnecessary to load SWA, but
										 // one call to write is faster than a
										 // loop to pick specifically S & SWA_sim
		write( fd, &status, sizeof(status));
		write( fd, (SControlParamSet*)this, sizeof(SControlParamSet));
		write( fd, (STunableSetFull*) this, sizeof(STunableSetFull));
		write( fd, &_fit, sizeof(_fit));
		write( fd, &_dfit, sizeof(_dfit));
		write( fd, &tunables0, sizeof(STunableSet));

		write( fd, &_stride, sizeof(_stride));
		write( fd, _t_stack->data, sizeof(STunableSetIndexable) * _stride);

		close( fd);

	} else {
//		FILE *f = fopen( fname, "w");
//		if ( !f )
//			return -1;
//		for ( guint p = 0; p < vector->len; p++ ) {
//			SPageSimulated &P = g_array_index (vector, SPageSimulated, p);
//			fprintf( f, "%g\t%g\t%g\t%g\t%g\n",
//				 P.NREM, P.REM, P.Wake, P.SWA, P.S);
//		}
//		fclose( f);
		fprintf( stderr, "Not implemented\n");
	}

	return 0;
}





gint
CModelRun::load( const gchar *fname, gboolean binary)
{
	if ( binary ) {
		int fd = open( fname, O_RDONLY);
		if ( fd == -1 )
			return -1;

		guint vector_length;
		read( fd, &vector_length, sizeof(guint));
		set_size( vector_length);

		read( fd, vector->data, vector_length * sizeof(SPageSimulated));

		read( fd, &status, sizeof(status));
		read( fd, (SControlParamSet*)this, sizeof(SControlParamSet));
		read( fd, (STunableSetFull*) this, sizeof(STunableSetFull));
		read( fd, &_fit, sizeof(_fit));
		read( fd, &_dfit, sizeof(_dfit));
		read( fd, &tunables0, sizeof(STunableSet));

		read( fd, &_stride, sizeof(_stride));
		g_array_set_size( _t_stack, _stride);
		read( fd, _t_stack->data, sizeof(STunableSetIndexable) * _stride);

		g_ptr_array_set_size( _swa_sim_stack, _stride);
		g_ptr_array_set_size( _s_stack,       _stride);
		for ( guint n = 0; n < _stride; n++ )
			nth_swa_sim_course(n) = nth_s_course(n) = NULL;

		close( fd);
	} else {
//		FILE *f = fopen( fname, "r");
//		if ( !f )
//			return -1;
//		guint p = 0;
//		while ( !feof( f) ) {
//			if ( p >= vector->len )
//				set_size( vector->len+20);
//			SPageSimulated &P = nth_page(p);
//			fscanf( f, "%g %g %g %g %g",
//				 P.NREM, P.REM, P.Wake, P.SWA, P.S);
//			p++;
//		}
//		set_size( p+1);
//		fclose( f);
	}

	return 0;
}



// EOF
