// ;-*-C++-*- *  Time-stamp: "2008-07-01 18:16:53 hmmr"

/***************************************************************************
 *            agh_io.hh
 *
 *  Copyright  2008  Andrei Zavada
 *  Email johnhommer@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _AGH_IO_H
#define _AGH_IO_H


#if HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <time.h>

#include "z_lists.hh"
#include "agh_enums.h"
#include "agh_data.hh"





#define AGH_EDFCHK_BAD_HEADER		(1 <<  0)
#define AGH_EDFCHK_BAD_VERSION		(1 <<  1)
#define AGH_EDFCHK_BAD_NUMFLD		(1 <<  2)
#define AGH_EDFCHK_BAD_RECORDING	(1 <<  3)
#define AGH_EDFCHK_DATE_UNPARSABLE	(1 <<  4)
#define AGH_EDFCHK_TIME_UNPARSABLE	(1 <<  5)
#define AGH_EDFCHK_NOSESSION		(1 <<  6)
#define AGH_EDFCHK_NOEPISODE		(1 <<  7)
#define AGH_EDFCHK_NONKEMP_SIGNALTYPE	(1 <<  8)
#define AGH_EDFCHK_NON1020_CHANNEL	(1 <<  9)
#define AGH_EDFCHK_NOCHANNEL		(1 << 10)
#define AGH_EDFCHK_DUP_CHANNELS		(1 << 11)
#define AGH_EDFCHK_NOGAIN		(1 << 12)

struct SSignal {
	gchar	Label               [16+1],  // maps to channel
		TransducerType      [80+1],
		PhysicalDim         [ 8+1],
		PhysicalMin_raw     [ 8+1],
		PhysicalMax_raw     [ 8+1],
		DigitalMin_raw      [ 8+1],
		DigitalMax_raw      [ 8+1],
		FilteringInfo       [80+1],
		SamplesPerRecord_raw[ 8+1],
		Reserved            [32+1];

      // these are now dynamically allocated: ~CEDFFile will free them
	gchar	*SignalType,
		*Channel;

	gint	DigitalMin,
		DigitalMax;
	gfloat	PhysicalMin,
		PhysicalMax,
		Scale;

	guint	SamplesPerRecord;
};





class CMeasurement;

class CEDFFile
  : public CHypnogram {

    friend class CMeasurement;

      // variable fields
	GArray *_signal_data;

	GString *_filename;

	gint read_header();

    public:
	gint status;
	TSourceAttachMode attach_mode;
	gchar *filename()	{ return CEDFFile::_filename->str; }

      // static fields (raw)
	gchar	VersionNumber_raw    [ 8+1],
		PatientID_raw        [80+1],  // maps to subject name
		RecordingID_raw      [80+1],  // maps to episode_name (session_name)
		RecordingDate_raw    [ 8+1],
		RecordingTime_raw    [ 8+1],
		HeaderLength_raw     [ 8+1],
		Reserved_raw         [44+1],
		NDataRecords_raw     [ 8+1],
		DataRecordSize_raw   [ 8+1],
		NSignals_raw         [ 4+1];
      // (relevant converted integers)
	guint	HeaderLength,
		NDataRecords,
		DataRecordSize,
		NSignals;
	struct tm timestamp_struct;
	time_t	timestamp;

      // RecordingID_raw parsed into episode_name and session_name
      // These will point right into corresponding fields above;
      // read_header() will have taken care to put NUL chars as appropriate
	gchar  *Session,
	       *Episode;

	CEDFFile()			   { _signal_data = g_array_new( FALSE, TRUE, sizeof(SSignal));
					     CEDFFile::_filename = g_string_sized_new( 40);             }
	CEDFFile( const gchar *fname)
	  : CHypnogram( fname)  // will do s/.edf/.hypnogram/ -- smart!
					   { _signal_data = g_array_new( FALSE, TRUE, sizeof(SSignal));
					     CEDFFile::_filename = g_string_new( fname);
					     read_header();                                             }

       ~CEDFFile();

	void make_phony( const gchar *fname)        { g_string_assign( CEDFFile::_filename, fname);
						      read_header(); }
	gboolean operator == ( CEDFFile &o) { return !strcmp( PatientID_raw, o.PatientID_raw) &&
						     !strcmp( RecordingID_raw, o.RecordingID_raw); }

	SSignal *H( guint i)   { return &g_array_index( _signal_data, SSignal, i); }
	SSignal &operator [] ( guint i)    { return  g_array_index( _signal_data, SSignal, i); }

	gint sig_no_by_channel_name( const gchar *ch);

	gulong length()   { return NDataRecords * DataRecordSize; }

	gint assimilate( const gchar *where_to, TSourceAttachMode option = AGH_SRCATTACH_COPY);

	gint ship_data( GArray *buffer, gint signal_no, gulong from, gulong len, gboolean swapbytes = FALSE);
	gint ship_data_raw( GArray *buffer, gint signal_no, gulong from, gulong len, gboolean swapbytes = FALSE);

	void dump( GString *out = NULL);

    private:
	guint TotalRecSamples;
};



#endif

// EOF
