// ;-*-C++-*- *  Time-stamp: "2008-07-01 00:47:58 hmmr"

/***************************************************************************
 *            agh_model.cc
 *
 *  Copyright  2008  Andrei Zavada
 *  Email johnhommer@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <stdlib.h>

#include "agh_expdesign.hh"
#include "agh_io.hh"
#include "agh_model.hh"




const gchar* const agh_v_names[] = {
	"rs",   "rc",
	"fc_R", "fc_W",
	"S_0",  "S_U",
	"t_a",  "t_p",
	"gc",
	"gc1", "gc2", "gc3", "gc4", "gc5",
};

const gchar* const agh_v_formats[] = {
	"%5.3f", "%5.3f",
	"%4.2f", "%4.2f",
	"%5.1f", "%5.1f",
	"%3.1f", "%3.1f",
	"%5.3f", "%5.3f",
	"%5.3f", "%5.3f",
	"%5.3f", "%5.3f",
};

const guint agh_v_dispf[] = {
	1e3, 1,
	1, 1,
	1, 1,
	1, 1,
	1e2, 1e2,
	1e2, 1e2,
	1e2, 1e2,
};

const gdouble agh_v_defaults[] = {
	.918e-3,	.283,
	.24,		1,
	300,		500,
	3,		3,
	.835e-2,	.835e-2,
	.835e-2,	.835e-2,
	.835e-2,	.835e-2,
};


const gdouble agh_v_min[] = {
	.100e-3,	.01,
	.04,	.04,
	9,	18,
	.4,	.4,
	.010e-2,	.010e-2,
	.010e-2,	.010e-2,
	.010e-2,	.010e-2,
};

const gdouble agh_v_max[] = {
	2.000e-3,	4.,
	4.,	4.,
	1005,	2010,
	22,	22,
	4.000e-2,	4.000e-2,
	4.000e-2,	4.000e-2,
	4.000e-2,	4.000e-2,
};

const gdouble agh_v_step[] = {
	.001e-3,	.002,
	.001,		.001,
	1,		1,
	0,		0,
	.004e-2,	.004e-2,
	.004e-2,	.004e-2,
	.004e-2,	.004e-2,
};




void
STunableSet::assign_defaults()
{
	rs   = agh_v_defaults[AGH_Vrs];
	rc   = agh_v_defaults[AGH_Vrc];
	fc_R = agh_v_defaults[AGH_Vfc_R];
	fc_W = agh_v_defaults[AGH_Vfc_W];
	S_0  = agh_v_defaults[AGH_VS_0];
	S_U  = agh_v_defaults[AGH_VS_U];
	t_a  = agh_v_defaults[AGH_Vt_a];
	t_p  = agh_v_defaults[AGH_Vt_p];
	gc   = agh_v_defaults[AGH_Vgc];
	for ( guint t = 0; t < AGH_EGC_MAX-1; t++ )
		egc[t] = agh_v_defaults[AGH_Vgc];
}

gboolean
STunableSet::check_sane()
{
//	FIX
	for ( guint t = 0; t < AGH_EGC_MAX-1; t++ )
		if ( egc[t] < agh_v_min[AGH_Vgc] || egc[t] > agh_v_max[AGH_Vgc] )
			return FALSE;
	return TRUE;
}




void
STunableSetFull::assign_defaults()
{
	value.pname.assign_defaults();

	guint t;
	for ( t = 0; t < AGH_VNUM + AGH_EGC_MAX-1; t++ )
		min.pidx[t] = agh_v_min[t], max.pidx[t] = agh_v_max[t], step.pidx[t] = agh_v_step[t];

	flags[AGH_Vrs  ] = 0,		    flags[AGH_Vrc ]  = AGH_T_REQUIRED;
	flags[AGH_Vfc_R] = AGH_T_REQUIRED,  flags[AGH_Vfc_W] = AGH_T_REQUIRED;
	flags[AGH_VS_0 ] = AGH_T_REQUIRED,  flags[AGH_VS_U ] = AGH_T_REQUIRED;
	flags[AGH_Vt_a ] = 0,		    flags[AGH_Vt_p ] = 0;
	flags[AGH_Vgc  ] = AGH_T_REQUIRED;
	for ( t = AGH_VNUM; t < AGH_VNUM + AGH_EGC_MAX-1; t++ )
		flags[t] = flags[AGH_Vgc];
}


gboolean
STunableSetFull::check_consisitent()
{
	if ( !(value.pname.check_sane() && min.pname.check_sane() && max.pname.check_sane()) )
		return FALSE;

	for ( guint t = 0; t < AGH_VNUM + AGH_EGC_MAX-1; t++ )
		if ( min.pidx[t] >= max.pidx[t] || step.pidx[t] >= (max.pidx[t] - min.pidx[t])/2 )
			return FALSE;
	return TRUE;
}






gboolean
SControlParamSet::check_sane()
{
	return	InitRand > 0 && InitRand < 1 &&
		MinSuffFitImprov > 0 && MinSuffFitImprov <= 5 &&
		MaxAllowedParamDrift >= 2 && MaxAllowedParamDrift <= 100 &&
		StrideLength >= 40 && StrideLength <= 5000 &&
		MaxStrides > 0 && MaxStrides <= 5000 &&
		RetriesoOnSimCycleFail >= 0 && RetriesoOnSimCycleFail <= 10 &&
		MaxHalvingTimes >= 0 && MaxHalvingTimes < 10 &&
		TeleportDistance >= 1 && TeleportDistance < 500;
}


void
SControlParamSet::assign_defaults()
{
	InitRand = .5,			MinSuffFitImprov = .05;
	MaxAllowedParamDrift = 20,	StrideLength = 400;
	MaxStrides = 100,		RetriesoOnSimCycleFail = 1;
	TeleportDistance = 50,		MaxHalvingTimes = 3;
}









void
SFFTParamsSet::dump()
{
	printf( "window_size: %d sec, type: %c\n"
		"   bin_size: %g Hz\n", 2 << window_size_2power, 'a' + window_type, bin_size);
}


void
STunableSetFull::dump()
{
	gchar strbuf[200];
	for ( guint t = 0; t < AGH_VNUM + AGH_EGC_MAX-1; t++ ) {
		snprintf( strbuf, 200,
			  "%8s: val = %s\t min = %10s\t max = %10s\t step = %10s;\t  req = %s\n",
			  agh_v_names[t], agh_v_formats[t], agh_v_formats[t], agh_v_formats[t], agh_v_formats[t], yesno( flags[t] & AGH_T_REQUIRED));
		printf( strbuf,
			value.pidx[t] * agh_v_dispf[t], min.pidx[t] * agh_v_dispf[t], max.pidx[t] * agh_v_dispf[t], step.pidx[t] * agh_v_dispf[t]);
	}
}









guint
CSCourse::nth_msmt_start_p( guint n)
{
	return difftime( M_by_idx(n)->_source->timestamp,
			 M_by_idx(0)->_source->timestamp) / pagesize;
}

guint
CSCourse::nth_msmt_end_p( guint n)
{
	return nth_msmt_start_p(n)
		+ (M_by_idx(n)->_source->NDataRecords * M_by_idx(n)->_source->DataRecordSize)
		/ pagesize;
}



gint
CSCourse::add_measurement( CMeasurement *M,
			   SFFTParamsSet &ifft_params,
			   gint smooth, gfloat glitch_mag)
{
	if ( !M )
		return AGH_SIMPREP_ENOMSMT;

	if ( measurements->len >= AGH_EGC_MAX )
		return AGH_SIMPREP_ETOOMANYMSMT;

	CEDFFile *F = M->source();
	if ( !F || strcmp( F->H(agh_current_channel)->SignalType, "EEG") )
		return AGH_SIMPREP_ESIGTYPE;

	if ( !F->CHypnogram::length() )
		return AGH_SIMPREP_ENOSCORE;

	double offset = measurements->len ? difftime( F->timestamp, M_by_idx(0)->_source->timestamp) :0.;
	if ( offset > 4 * 24 * 3600 )
		return AGH_SIMPREP_EFARAPART;

//	M->dump();
//	printf( asctime( &F->timestamp_struct));
//	printf( "m = %d, offset = %d h\n", measurements->len, (guint)(offset/3600));

	g_ptr_array_add( measurements, M);

	GArray *buffer_pow = g_array_new( FALSE, FALSE, sizeof(double));
	M->ship_power_course( buffer_pow, NULL, pagesize, ifft_params,
			      freq_from, freq_upto,
			      smooth, glitch_mag,
			      FALSE);   // no artifact filtering

	guint	Fn_startpage = offset/pagesize,
		timeline_end = Fn_startpage + buffer_pow->len,
		old_timeline_end = vector->len;
	guint p;
//	printf( "F[%d]start = %d;  timeline_end = %d p (was %d)\n",
//		measurements->len-1, Fn_startpage, timeline_end, old_timeline_end);

      // fill the time between episodes with WAKE
	g_array_set_size( vector, timeline_end);
	for ( p = old_timeline_end; p < Fn_startpage; p++ )
		g_array_index (vector, SPageSimulated, p).Wake = 1.;

	for ( p = 1; p < buffer_pow->len; p++ ) {  // omit the first page with unexplainable SWA surge
		SPageSimulated &P = g_array_index (vector, SPageSimulated, p + Fn_startpage);
		P.NREM	= F->nth_page(p).NREM;                         // one for the scores
		P.REM	= F->nth_page(p).REM;
		P.Wake	= F->nth_page(p).Wake;
		P.SWA = g_array_index (buffer_pow, double, p);  // and one for the power
//		P.S   = 0.;
		_pages_with_SWA += P.has_swa();
	}

	if ( measurements->len == 1 ) {

		for ( p = 3; p < vector->len; p++ )
			if ( (g_array_index (vector, SPageSimulated, p  ).NREM +
			      g_array_index (vector, SPageSimulated, p-1).NREM +
			      g_array_index (vector, SPageSimulated, p-2).NREM) / 3 > 1./3 ) {
				_sim_start = p-2;
				_SWA_0 = g_array_index (vector, SPageSimulated, _sim_start).SWA;
				break;
			}

		_SWA_L = _SWA_100 = 0.;
		for ( p = _sim_start; p < vector->len; p++ ) {
			SPageSimulated &P = g_array_index (vector, SPageSimulated, p);
			if ( P.REM > .5 )
			     _SWA_L = MAX (_SWA_L, P.SWA);
			if ( P.NREM + P.REM > 1./3 )
				_SWA_100 += P.SWA;
		}
//		_SWA_L *= .95;
		_SWA_100 /= _pages_with_SWA;
		_SWA_L = .05 * _SWA_100;
//		printf( "avg SWA = %g (over %d pp);  SWA_L = %g;  SWA[%d] = %g\n",
//			_SWA_100, _pages_with_SWA, _SWA_L, _sim_start, _SWA_0);

		_baseline_end = buffer_pow->len;
//		_S_baseline_end = g_array_index (vector, SPageSimulated, _baseline_end).SWA;
	}

	return 0;
}






CModelRun::CModelRun( gfloat from, gfloat upto, guint ipagesize,
		      SFFTParamsSet    &ifft_params,
		      SControlParamSet &iparam_set,
		      STunableSetFull  &itunable_set)
    : CSCourse( from, upto, ipagesize, ifft_params)
{
	fft_params = ifft_params;
	*(SControlParamSet*)this = iparam_set;
	*(STunableSetFull*) this = itunable_set;
	randomize();
	tunables0 = itunable_set.value.pname;

	_stride = 0;
	_fit = 0.;
	_shadow  = g_array_new( FALSE, FALSE, sizeof(SPage));

	_t_stack = g_array_new( FALSE, FALSE, sizeof(STunableSetIndexable));
	_swa_sim_stack = g_ptr_array_new();
	_s_stack       = g_ptr_array_new();

	status = 0;
}


CModelRun::~CModelRun()
{
	if ( _shadow  ) g_array_free( _shadow,  TRUE);
	if ( _t_stack ) g_array_free( _t_stack, TRUE);

	if ( _swa_sim_stack ) {
		for ( guint n = 0; n < _swa_sim_stack->len; n++ ) {
			if ( nth_swa_sim_course(n) )
				g_array_free( nth_swa_sim_course(n), TRUE);
			if ( nth_s_course(n) )
				g_array_free( nth_s_course(n), TRUE);
		}
		g_ptr_array_free( _swa_sim_stack, TRUE);
		g_ptr_array_free( _s_stack, TRUE);
	}
}



void
CModelRun::randomize()
{
	for ( guint t = 0; t < AGH_VNUM; t++ )
		if ( step.pidx[t] > 0. ) {
			double iv;
			do  iv = value.pidx[t] * (double)(rand() - RAND_MAX/2)/RAND_MAX * InitRand;
			while ( value.pidx[t] + iv > max.pidx[t] || value.pidx[t] + iv < min.pidx[t] );
			value.pidx[t] += iv;
		}
}


CSimulation::CSimulation( CSubject *J, CRefList<CMeasurement> *D, COrderedName *H,
			  gfloat from, gfloat upto, guint ipagesize,
			  SFFTParamsSet    &ifft_params,
			  SControlParamSet &iparam_set,
			  STunableSetFull  &itunable_set)
    : CModelRun( from, upto, ipagesize,
		 ifft_params, iparam_set, itunable_set)
{
	_subject = J, _session = D, _channel = H;
	_basename = g_string_sized_new( 120);
	g_string_printf( _basename,
			 "%s/%s/%s/SIMULATIONS",
			 _subject->belongs->name(), _subject->name(), _session->name());

	g_mkdir_with_parents( _basename->str, 0755);
}








gint
CExpDesign::setup_modrun( CSubject *J, CRefList<CMeasurement> *D, COrderedName *H,
			  gfloat from, gfloat upto,
			  CSimulation **S_p)
{
	CSimulation *Sy = new CSimulation,
		    *S;
	Sy->make_phony( J, D, H,
			agh_sim_operating_range_from, agh_sim_operating_range_upto,
			pagesize,
			fft_params.window_size_2power, fft_params.bin_size, fft_params.window_type);
	if ( !(S = simulations->exist( Sy)) ) {
		simulations->add( S = new CSimulation( J, D, H,
						       from, upto, pagesize,
						       fft_params,
						       control_params, tunables));

		gint retval;
		if ( episode_names->n() == 1 && control_params.DBAmendment2 )
			return AGH_SIMPREP_EAMENDMENTS_INEFFECTIVE;

		if ( episode_names->n() == 1 && tunables.step.pname.rs > 0. )
			return AGH_SIMPREP_ERS_NONSENSICAL;

		for ( guint e = 0; e < episode_names->n(); e++ ) {
			retval = S->add_measurement( M_by_EJHD( episode_names->nth(e),
								J, H, D),
						     fft_params, agh_smoothover, agh_glitch_mag);
			if ( retval ) {
				simulations->del( S);
				if ( S_p )  *S_p = NULL;
				return retval;
			}
		}
	}

	if ( S->status | ~AGH_MODRUN_INMEM )
		S->load( S->make_fname());  // load SWA_sim and S and tunables, if they exist

	if ( S_p )
		*S_p = S;

	return 0;
}


void
CExpDesign::reset_modrun( CModelRun *R)
{
	R->fft_params = fft_params;
	*(SControlParamSet*)R = control_params;
	*(STunableSetFull*)R  = tunables;
	R->randomize();

	R->_fit = 0.;

	if ( R->_swa_sim_stack->len )
		for ( guint n = 0; n < R->_swa_sim_stack->len; n++ ) {
			GArray* &A = R->nth_swa_sim_course(n);
			GArray* &B = R->nth_s_course(n);
			if ( A )  g_array_free( A, TRUE), A = NULL;
			if ( B )  g_array_free( B, TRUE), B = NULL;
		}

	R->status  = 0;
	R->_stride = 0;
}




// EOF
