// ;-*-C++-*- *  Time-stamp: "2008-06-27 15:12:54 hmmr"

/***************************************************************************
 *            agh_model.hh
 *
 *  Copyright  2008  Andrei Zavada
 *  Email johnhommer@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _AGH_MODEL_H
#define _AGH_MODEL_H


#if HAVE_CONFIG_H
#  include <config.h>
#endif



#include <glib.h>

#include "z_lists.hh"
#include "agh_enums.h"

#include "agh_data.hh"


inline const gchar *yesno( gboolean val)  { return val ?"yes" :"no"; }






class CMeasurement;

class CSCourse {

    friend class CModelRun;

    public:
	guint	_sim_start,
		_baseline_end,
		_pages_with_SWA;
	gdouble	_SWA_L,
		_SWA_0;

	gdouble	_SWA_100;
	gfloat	freq_from, freq_upto;
	guint	pagesize;
	SFFTParamsSet fft_params;

	GPtrArray *measurements;

	GArray	  *vector;

	CSCourse()                  { measurements = NULL, vector = NULL; }

	CSCourse( gfloat from, gfloat upto,
		  guint ipagesize,
		  SFFTParamsSet &ifft_params)	{ freq_from = from, freq_upto = upto, pagesize = ipagesize;
						  fft_params = ifft_params;
						  measurements = g_ptr_array_sized_new( 4);
						  vector = g_array_new( FALSE, TRUE, sizeof(SPageSimulated));
						  _pages_with_SWA = 0;  }

       ~CSCourse()				{ if ( vector )        g_array_free( vector, TRUE);
						  if ( measurements )  g_ptr_array_free( measurements, TRUE); }

	SPageSimulated &nth_page( guint p)	{ return g_array_index (vector, SPageSimulated, p); }
	void set_size( guint sz)		{ g_array_set_size( vector, sz);         }

	gint add_measurement( CMeasurement*, SFFTParamsSet&,
			      gint smooth, gfloat glitch_mag);

	CMeasurement *M_by_idx( guint m)         { return (CMeasurement*) g_ptr_array_index (measurements, m); }
	guint nth_msmt_start_p( guint n);
	guint nth_msmt_end_p( guint n);
};








G_BEGIN_DECLS

extern const gchar* const agh_v_names	[AGH_VNUM + AGH_EGC_MAX-1];
extern const gchar* const agh_v_formats	[AGH_VNUM + AGH_EGC_MAX-1];
extern const guint  agh_v_dispf		[AGH_VNUM + AGH_EGC_MAX-1];
extern const gdouble agh_v_defaults	[AGH_VNUM + AGH_EGC_MAX-1];
extern const gdouble agh_v_min		[AGH_VNUM + AGH_EGC_MAX-1];
extern const gdouble agh_v_max		[AGH_VNUM + AGH_EGC_MAX-1];
extern const gdouble agh_v_step		[AGH_VNUM + AGH_EGC_MAX-1];

G_END_DECLS

struct STunableSet {
	gdouble	rs,   rc,
		fc_R, fc_W,
		S_0,  S_U,
		t_a,  t_p,
		gc;
	gdouble	egc[AGH_EGC_MAX-1];  // gc will be egc[0]

	gboolean check_sane();
	void assign_defaults();
};


union STunableSetIndexable {
	STunableSet pname;
	gdouble pidx[AGH_VNUM + AGH_EGC_MAX-1];
};



#define AGH_T_REQUIRED	 1
#define AGH_T_STABLE	(1 << 2)


struct STunableSetFull {
	STunableSetIndexable value, step, min, max;
	gint	flags[AGH_VNUM + AGH_EGC_MAX-1];

	gboolean check_consisitent();
	void assign_defaults();

	void unadjust_for_ppm( gfloat ppm)
	{	value.pname.rs   *= ppm,  min.pname.rs   *= ppm,  max.pname.rs   *= ppm,  step.pname.rs   *= ppm;
		value.pname.rc   *= ppm,  min.pname.rc   *= ppm,  max.pname.rc   *= ppm,  step.pname.rc   *= ppm;
		value.pname.fc_R *= ppm,  min.pname.fc_R *= ppm,  max.pname.fc_R *= ppm,  step.pname.fc_R *= ppm;
		value.pname.fc_W *= ppm,  min.pname.fc_W *= ppm,  max.pname.fc_W *= ppm,  step.pname.fc_W *= ppm;
		value.pname.gc   *= ppm,  min.pname.gc   *= ppm,  max.pname.gc   *= ppm,  step.pname.gc   *= ppm;
		for ( guint t = 0; t < AGH_EGC_MAX-1; t++ )
			value.pname.egc[t] *= ppm,  min.pname.egc[t] *= ppm,  max.pname.egc[t] *= ppm,  step.pname.egc[t] *= ppm;
		value.pname.t_a  /= ppm,  min.pname.t_a  /= ppm,  max.pname.t_a  /= ppm,  step.pname.t_a  /= ppm;
		value.pname.t_p  /= ppm,  min.pname.t_p  /= ppm,  max.pname.t_p  /= ppm,  step.pname.t_p  /= ppm;	}

	void adjust_for_ppm( gfloat ppm)
	{	value.pname.rs   /= ppm,  min.pname.rs   /= ppm,  max.pname.rs   /= ppm,  step.pname.rs   /= ppm;
		value.pname.rc   /= ppm,  min.pname.rc   /= ppm,  max.pname.rc   /= ppm,  step.pname.rc   /= ppm;
		value.pname.fc_R /= ppm,  min.pname.fc_R /= ppm,  max.pname.fc_R /= ppm,  step.pname.fc_R /= ppm;
		value.pname.fc_W /= ppm,  min.pname.fc_W /= ppm,  max.pname.fc_W /= ppm,  step.pname.fc_W /= ppm;
		value.pname.gc   /= ppm,  min.pname.gc   /= ppm,  max.pname.gc   /= ppm,  step.pname.gc   /= ppm;
		for ( guint t = 0; t < AGH_EGC_MAX-1; t++ )
			value.pname.egc[t] /= ppm,  min.pname.egc[t] /= ppm,  max.pname.egc[t] /= ppm,  step.pname.egc[t] /= ppm;
		value.pname.t_a  *= ppm,  min.pname.t_a  *= ppm,  max.pname.t_a  *= ppm,  step.pname.t_a  *= ppm;
		value.pname.t_p  *= ppm,  min.pname.t_p  *= ppm,  max.pname.t_p  *= ppm,  step.pname.t_p  *= ppm;	}


	void dump();
};





struct SControlParamSet {

	gfloat	InitRand,
		MinSuffFitImprov;
	guint	MaxAllowedParamDrift,
		StrideLength,
		MaxStrides,
		RetriesoOnSimCycleFail,
		TeleportDistance,
		MaxHalvingTimes;

	gboolean DBAmendment1,
		 DBAmendment2,
		 AZAmendment,
		 ScoreMVTAsWake,
		 ScoreUnscoredAsWake;

	gboolean check_sane();
	void assign_defaults();
};




class CExpDesign;

class CModelRun
  : public SControlParamSet, public STunableSetFull,
    public CSCourse {

    friend class CExpDesign;
    friend class CSimulation;

	guint	_stride;
	gdouble	_fit,
		_dfit;

	GArray	  *_t_stack;
	GPtrArray *_swa_sim_stack,
		  *_s_stack;

	STunableSet tunables0;

    public:
	GArray *_shadow;  // we need shadow to hold scores as modified per Score{MVT,Unscored}As... and by t_{a,p},
			  // and also to avoid recreating it before each stride
	gint status;

	CModelRun()	{ _shadow = NULL,  _t_stack = NULL;
			  _swa_sim_stack = _s_stack = NULL; }
	CModelRun( gfloat from, gfloat upto, guint ipagesize,
		   SFFTParamsSet&, SControlParamSet&, STunableSetFull&);
       ~CModelRun();

	guint stride()	{ return _stride; }
	gdouble fit()	{ return _fit;    }
	gdouble dfit()	{ return _dfit;   }
	STunableSetIndexable &nth_t_set( guint t)  { return g_array_index (_t_stack, STunableSetIndexable, t); }
	GArray* &nth_swa_sim_course( guint n)       { return *(GArray**) &g_ptr_array_index (_swa_sim_stack, n); }
	GArray* &nth_s_course( guint n)             { return *(GArray**) &g_ptr_array_index (_s_stack,       n); }

	void randomize();
	void do_cycle();
	gint do_stride( gint, GString*);

	gint save( const gchar*, gboolean binary = TRUE);
	gint load( const gchar*, gboolean binary = TRUE);

    private:
	inline gdouble &which_gc( guint p);  // selects an episode's egc by page, or returns &gc if !AZAmendment
	void extend_rem( guint, guint, gboolean);
	void prepare_shadow();
};




class CSubject;

class CSimulation
  : public CModelRun {

	GString *_basename;

    public:
	CSubject		*_subject;
	CRefList<CMeasurement>	*_session;
	COrderedName		*_channel;

	gboolean operator == ( CSimulation &o)  { return _subject == o._subject &&
							 _session == o._session &&
							 _channel == o._channel &&
							 freq_from == o.freq_from &&
							 freq_upto == o.freq_upto &&
							 pagesize == o.pagesize &&
							 fft_params == o.fft_params;    }

	CSimulation()  { status = Z_OBJECT_PHONY; _basename = NULL; }
	void make_phony( CSubject *J, CRefList<CMeasurement> *D, COrderedName *H,
			 gfloat ifreq_from, gfloat ifreq_upto, guint ipagesize,
			 guint iwindow_size_2power, gfloat ibin_size, TFFTWinType iwindow_type)
	{
		_subject = J, _session = D, _channel = H;
		freq_from = ifreq_from, freq_upto = ifreq_upto, pagesize = ipagesize;
		fft_params.window_size_2power = iwindow_size_2power, fft_params.bin_size = ibin_size, fft_params.window_type = iwindow_type;
	}

	CSimulation( CSubject *J, CRefList<CMeasurement> *D, COrderedName *H,
		     gfloat from, gfloat upto, guint ipagesize,
		     SFFTParamsSet&, SControlParamSet&, STunableSetFull&);

       ~CSimulation()  { if ( _basename )  g_string_free( _basename, TRUE); }


	const gchar *make_fname();
};



#endif

// EOF
