// ;-*-C++-*- *  Time-stamp: "2008-06-27 18:13:11 hmmr"

/***************************************************************************
 *            agh_simulation_cycle.cc
 *
 *  Copyright  2008  Andrei Zavada
 *  Email johnhommer@gmail.com

 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */




#include <stdlib.h>
#include <math.h>

#include "agh_model.hh"




#define P(p)	g_array_index (vector,  SPageSimulated, p)
#define T(p)	g_array_index (_shadow, SPage, p)




void
CModelRun::prepare_shadow()
{
	guint p;

      // copy shadow, patch Unscored and MVT
	if ( _shadow->len != vector->len )
		g_array_set_size( _shadow, vector->len);
	for ( p = 0; p < vector->len; p++ )
		T(p) = (SPage)P(p);  // cast to a base class

	if ( ScoreUnscoredAsWake ) {
		for ( p = _sim_start; p < vector->len; p++ )
			if ( P(p).NREM + P(p).REM + P(p).Wake == 0 ) { // this is Unscored
//				printf( " (patching Unscored as Wake at %d)\n", p);
				T(p).Wake = 1.;
			}
	} else {  // ... as prev page
		guint pp;
		for ( p = (_sim_start > 0) ?_sim_start :1; p < vector->len; p++ )
			if ( P(p).NREM + P(p).REM + P(p).Wake == 0 ) {
//				printf( " (patching Unscored as prev at %d)\n", p);
				pp = p-1;
				do
					T(p) = (SPage) P(pp);
				while ( ++p < vector->len  &&  P(p).NREM + P(p).REM + P(p).Wake == 0. );
			}
	}

	if ( ScoreMVTAsWake ) {
		for ( p = _sim_start; p < vector->len; p++ )
			if ( P(p).Wake == AGH_MVT_WAKE_VALUE ) { // this is MVT
//				printf( " (patching MVT as Wake at %d)\n", p);
				T(p).Wake = 1.;
			}
	} else {  // ... as prev page
		guint pp;
		for ( p = (_sim_start > 0) ?_sim_start :1; p < vector->len; p++ )
			if ( P(p).Wake == AGH_MVT_WAKE_VALUE ) {
//				printf( " (patching MVT as prev at %d)\n", p);
				pp = p-1;
				do
					T(p) = (SPage) P(pp);
				while ( ++p < vector->len  &&  P(p).Wake == AGH_MVT_WAKE_VALUE );
			}
	}

	status |= AGH_MODRUN_PREPARED;
}







static guint  Ma[AGH_EGC_MAX], Mz[AGH_EGC_MAX];  // meaning, no re-entrancy for do_stride

inline gdouble&
CModelRun::which_gc( guint p)
{
	if ( AZAmendment )
		for ( guint m = measurements->len-1; m >= 1; m-- )
			if ( p >= Ma[m] )
				return value.pidx[AGH_VNUM + m - 1];
	return value.pname.gc;
}








void
CModelRun::extend_rem( guint da, guint dz, gboolean fresh)
{
	guint	a  =               da,
		z  = vector->len - dz,
		p, pi, di;

	if ( fresh )
		for ( p = 0; p < vector->len; p++ )
			T(p).REM = P(p).REM;

	for ( p = a; p < z; p++ )
		if ( P(p).REM > 0.33 ) {  // only deal with some substantial REM
			for ( pi = p; P(pi).REM > .33 && pi < z; pi++ ) {

				// pull front
				di = da;
				do  {
					if ( P(pi).REM > P(pi - di).REM )
						T(pi - di).REM = P(pi).REM;
					di--;
				} while ( di );

				// push end
				di = dz;
				do {
					if ( P(pi).REM > P(pi + di).REM )
						T(pi + di).REM = P(pi).REM;
					di--;
				} while ( di );

			}  // perhaps, in addition to spreading the boundary value to regions before and after existing REM,
			   // we should also bump the existing values inside it?
			p = pi;
		}
}







gint
CModelRun::do_stride( gint from_stride, GString *logentry)
{

//	printf( "Data vector length = %d pp (%d with SWA), starting at %d\n",
//		vector->len, _pages_with_SWA, _sim_start);
//	dump();

	gint	retval = 0;

	guint	t, m, p, c;

	const gdouble	ppm = 60. / pagesize;


	static GString *format_str = g_string_sized_new( 120);

	if ( logentry )	g_string_printf( logentry, "Stride %d\n", from_stride+1);

#define X(t)  agh_v_names[t], agh_v_formats[t]
#define Y(t)  value.pidx[t] * agh_v_dispf[t]
      // print damn pretty
	if ( logentry )
		if ( AZAmendment ) {
			g_string_printf( format_str,
					 "In:\t %s:%s\t %s:%s\t %s:%s\t %s:%s\n"
					 "   \t %s:%s\t %s:%s\t %s:%s\t %s:%s\n",
					 X (AGH_VS_0),  X (AGH_Vfc_R),  X (AGH_Vt_a), X (AGH_Vrc),
					 X (AGH_VS_U),  X (AGH_Vfc_W),  X (AGH_Vt_p), X (AGH_Vrs));
			g_string_append_printf( logentry, format_str->str,
						Y (AGH_VS_0),  Y (AGH_Vfc_R),  Y (AGH_Vt_a),  Y (AGH_Vrc),
						Y (AGH_VS_U),  Y (AGH_Vfc_W),  Y (AGH_Vt_p),  Y (AGH_Vrs));

			g_string_append_printf( logentry, "  extgc: ");
			g_string_printf( format_str, "  %s", agh_v_formats[AGH_Vgc]);
			for ( m = 0; m < measurements->len; m++ )
				g_string_append_printf( logentry, format_str->str, value.pidx[AGH_VNUM+m-1] * agh_v_dispf[AGH_Vgc]);
			g_string_append_printf( logentry, "\n");

		} else {
			g_string_printf( format_str,
					 "In:\t %s:%s\t %s:%s\t %s:%s\t %s:%s\n"
					 "   \t %s:%s\t %s:%s\t %s:%s\t %s:%s\t %s:%s\n",
					 X (AGH_VS_0),  X (AGH_Vfc_R),  X (AGH_Vt_a), X (AGH_Vrc),
					 X (AGH_VS_U),  X (AGH_Vfc_W),  X (AGH_Vt_p), X (AGH_Vrs), X (AGH_Vgc));
			g_string_append_printf( logentry, format_str->str,
						Y (AGH_VS_0),  Y (AGH_Vfc_R),  Y (AGH_Vt_a),  Y (AGH_Vrc),
						Y (AGH_VS_U),  Y (AGH_Vfc_W),  Y (AGH_Vt_p),  Y (AGH_Vrs),  Y (AGH_Vgc));
		}
#undef X
#undef Y


	if ( from_stride == -1 )
		from_stride = _stride;

      // free up buffers higher on stack than the stride to pick up simulation from
	if ( from_stride < _stride ) {
		for ( guint n = from_stride; n < _stride; n++ ) {
			if ( nth_swa_sim_course(n) )  g_array_free( nth_swa_sim_course(n), TRUE), nth_swa_sim_course(n) = NULL;
			if ( nth_s_course(n)       )  g_array_free( nth_s_course(n),       TRUE), nth_s_course(n)       = NULL;
		}
		_stride = from_stride;
	}


	if ( _stride >= _t_stack->len )
		g_array_set_size( _t_stack, _stride + 5);
	if ( _stride >= _swa_sim_stack->len ) {
		g_ptr_array_set_size( _swa_sim_stack, _stride + 5);
		g_ptr_array_set_size( _s_stack,       _stride + 5);
	}


	guint	V_num = AGH_VNUM + (AZAmendment ?measurements->len - 1 :0);


      // push a copy of tunable set onto stack, also to see how far we've drifted on stride completion
	STunableSetIndexable *value0 = &g_array_index (_t_stack, STunableSetIndexable, _stride);
	*value0 = value;

      // also push the entire SWA_sim and S vectors
	{
		nth_swa_sim_course(_stride) = g_array_sized_new( FALSE, FALSE, sizeof(double), vector->len);
		nth_s_course(_stride)       = g_array_sized_new( FALSE, FALSE, sizeof(double), vector->len);
		GArray *a1 = nth_swa_sim_course(_stride), *a2 = nth_s_course(_stride);
		for ( p = 0; p < vector->len; p++ ) {
			g_array_index (a1, double, p) = P(p).SWA_sim;
			g_array_index (a2, double, p) = P(p).S;
		}
	}

	_stride++;

      // convert values in min and min^-1 according to pages-per-minute
	adjust_for_ppm( ppm);

      // substitute S_0 and S_U, expressed in %, with abs values
	value.pname.S_0 *= _SWA_100/100,  min.pname.S_0 *= _SWA_100/100,  max.pname.S_0 *= _SWA_100/100,  step.pname.S_0 *= _SWA_100/100;
	value.pname.S_U *= _SWA_100/100,  min.pname.S_U *= _SWA_100/100,  max.pname.S_U *= _SWA_100/100,  step.pname.S_U *= _SWA_100/100;


      // prepare timeline
	if ( status | ~AGH_MODRUN_PREPARED ) {
		prepare_shadow();  // will also call extend_rem, with ppm-adjusted t_{a,p}
		for ( m = 0; m < measurements->len; m++ )
			Ma[m] = nth_msmt_start_p(m), Mz[m] = nth_msmt_end_p(m);
	}
	extend_rem( (int)round( value.pname.t_a), (int)round( value.pname.t_p), TRUE);



      // set up tunable accounting
	guint	now_tuning, left_to_tune;
	gint	halved[V_num],
		direction[V_num],
		done_tuning[V_num];
	memset( (void*)halved, 0, sizeof(gint) * V_num);

      // randomize directions initially; these directions will
      // be preserved across cycles to minimize roundabouts
	for ( t = 0; t < V_num; t++ )
		direction[t] = (rand() - RAND_MAX > -RAND_MAX/2) ?1 :-1;

      // this is a copy of steps, to be used to restore steps before every next cycle
      // after they have possibly been halved
	gdouble	step0_adjusted[V_num];
	for ( t = 0; t < V_num; t++ )
		step0_adjusted[t] = step.pidx[t];

	if ( DBAmendment2 ) {
		P(_baseline_end).S = _SWA_100 * 3; // will be overwritten at completion of the first iteration
		step.pname.S_U = 0.; // not subject to tuning under DB II; a 0 value also suppresses reporting
				     // of bogus increment at exit
	}


	P(_sim_start).SWA_sim = _SWA_0;  // this is the primer for the entire SWA_sim line


	gdouble	fit0 = _fit,
		fit_p = 0., fit_pp = 0.,
		value_pp = 0., value_p = 0.;

	gint	nudge;
	gboolean just_about;

	for ( c = 0; c < StrideLength; c++ ) {

	      // get the no. of tunables actually to tune
		left_to_tune = V_num;
		for ( t = 0; t < V_num; t++ )
			left_to_tune -= (done_tuning[t] = (step.pidx[t] == 0.));


//		printf( "--- cycle %4d\n", c);
//		printf( "--- cycle %4d :: S[sim_start] = %07g;  S[BL_end] = %07g;  S_U = %07g;   S[24] would be %07g\n",
//			c, Pc(_sim_start).S, Pc(_baseline_end).S,
//			value.pname.S_U,
//			value.pname.S_U
//			- (value.pname.S_U - Pc(_baseline_end).S)
//			* exp( -(24*60*ppm - (_baseline_end - _sim_start)) * value.pname.rs));

		for ( t = 0; t < left_to_tune; t++ ) {

		      // select next tunable randomly
			do
				now_tuning = (int)( (gdouble)rand() / RAND_MAX * V_num );
			while ( done_tuning[now_tuning] );
//			printf( "now tuning: %s: value = %g\n", agh_v_names[now_tuning], value.pidx[now_tuning]);

			just_about = FALSE;  // when we are smack at the minimum already at nudge == 0,
					     // incrementing in either direction will be wrong way; this var
					     // will be flagged in the 'wrong way' case, and cleared
					     // otherwise; and if encountered on suspicion of going wrong way again,
					     // will bail out with success

			for ( nudge = 0; nudge < 3; nudge++ ) {

				fit_pp   = fit_p,    fit_p   = _fit;
				value_pp = value_p,  value_p = value.pidx[now_tuning];
				value.pidx[now_tuning] += direction[now_tuning] * step.pidx[now_tuning];

				P(_sim_start).S = value.pname.S_0;

				if ( step.pname.t_a > 0. || step.pname.t_p > 0. )
					extend_rem( (guint)round( value.pname.t_a), (guint)round( value.pname.t_p), TRUE);

				_fit = 0.;
				for ( p = _sim_start; p < vector->len-1; p++ ) {

					if ( DBAmendment2 ) {
						double edt = exp( -(24*60*ppm + _sim_start - _baseline_end) * value.pname.rs);
						value.pname.S_U = (P(_sim_start).S - P(_baseline_end).S * edt) / (1. - edt);
					}

					int	WT = (T(p).Wake > 0.33);

					double	pS = P(p).S / value.pname.S_U;
					P(p+1).SWA_sim = P(p).SWA_sim
						+ value.pname.rc * P(p).SWA_sim * pS * (1. - P(p).SWA_sim / P(p).S)
						  * (DBAmendment1 ?(1. - T(p).Wake) :1)
						- value.pname.fc_R * (P(p).SWA_sim - _SWA_L) * T(p).REM
						- value.pname.fc_W * (P(p).SWA_sim - _SWA_L) * T(p).Wake;

					P(p+1).S = P(p).S + ( (WT && DBAmendment1)
							       ?0
							       :(-which_gc(p) * P(p).SWA_sim) )
						+ (value.pname.S_U - P(p).S) * value.pname.rs;

					if ( T(p).has_swa() )
						_fit += pow( P(p).SWA - P(p).SWA_sim, 2);
				}

				if ( nudge == 2 ) {
//					printf( "%4s: %9.7f -> %9.7f -> %9.7f    fit: %7.5f -> %7.5f -> %7.5f",
//						agh_v_names[now_tuning], value_pp, value_p, value.pidx[now_tuning], fit_pp, fit_p, _fit);
					if ( isnan( value.pidx[now_tuning]) ) {
						if ( logentry )	g_string_append_printf( logentry, "%s goes nan\n", agh_v_names[now_tuning]);
						retval = AGH_SIMCYCLE_ENAN;
						goto out;
					}
					if        ( value.pidx[now_tuning] > max.pidx[now_tuning] ) {
						if ( logentry )	g_string_append_printf( logentry, "%s hit upper limit (%g)\n",
											agh_v_names[now_tuning], value.pidx[now_tuning] * agh_v_dispf[now_tuning]);
						retval = AGH_SIMCYCLE_EHITMAX;
						goto out;
					} else if ( value.pidx[now_tuning] < min.pidx[now_tuning] ) {
						if ( logentry )	g_string_append_printf( logentry, "%s hit lower limit (%g)\n",
											agh_v_names[now_tuning], value.pidx[now_tuning] * agh_v_dispf[now_tuning]);
						retval = AGH_SIMCYCLE_EHITMIN;
						goto out;
					}
					if ( value.pname.S_U <= value.pname.S_0 ) {
						if ( logentry )	g_string_append_printf( logentry, "S_U crashing on S_0 (while tuning %s)\n", agh_v_names[now_tuning]);
						retval = AGH_SIMCYCLE_ESUS0CRASH;
						goto out;
					}

				      // take decision
					if ( fit_pp < fit_p  &&  fit_p < _fit ) { // absolutely wrong
						if ( just_about )  {  // just been here, meaning value_pp is min
//							printf( " is OK\n");
							value.pidx[now_tuning] = value_pp;
							break;
						} else {
//							printf( " wrong way\n");
							direction[now_tuning] = -direction[now_tuning];
							value.pidx[now_tuning] = value_pp;
							just_about = TRUE;
							nudge = -1;
							continue;  // avoid getting incremented
						}
					}

					if ( fit_pp < fit_p  &&  fit_p > _fit ) { // local max
						if ( halved[now_tuning] == 0 )  { // once we've been near a minimum, don't teleport,
										  // but rather let other tunables try to unsettle the current one
							if ( fit_pp < _fit ) {
								direction[now_tuning] = -direction[now_tuning];
								value.pidx[now_tuning] = value_pp;
							}

//							printf( " teleport from near local maximum at %g/%d\n", value_p, halved[now_tuning]);
							value.pidx[now_tuning] += (direction[now_tuning] * (gint)TeleportDistance * step.pidx[now_tuning] * (double)rand()/RAND_MAX);
//							printf( "%d steps (distance %g, step %g)\n",
//								direction[now_tuning] * (gint)TeleportDistance,
//								direction[now_tuning] * (gint)TeleportDistance * step.pidx[now_tuning],
//								step.pidx[now_tuning]);
							nudge = -1;
							continue;
						}
//						printf( " just fine\n");

					} else if ( fit_pp > fit_p  &&  fit_p < _fit ) { // local min
						value.pidx[now_tuning] = value_p;
						if ( halved[now_tuning] < MaxHalvingTimes ) {
//							printf( " near stable at %g: halving step\n", value_p);
							if ( fit_pp < _fit )
								direction[now_tuning] = -direction[now_tuning];
							step.pidx[now_tuning] /= 2;
							halved[now_tuning]++;
							nudge = -1;
							continue;
						} else { // consider stable
							;
//							printf( " done tuning at %g/%d\n", value_p, halved[now_tuning]);
						}
					}
//					printf( "\n");
				}
			} // nudges
			done_tuning[now_tuning] = TRUE;

		} // tunables

	} // cycles in stride

out:
      // restore steps after they may have been halved
	for ( t = 0; t < V_num; t++ )
		step.pidx[t] = step0_adjusted[t];

      // revert S values to %% (except steps, which are restored en masse, below)
	value.pname.S_0 /= _SWA_100/100, min.pname.S_0 /= _SWA_100/100, max.pname.S_0 /= _SWA_100/100, step.pname.S_0 /= _SWA_100/100;
	value.pname.S_U /= _SWA_100/100, min.pname.S_U /= _SWA_100/100, max.pname.S_U /= _SWA_100/100, step.pname.S_U /= _SWA_100/100;

      // revert tunable values to per-minute units
	unadjust_for_ppm( ppm);

      // flag stable tunables
	if ( retval == 0 ) {
		gint	n_unstable = V_num,
			n_required_unstable = 0;
		for ( t = 0; t < V_num; t++ )
			if ( fabs( value.pidx[t] - value0->pidx[t]) <= step.pidx[t] * MaxAllowedParamDrift ) {  // `or equal' will capture non-tunable tunables which have step = 0
				flags[t] |=  AGH_T_STABLE;
				n_unstable--;
			} else {
				flags[t] &= ~AGH_T_STABLE;
				if ( flags[t] & AGH_T_REQUIRED )
					n_required_unstable++;
			}
		if ( n_unstable == 0 )		status |=  AGH_MODRUN_ALLSTABLE;
		else				status &= ~AGH_MODRUN_ALLSTABLE;
		if ( n_required_unstable == 0 )	status |=  AGH_MODRUN_ALLREQSTABLE;
		else				status &= ~AGH_MODRUN_ALLREQSTABLE;


		_fit /= (_pages_with_SWA  / _SWA_100/100);
		_dfit = (fit0 - _fit) / fit0 * 100;

		if ( fabs( _dfit) < MinSuffFitImprov )
			status |= AGH_MODRUN_FITSUFFICIENT;
		else
			status &= ~AGH_MODRUN_FITSUFFICIENT;
		status |= AGH_MODRUN_INMEM;

#define X(t)  agh_v_names[t], agh_v_formats[t], halved[t], (flags[t] & AGH_T_STABLE) ?"\342\234\246" :""
#define Y(t)  value.pidx[t] * agh_v_dispf[t]
		if ( logentry ) {
			if ( AZAmendment ) {
				g_string_printf( format_str,
						 "Out:\t %s:%s/%d%s\t %s:%s/%d%s\t %s:%s/%d%s\t %s:%s/%d%s\n"
						 "    \t %s:%s/%d%s\t %s:%s/%d%s\t %s:%s/%d%s\t %s:%s/%d%s\n",
						 X (AGH_VS_0),  X (AGH_Vfc_R),  X (AGH_Vt_a), X (AGH_Vrc),
						 X (AGH_VS_U),  X (AGH_Vfc_W),  X (AGH_Vt_p), X (AGH_Vrs));
				g_string_append_printf( logentry, format_str->str,
							Y (AGH_VS_0),  Y (AGH_Vfc_R),  Y (AGH_Vt_a),  Y (AGH_Vrc),
							Y (AGH_VS_U),  Y (AGH_Vfc_W),  Y (AGH_Vt_p),  Y (AGH_Vrs));

				g_string_append_printf( logentry, "  extgc: ");
				g_string_printf( format_str, "  %s", agh_v_formats[AGH_Vgc]);
				for ( m = 0; m < measurements->len; m++ )
					g_string_append_printf( logentry, format_str->str, value.pidx[AGH_VNUM+m-1] * agh_v_dispf[AGH_Vgc]);
				g_string_append_printf( logentry, "\n");

			} else {
				g_string_printf( format_str,
						 "Out:\t %s:%s/%d%s\t %s:%s/%d%s\t %s:%s/%d%s\t %s:%s/%d%s\n"
						 "    \t %s:%s/%d%s\t %s:%s/%d%s\t %s:%s/%d%s\t %s:%s/%d%s\t %s:%s/%d%s\n",
						 X (AGH_VS_0),  X (AGH_Vfc_R),  X (AGH_Vt_a),  X (AGH_Vrc),
						 X (AGH_VS_U),  X (AGH_Vfc_W),  X (AGH_Vt_p),  X (AGH_Vrs),  X (AGH_Vgc));
				g_string_append_printf( logentry, format_str->str,
							Y (AGH_VS_0),  Y (AGH_Vfc_R),  Y (AGH_Vt_a),  Y (AGH_Vrc),
							Y (AGH_VS_U),  Y (AGH_Vfc_W),  Y (AGH_Vt_p),  Y (AGH_Vrs),  Y (AGH_Vgc));
			}

			g_string_append_printf( logentry,
						"  fit: %8g%% of avg SWA (%4.2f%%, %ssufficient)\n"
						"  %d tunable(s) unstable, of which %d required\n\n",
						_fit, _dfit, (status & AGH_MODRUN_FITSUFFICIENT) ?"" :"not ",
						n_unstable, n_required_unstable);
		}
#undef X
#undef Y
	} else {
//		status &= ~(AGH_MODRUN_S_AVAILABLE | AGH_MODRUN_FITSUFFICIENT | AGH_MODRUN_ALLSTABLE | AGH_MODRUN_ALLREQSTABLE, status |= AGH_MODRUN_FAILED;
		status = AGH_MODRUN_FAILED | AGH_MODRUN_PREPARED; // just that
		_fit = 0.;
		for ( t = 0; t < V_num; t++ )
			flags[t] |= ~AGH_T_STABLE;
	}

	return retval;
}













void
CModelRun::do_cycle()
{
	const gdouble	ppm = 60. / pagesize;

      // convert values in min and min^-1 according to pages-per-minute
	adjust_for_ppm( ppm);

      // substitute S_0 and S_U, expressed in %, with abs values
	value.pname.S_0 *= _SWA_100/100,  min.pname.S_0 *= _SWA_100/100,  max.pname.S_0 *= _SWA_100/100,  step.pname.S_0 *= _SWA_100/100;
	value.pname.S_U *= _SWA_100/100,  min.pname.S_U *= _SWA_100/100,  max.pname.S_U *= _SWA_100/100,  step.pname.S_U *= _SWA_100/100;


      // prepare timeline
	if ( status | ~AGH_MODRUN_PREPARED ) {
		prepare_shadow();  // will also call extend_rem, with ppm-adjusted t_{a,p}
		for ( guint m = 0; m < measurements->len; m++ )
			Ma[m] = nth_msmt_start_p(m), Mz[m] = nth_msmt_end_p(m);
	}
	extend_rem( (int)round( value.pname.t_a), (int)round( value.pname.t_p), TRUE);



	gdouble	step0_S_U = step.pname.S_U;
	if ( DBAmendment2 ) {
		P(_baseline_end).S = _SWA_100 * 3; // will be overwritten at completion of the first iteration
		step.pname.S_U = 0.;
	}


//	_fit = 0.;

	P(_sim_start).SWA_sim = _SWA_0;  // this is the primer for the entire SWA_sim line
	P(_sim_start).S = value.pname.S_0;

	for ( guint p = _sim_start; p < vector->len-1; p++ ) {

		if ( DBAmendment2 ) {
			double edt = exp( -(24*60*ppm + _sim_start - _baseline_end) * value.pname.rs);
			value.pname.S_U = (P(_sim_start).S - P(_baseline_end).S * edt) / (1. - edt);
		}

		int	WT = (T(p).Wake > 0.33);

		double	pS = P(p).S / value.pname.S_U;
		P(p+1).SWA_sim = P(p).SWA_sim
			+ value.pname.rc * P(p).SWA_sim * pS * (1. - P(p).SWA_sim / P(p).S)
			* (DBAmendment1 ?(1. - T(p).Wake) :1)
			- value.pname.fc_R * (P(p).SWA_sim - _SWA_L) * T(p).REM
			- value.pname.fc_W * (P(p).SWA_sim - _SWA_L) * T(p).Wake;

		P(p+1).S = P(p).S + ( (WT && DBAmendment1)
				      ?0
				      :(-which_gc(p) * P(p).SWA_sim) )
			+ (value.pname.S_U - P(p).S) * value.pname.rs;

//		if ( T(p).has_swa() )
//			_fit += pow( P(p).SWA - P(p).SWA_sim, 2);
	}

      // restore step.pname.S_U
	step.pname.S_U = step0_S_U;

      // revert S values to %% (except steps, which are restored en masse, below)
	value.pname.S_0 /= _SWA_100/100, min.pname.S_0 /= _SWA_100/100, max.pname.S_0 /= _SWA_100/100, step.pname.S_0 /= _SWA_100/100;
	value.pname.S_U /= _SWA_100/100, min.pname.S_U /= _SWA_100/100, max.pname.S_U /= _SWA_100/100, step.pname.S_U /= _SWA_100/100;

      // revert tunable values to per-minute units
	unadjust_for_ppm( ppm);
}


// EOF
