// ;-*-C-*- *  Time-stamp: "2008-07-02 02:33:22 hmmr"

/***************************************************************************
 *            agh_ui-design-scoring-facility.c
 *
 *  Copyright  2008  Andrei Zavada
 *  Email johnhommer@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <glade/glade.h>
#include <string.h>
#include <math.h>
#include "agh_iface.hh"
#include "agh_ui.h"



GtkListStore *agh_mDesignSetupPageSize;

GtkWidget
	*wScoringFacility,
	*eScoringFacPageSize;

static GtkWidget
	*daScoringFacProfileView,
	*cScoringFacPageViews,
	*daScoringFacHypnogram,
	*bScoringFacBack,
	*bScoringFacForward,
	*lScoringFacTotalPages,
	*lScoringFacPercentScored,
	*eScoringFacCurrentPage,
	*lScoringFacCurrentPos,
	*lScoreStatsNREMPercent,
	*lScoreStatsREMPercent,
	*lScoreStatsWakePercent,
	*lScoringFacCurrentStage,
	*bScoreClear, *bScoreNREM1, *bScoreNREM2, *bScoreNREM3, *bScoreNREM4,
	*bScoreREM,   *bScoreWake,  *bScoreMVT;



static const gchar* const __bg_rgb_score[] = {
 "#2A0952", "#2D2D8B", "#000064", "#000046",
 "#00002a", "#610863", "#8D987F", "#4F3D02",
 "#000000", "#000000"
};

static const gchar* const __fg_rgb_score[] = {
 "#dddddd", "#FFEDB5", "#FFEDB5", "#FFEDB5",
 "#FFF7DF", "#E2E2E2", "#BDBDBD", "#BDBDBD",
 "#FFFFFF", "#00FE1E"
};

static const gchar __bg_rgb_power[] = "#111142";
static const gchar __fg_rgb_power[] = "#FEFF43";

static GdkColor	__fg_score[10], __bg_score[10],
		__fg_power,     __bg_power;

static GdkGC *__gc_score[8], *__gc_score_line, *__gc_score_cursor, *__gc_power;







static gchar	__label_text[60];







static gulong	__signal_length; // in sec
static guint	__total_pages; // = __signal_length / agh_pagesize_values[agh_pagesize_item]
static guint	__cur_page,
		__cur_page_app,
		__cur_pos_hr, __cur_pos_min, __cur_pos_sec;

typedef struct {
	gchar	  *name;

	GArray	  *signal;
	gint	   samplerate;
	gfloat	   signal_scale,
		   display_scale;
	GtkWidget *page_view;

	GArray	  *power;
	gfloat	   from,
		   upto;
	gdouble	   max_value;
	GtkWidget *profile_view;

	GArray	  *track;
	gboolean   new_marks;
	gfloat	   dirty_percent;

	gboolean   visible;
	GtkWidget *expander,
		  *vbox;
	gulong     expose_handler_id;

} SChannelPresentation;

static GArray	*__channels;





#define PSZ  agh_pagesize_values[agh_pagesize_item]
#define APSZ agh_pagesize_values[  __pagesize_item]

#define P2AP(p)  (guint)((p) * (float)PSZ / (float)APSZ)
#define AP2P(p)  (guint)((p) * (float)APSZ / (float)PSZ)

static guint	__pagesize_item = 4;


static guint __pagesize_ticks[6] = {
	5, 5, 3, 4, 6, 12
};



static GArray	*__hypnogram;

static void
__scale_hypnogram( guint from, guint into)
{
	GArray *hyp_new = g_array_new( FALSE, TRUE, sizeof(char));
	g_array_set_size( hyp_new, __hypnogram->len * from/into);
	guint p;
	for ( p = 0; p < hyp_new->len; p++ )
		g_array_index (hyp_new, gchar, p) = g_array_index (__hypnogram, gchar, p * into/from);
	g_array_free( __hypnogram, TRUE);
	__hypnogram = hyp_new;
	g_array_set_size( __hypnogram, __total_pages);
}





static gint	__cur_stage;

guint SCOREID( gchar c)  {  return c ?((guint)c - '1' + 1) :0;  }

gchar *__score_names[] = {
	"blank",
	"NREM1", "NREM2", "NREM3", "NREM4",
	"REM",
	"Wake",
	"MVT"
};



static guint __score_hypn_depth[8] = {
	0, 20, 23, 30, 33, 5, 10, 1
};






static void __repaint_score_stats();

static gfloat
__percent_scored()
{
	guint	scored_pages = 0,
		p;
	for ( p = 0; p < __hypnogram->len; p++ )
		scored_pages += (g_array_index( __hypnogram, gchar, p) != '\0');
	return (gfloat) scored_pages / p * 100;
}

static gfloat
__percent_NREM()
{
	guint	nrem_pages = 0,
		p;
	for ( p = 0; p < __hypnogram->len; p++ )
		nrem_pages += (g_array_index (__hypnogram, gchar, p) == '1' ||
			       g_array_index (__hypnogram, gchar, p) == '2' ||
			       g_array_index (__hypnogram, gchar, p) == '3' ||
			       g_array_index (__hypnogram, gchar, p) == '4' );
	return (gfloat) nrem_pages / p * 100;
}

static gfloat
__percent_REM()
{
	guint	rem_pages = 0,
		p;
	for ( p = 0; p < __hypnogram->len; p++ )
		rem_pages += (g_array_index (__hypnogram, gchar, p) == '5' );
	return (gfloat) rem_pages / p * 100;
}

static gfloat
__percent_Wake()
{
	guint	wake_pages = 0,
		p;
	for ( p = 0; p < __hypnogram->len; p++ )
		wake_pages += (g_array_index (__hypnogram, gchar, p) == '6' ||
			       g_array_index (__hypnogram, gchar, p) == '7' );
	return (gfloat) wake_pages / p * 100;
}




static GtkWidget *__marking_in_widget;
static guint __mark_start;
static void __mark_region( guint, guint, SChannelPresentation*, gchar);


static void
__calculate_dirty_percent( SChannelPresentation *Ch)
{
	guint	dirty_secs = 0,
		p;
	for ( p = 0; p < Ch->track->len; p++ )
		dirty_secs += g_array_index (Ch->track, gchar, p);
	Ch->dirty_percent = (gfloat) dirty_secs / p * 100;
}




gboolean daScoringFacPageView_expose_event_cb( GtkWidget*, GdkEventExpose*, gpointer);
gboolean daScoringFacPageView_button_press_event_cb( GtkWidget*, GdkEventButton*, gpointer);
gboolean daScoringFacPageView_button_release_event_cb( GtkWidget*, GdkEventButton*, gpointer);
gboolean daScoringFacPageView_scroll_event_cb( GtkWidget*, GdkEventScroll*, gpointer);
gboolean cScoringFacPageViewExpander_activate_cb( GtkExpander*, gpointer);

gboolean daScoringFacProfileView_expose_event_cb( GtkWidget*, GdkEventExpose*, gpointer);
gboolean daScoringFacProfileView_button_press_event_cb( GtkWidget*, GdkEventButton*, gpointer);
gboolean daScoringFacProfileView_scroll_event_cb( GtkWidget*, GdkEventScroll*, gpointer);








gint
agh_construct_ScoringFacility( GladeXML *xml)
{
	GtkCellRenderer *renderer;

	if ( !(wScoringFacility         = glade_xml_get_widget( xml, "wScoringFacility")) ||
	     !(cScoringFacPageViews     = glade_xml_get_widget( xml, "cScoringFacPageViews")) ||
	     !(daScoringFacHypnogram    = glade_xml_get_widget( xml, "daScoringFacHypnogram")) ||
	     !(bScoringFacBack          = glade_xml_get_widget( xml, "bScoringFacBack")) ||
	     !(bScoringFacForward       = glade_xml_get_widget( xml, "bScoringFacForward")) ||
	     !(lScoringFacTotalPages    = glade_xml_get_widget( xml, "lScoringFacTotalPages")) ||
	     !(eScoringFacPageSize      = glade_xml_get_widget( xml, "eScoringFacPageSize")) ||
	     !(eScoringFacCurrentPage   = glade_xml_get_widget( xml, "eScoringFacCurrentPage")) ||
	     !(lScoringFacCurrentStage  = glade_xml_get_widget( xml, "lScoringFacCurrentStage")) ||
	     !(lScoringFacCurrentPos    = glade_xml_get_widget( xml, "lScoringFacCurrentPos")) ||
	     !(lScoringFacPercentScored = glade_xml_get_widget( xml, "lScoringFacPercentScored")) ||
	     !(lScoreStatsNREMPercent   = glade_xml_get_widget( xml, "lScoreStatsNREMPercent")) ||
	     !(lScoreStatsREMPercent    = glade_xml_get_widget( xml, "lScoreStatsREMPercent")) ||
	     !(lScoreStatsWakePercent   = glade_xml_get_widget( xml, "lScoreStatsWakePercent")) ||
	     !(bScoreClear  = glade_xml_get_widget( xml, "bScoreClear")) ||
	     !(bScoreNREM1  = glade_xml_get_widget( xml, "bScoreNREM1")) ||
	     !(bScoreNREM2  = glade_xml_get_widget( xml, "bScoreNREM2")) ||
	     !(bScoreNREM3  = glade_xml_get_widget( xml, "bScoreNREM3")) ||
	     !(bScoreNREM4  = glade_xml_get_widget( xml, "bScoreNREM4")) ||
	     !(bScoreREM    = glade_xml_get_widget( xml, "bScoreREM"))   ||
	     !(bScoreWake   = glade_xml_get_widget( xml, "bScoreWake"))  ||
	     !(bScoreMVT    = glade_xml_get_widget( xml, "bScoreMVT")) )
		return -1;

	gtk_combo_box_set_model( GTK_COMBO_BOX (eScoringFacPageSize),
				 GTK_TREE_MODEL (agh_mDesignSetupPageSize));

	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (eScoringFacPageSize), renderer, FALSE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (eScoringFacPageSize), renderer,
					"text", 0,
					NULL);


	GdkVisual *visual = gdk_visual_get_system();
	GdkGCValues xx;
	guint i;
	for ( i = 0; i < 9; i++ ) {
		gdk_color_parse( __fg_rgb_score[i], &__fg_score[i]), gdk_colormap_alloc_color( gtk_widget_get_colormap( daScoringFacHypnogram), &__fg_score[i], FALSE, TRUE);
		gdk_color_parse( __bg_rgb_score[i], &__bg_score[i]), gdk_colormap_alloc_color( gtk_widget_get_colormap( daScoringFacHypnogram), &__bg_score[i], FALSE, TRUE);

		xx.foreground = __bg_score[i], xx.background = __fg_score[0];  // bg <-> fg
		xx.line_width = 1, xx.line_style = GDK_LINE_SOLID;

		__gc_score[i] = gtk_gc_get( visual->depth, gtk_widget_get_colormap( daScoringFacHypnogram),
					    &xx, GDK_GC_FOREGROUND | GDK_GC_BACKGROUND | GDK_GC_LINE_WIDTH | GDK_GC_LINE_STYLE);
	}
	gdk_color_parse( __bg_rgb_score[0], &__bg_score[0]), gdk_colormap_alloc_color( gtk_widget_get_colormap( daScoringFacHypnogram), &__bg_score[0], FALSE, TRUE);
	gtk_widget_modify_bg( daScoringFacHypnogram, GTK_STATE_NORMAL, &__bg_score[0]);

	gdk_color_parse( __fg_rgb_score[8], &__fg_score[8]), gdk_colormap_alloc_color( gtk_widget_get_colormap( daScoringFacHypnogram), &__fg_score[8], FALSE, TRUE);
	xx.foreground = __fg_score[8];
	xx.line_width = 2, xx.line_style = GDK_LINE_SOLID, xx.join_style = GDK_JOIN_ROUND, xx.cap_style = GDK_CAP_BUTT;
	__gc_score_line = gtk_gc_get( visual->depth, gtk_widget_get_colormap( daScoringFacHypnogram),
				      &xx, GDK_GC_FOREGROUND | GDK_GC_LINE_WIDTH | GDK_GC_LINE_STYLE | GDK_GC_JOIN_STYLE | GDK_GC_CAP_STYLE);

	gdk_color_parse( __fg_rgb_score[9], &__fg_score[9]), gdk_colormap_alloc_color( gtk_widget_get_colormap( daScoringFacHypnogram), &__fg_score[9], FALSE, TRUE);
	xx.foreground = __fg_score[9];
	xx.line_width = 1, xx.line_style = GDK_LINE_SOLID;
	__gc_score_cursor = gtk_gc_get( visual->depth, gtk_widget_get_colormap( daScoringFacHypnogram),
					&xx, GDK_GC_FOREGROUND | GDK_GC_LINE_WIDTH | GDK_GC_LINE_STYLE);

	gdk_color_parse( __fg_rgb_power, &__fg_power), gdk_colormap_alloc_color( gtk_widget_get_colormap( daScoringFacHypnogram), &__fg_power, FALSE, TRUE);
	gdk_color_parse( __bg_rgb_power, &__bg_power), gdk_colormap_alloc_color( gtk_widget_get_colormap( daScoringFacHypnogram), &__bg_power, FALSE, TRUE);
	xx.foreground = __fg_power, xx.background = __bg_power;
	xx.line_width = 1, xx.line_style = GDK_LINE_SOLID;
	__gc_power = gtk_gc_get( visual->depth, gtk_widget_get_colormap( daScoringFacHypnogram),
				 &xx, GDK_GC_FOREGROUND | GDK_GC_BACKGROUND | GDK_GC_LINE_WIDTH | GDK_GC_LINE_STYLE);


	if ( !__channels ) {
		__channels  = g_array_new( FALSE, FALSE, sizeof(SChannelPresentation));
		__hypnogram = g_array_new( FALSE,  TRUE, sizeof(gchar));
	}

	return 0;
}




gboolean
__agh_prepare_score_facility()
{
	agh_set_cursor_busy( TRUE, wMainWindow);
	gtk_statusbar_push( GTK_STATUSBAR (sbMainStatusBar), agh_sb_context_id_DesignMeasurements,
			    "Reading edf...");
	while ( gtk_events_pending() )
		gtk_main_iteration();

      // clean up after previous viewing
	gtk_container_foreach( GTK_CONTAINER (cScoringFacPageViews), gtk_widget_destroy, NULL);

	guint h;
	for ( h = 0; h < __channels->len; h++ ) {
		SChannelPresentation *Ch = &g_array_index (__channels, SChannelPresentation, h);
		if ( Ch->signal ) { g_array_free( Ch->signal, TRUE);  Ch->signal = NULL; }
		if ( Ch->power )  { g_array_free( Ch->power, TRUE);   Ch->power  = NULL; }
		if ( Ch->track )  { g_array_free( Ch->track, TRUE);   Ch->track  = NULL; }
	}
	__cur_page = 0;
	__signal_length = 0; // is set to the longest signal, below

	guint __pagesize;


      // set up channel representations
	g_array_set_size( __channels, agh_get_n_channels());

	guint n_visible = 0;
	for ( h = 0; h < __channels->len; h++ ) {
		SChannelPresentation *Ch = &g_array_index (__channels, SChannelPresentation, h);
		Ch->name = agh_get_nth_channel_name( h);

		Ch->signal = g_array_new( FALSE, FALSE, sizeof(gint16));
		agh_msmt_get_signal_vector( agh_j_name->str, agh_e_name->str, Ch->name,
					    Ch->signal, &Ch->samplerate, &Ch->signal_scale);

		Ch->from = 2, Ch->upto = 3;
		if ( !strcmp( agh_msmt_get_signal_type( agh_j_name->str, agh_e_name->str, Ch->name),
			      "EEG") &&
		     agh_msmt_has_edf( agh_j_name->str, agh_e_name->str, Ch->name) ) {
			Ch->power = g_array_new( FALSE, FALSE, sizeof(double));
			Ch->track = g_array_new( FALSE, FALSE, sizeof(char));
			agh_msmt_get_power( agh_j_name->str, agh_e_name->str, Ch->name,
					    Ch->power, Ch->track, Ch->from, Ch->upto, &Ch->max_value,
					    FALSE);
//			printf( "track->len = %d\n", Ch->track->len);
			__calculate_dirty_percent( Ch);
		} else
			Ch->power = Ch->track = NULL;

		Ch->new_marks = FALSE;

		if ( Ch->signal->len == 0 )
			Ch->visible = FALSE;
		else {
			Ch->visible = TRUE;
			n_visible++;

			Ch->expander = gtk_expander_new( Ch->name);
			gtk_box_pack_start( GTK_BOX (cScoringFacPageViews),
					    Ch->expander, TRUE, TRUE, 3);
			gtk_expander_set_expanded( GTK_EXPANDER (Ch->expander),
						   h == agh_current_channel);
			g_signal_connect_after( Ch->expander, "activate",
					  G_CALLBACK (cScoringFacPageViewExpander_activate_cb),
					  (gpointer)Ch);

			gtk_container_add( GTK_CONTAINER (Ch->expander),
					   Ch->vbox = gtk_vbox_new( FALSE, 3));

		      // set up page view
			if ( Ch->signal->len ) {
				__signal_length = MAX( __signal_length, Ch->signal->len / Ch->samplerate);
				Ch->display_scale = 20;

				gtk_container_add( GTK_CONTAINER (Ch->vbox),
						   Ch->page_view = gtk_drawing_area_new());
				g_object_set( G_OBJECT (Ch->page_view),
					      "app-paintable", TRUE,
					      "height-request", 120,
					      NULL);
				Ch->expose_handler_id = g_signal_connect_after( Ch->page_view, "expose-event",
										G_CALLBACK (daScoringFacPageView_expose_event_cb),
										(gpointer)Ch);
				g_signal_connect_after( Ch->page_view, "button-press-event",
							G_CALLBACK (daScoringFacPageView_button_press_event_cb),
							(gpointer)Ch);
				g_signal_connect_after( Ch->page_view, "button-release-event",
							G_CALLBACK (daScoringFacPageView_button_release_event_cb),
							(gpointer)Ch);
				g_signal_connect_after( Ch->page_view, "scroll-event",
							G_CALLBACK (daScoringFacPageView_scroll_event_cb),
							(gpointer)Ch);
				gtk_widget_add_events( Ch->page_view, (GdkEventMask) GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
			} else
				Ch->page_view = NULL;

		      // set up profile view
			if ( Ch->power && Ch->power->len ) {
				__signal_length = MAX( __signal_length,
						       Ch->power->len * PSZ);
				gtk_container_add( GTK_CONTAINER (Ch->vbox),
						   Ch->profile_view = gtk_drawing_area_new());
				g_object_set( G_OBJECT (Ch->profile_view),
					      "app-paintable", TRUE,
					      "height-request", 60,
					      NULL);
				g_signal_connect_after( Ch->profile_view, "expose-event",
							G_CALLBACK (daScoringFacProfileView_expose_event_cb),
							(gpointer)Ch);
				g_signal_connect_after( Ch->profile_view, "button-press-event",
							G_CALLBACK (daScoringFacProfileView_button_press_event_cb),
							(gpointer)Ch);
				g_signal_connect_after( Ch->profile_view, "scroll-event",
							G_CALLBACK (daScoringFacProfileView_scroll_event_cb),
							(gpointer)Ch);
				gtk_widget_add_events( Ch->profile_view, (GdkEventMask) GDK_BUTTON_PRESS_MASK);

				gtk_widget_modify_fg( Ch->profile_view, GTK_STATE_NORMAL, &__fg_power);
				gtk_widget_modify_bg( Ch->profile_view, GTK_STATE_NORMAL, &__bg_power);

			} else
				Ch->profile_view = NULL;
		}

	}

	gtk_statusbar_pop( GTK_STATUSBAR (sbMainStatusBar), agh_sb_context_id_DesignMeasurements);
	agh_set_cursor_busy( FALSE, wMainWindow);

	if ( !n_visible || !g_array_index (__channels, SChannelPresentation, agh_current_channel).expander )
		return FALSE;

//	printf( "total_pages = %d\n", __total_pages);


      // get scores
	g_array_set_size( __hypnogram, 0);
	agh_scores_get( agh_j_name->str, agh_e_name->str,
			__hypnogram, &__pagesize);

	if ( __pagesize != agh_pagesize_get_current() ) {
		GString *str = g_string_sized_new( 240);
		g_string_printf( str, "Existing hypnogram for this measurement was scored "
				 "with a page size (%d sec) that is different from the size currently "
				 "set as default (%d sec).\n\n"
				 "Do you want to convert the existing hypnogram to the current page size?  "
				 "(Else, the current page size will be set to %d sec.)",
				 __pagesize, agh_pagesize_get_current(), __pagesize);
		if ( agh_pop_question( str->str) == GTK_RESPONSE_YES ) {
			__scale_hypnogram( __pagesize, PSZ);
			agh_msmt_set_pagesize( agh_j_name->str, agh_e_name->str, PSZ);
			gtk_combo_box_set_active( GTK_COMBO_BOX (ePageSize),
						  agh_pagesize_item);
		} else {
			agh_pagesize_set_current( __pagesize);
			agh_collect_msmts_from_tree();
			agh_populate_mDesignMeasurements();
		}

		g_string_free( str, TRUE);
	}
	__total_pages = __signal_length / PSZ;

      // if there is no hypnogram in fact, agh_scores_get will have
      // shrunk the size of __hypnogram to 0: set it right again
	if ( __hypnogram->len != __total_pages )
		g_array_set_size( __hypnogram, __total_pages);


      // set up other controls
	gtk_spin_button_set_value( GTK_SPIN_BUTTON (eScoringFacCurrentPage),
				   __cur_page = __cur_page_app = 1);
	gtk_combo_box_set_active( GTK_COMBO_BOX (eScoringFacPageSize),
				  __pagesize_item = agh_pagesize_item);

	g_signal_emit_by_name( eScoringFacCurrentPage, "value-changed");
	g_signal_emit_by_name( eScoringFacPageSize, "changed");

	__repaint_score_stats();

//	gdk_window_set_debug_updates( TRUE);

	agh_populate_mDesignMeasurements();
	gtk_widget_queue_draw( tvDesignMeasurements);

	return TRUE;
}








// -------------------- Page

gboolean
cScoringFacPageViewExpander_activate_cb( GtkExpander *expander, gpointer userdata)
{
	g_signal_emit_by_name( eScoringFacCurrentPage, "value-changed");
}



gboolean
daScoringFacPageView_expose_event_cb( GtkWidget *wid, GdkEventExpose *event, gpointer userdata)
{
	SChannelPresentation *Ch = (SChannelPresentation*) userdata;
	if ( !Ch->visible || !gtk_expander_get_expanded( GTK_EXPANDER (Ch->expander)) || !Ch->signal->len )
		return TRUE;

	static GArray *lines = NULL;
	static PangoLayout *layout = NULL;

//	printf( "channel %s, ev->count = %d\n", ((SChannelPresentation*)Ch)->name, event->count);
	gint ht, wd;
	gdk_drawable_get_size( wid->window,
			       &wd, &ht);

	if ( !lines ) {
		lines = g_array_sized_new( FALSE, FALSE, wd, sizeof(GdkPoint));
		layout = gtk_widget_create_pango_layout( wid, "");
	}

	if ( lines->len < wd )
		g_array_set_size( lines, wd);

#define X(x) g_array_index (Ch->signal, gint16, x)
#define L(x) g_array_index (lines, GdkPoint, x)
	guint i;
	for ( i = 0; i < wd; i++ ) {
		L(i).x = i;
		L(i).y = - X( lroundf(((gfloat)i / wd + __cur_page_app) * Ch->samplerate * APSZ) )
			* Ch->signal_scale
			* (ht / Ch->display_scale)
			+ ht/2;
	}
#undef L
#undef X

	guint dpuV = 1 * (ht / Ch->display_scale);
	gdk_draw_line( wid->window, wid->style->fg_gc[GTK_STATE_NORMAL],
		       10, 10,
		       10, 10 + dpuV);

	gdk_draw_lines( wid->window, wid->style->fg_gc[GTK_STATE_NORMAL],
			(GdkPoint*)lines->data, lines->len);

	snprintf( __label_text, 50, "<b><small>1 \302\265V</small></b>");
	pango_layout_set_markup( layout, __label_text, -1);
	gdk_draw_layout( wid->window, wid->style->fg_gc[GTK_STATE_NORMAL],
			 15, 10,
			 layout);

	for ( i = 0; i < __pagesize_ticks[__pagesize_item]; i++ ) {
		guint tick_pos = i * APSZ / __pagesize_ticks[__pagesize_item];
		snprintf( __label_text, 23, "<small>%2d</small>", tick_pos);
		pango_layout_set_markup( layout, __label_text, -1);
		gdk_draw_layout( wid->window, wid->style->fg_gc[GTK_STATE_NORMAL],
				 i * wd / __pagesize_ticks[__pagesize_item] + 5,
				 120 - 15,
				 layout);
		gdk_draw_line( wid->window, wid->style->fg_gc[GTK_STATE_NORMAL],
			       i * wd / __pagesize_ticks[__pagesize_item], 110,
			       i * wd / __pagesize_ticks[__pagesize_item], 100);
	}

	if ( Ch->track ) {
		guint	cur_page_start_s =  __cur_page_app      * APSZ,
			cur_page_end_s   = (__cur_page_app + 1) * APSZ;
		for ( i = cur_page_start_s; i < cur_page_end_s; i++ ) {
			if ( g_array_index (Ch->track, gchar, i) ) {
				snprintf( __label_text, 20, "<b>\342\234\230</b>");
				pango_layout_set_markup( layout, __label_text, -1);
				gdk_draw_layout( wid->window, wid->style->fg_gc[GTK_STATE_NORMAL],
						 (i % APSZ + .5)
						 / APSZ * wd,
						 15,
						 layout);
			}
		}
		snprintf( __label_text, 40, "<small><i>%4.2f %% dirty</i></small>", Ch->dirty_percent);
		pango_layout_set_markup( layout, __label_text, -1);
		gdk_draw_layout( wid->window, wid->style->fg_gc[GTK_STATE_NORMAL],
				 wd - 70,
				 ht - 15,
				 layout);
	}

	return TRUE;
}




gboolean
daScoringFacPageView_button_press_event_cb( GtkWidget *wid, GdkEventButton *event, gpointer userdata)
{
	SChannelPresentation *Ch = (SChannelPresentation*) userdata;

	gint wd, ht;
	guint h;
	gdk_drawable_get_size( wid->window, &wd, &ht);

	switch ( event->button ) {
	case 2:
		if ( event->state & GDK_CONTROL_MASK )
			for ( h = 0; h < __channels->len; h++ )
				g_array_index (__channels, SChannelPresentation, h).display_scale = 20;
		else
			Ch->display_scale = 20;
		gtk_widget_queue_draw( wid);
	    break;
	case 1:
	case 3:
		if ( Ch->track ) {
			__marking_in_widget = wid;
			__mark_start = (__cur_page_app + event->x / wd) * APSZ;
		}
	    break;
	}


	return TRUE;
}




gboolean
daScoringFacPageView_button_release_event_cb( GtkWidget *wid, GdkEventButton *event, gpointer userdata)
{
	SChannelPresentation *Ch = (SChannelPresentation*) userdata;

	gint wd, ht;
	gdk_drawable_get_size( wid->window, &wd, &ht);

	if ( wid != __marking_in_widget || !Ch->track )
		return TRUE;
	switch ( event->button ) {
	case 1:
		__mark_region( __mark_start, (__cur_page_app + event->x / wd) * APSZ, Ch, 1);
	    break;
	case 3:
		__mark_region( __mark_start, (__cur_page_app + event->x / wd) * APSZ, Ch, 0);
	    break;
	}

	if ( event->state & GDK_MOD1_MASK ) {
		agh_msmt_get_power( agh_j_name->str, agh_e_name->str, Ch->name,
				    Ch->power, Ch->track, Ch->from, Ch->upto, &Ch->max_value, TRUE);
		gtk_widget_queue_draw( Ch->profile_view);
		Ch->new_marks = FALSE;
	}

	return TRUE;
}




static void
__mark_region( guint x1, guint x2, SChannelPresentation* Ch, gchar value)
{
	if ( x1 > x2 ) {
		guint _ = x1;
		x1 = x2, x2 = _;
	}
	if ( x1 < 0 )
		x1 = 0;
	if ( x2 >= __signal_length )
		x2 = __signal_length-1;

	guint s;
	for ( s = x1; s <= x2; s++ )
		g_array_index (Ch->track, gchar, s) = value;

	agh_msmt_update_track( agh_j_name->str, agh_e_name->str, Ch->name,
			       Ch->track);

	Ch->new_marks = TRUE;
	__calculate_dirty_percent( Ch);

	gtk_widget_queue_draw( Ch->page_view);
}


void
iScoreArtifactsApply_activate_cb()
{
	agh_set_cursor_busy( TRUE, wScoringFacility);
	guint h;
	for ( h = 0; h < __channels->len; h++ ) {
		SChannelPresentation *Ch = &g_array_index (__channels, SChannelPresentation, h);
		if ( Ch->track && Ch->new_marks ) {
			agh_msmt_get_power( agh_j_name->str, agh_e_name->str, Ch->name,
					    Ch->power, Ch->track, Ch->from, Ch->upto, &Ch->max_value, TRUE);
			gtk_widget_queue_draw( Ch->profile_view);
			Ch->new_marks = FALSE;
		}
	}
	agh_set_cursor_busy( FALSE, wScoringFacility);
}

void
iScoreArtifactsClear_activate_cb()
{
	if ( agh_pop_question( "All marked artifacts will be lost in all channels. Continue?") != GTK_RESPONSE_YES )
		return;

	agh_set_cursor_busy( TRUE, wScoringFacility);
	guint h;
	for ( h = 0; h < __channels->len; h++ ) {
		SChannelPresentation *Ch = &g_array_index (__channels, SChannelPresentation, h);
		if ( Ch->track ) {
			memset( Ch->track->data, 0, Ch->track->len);
			agh_msmt_update_track( agh_j_name->str, agh_e_name->str, Ch->name,
					       Ch->track);
			agh_msmt_get_power( agh_j_name->str, agh_e_name->str, Ch->name,
					    Ch->power, Ch->track, Ch->from, Ch->upto, &Ch->max_value,
					    TRUE);
			gtk_widget_queue_draw( Ch->profile_view);
			gtk_widget_queue_draw( Ch->page_view);
			Ch->new_marks = FALSE;
		}
	}
	agh_set_cursor_busy( FALSE, wScoringFacility);
}




gboolean
daScoringFacPageView_scroll_event_cb( GtkWidget *wid, GdkEventScroll *event, gpointer userdata)
{
	SChannelPresentation *Ch = (SChannelPresentation*) userdata;

	guint h;
	switch ( event->direction ) {
	case GDK_SCROLL_DOWN:
		if ( Ch->display_scale < 250 )
			Ch->display_scale *= 1.5;
	    break;
	case GDK_SCROLL_UP:
		if ( Ch->display_scale > .001 )
			Ch->display_scale /= 1.5;
	    break;
	}

	if ( event->state & GDK_CONTROL_MASK ) {
		for ( h = 0; h < __channels->len; h++ )
			g_array_index (__channels, SChannelPresentation, h).display_scale =
				Ch->display_scale;
		gtk_widget_queue_draw( cScoringFacPageViews);
	} else
		gtk_widget_queue_draw( wid);

	return TRUE;
}










// -------------------- Profile

gboolean
daScoringFacProfileView_expose_event_cb( GtkWidget *wid, GdkEventExpose *event, gpointer userdata)
{
	SChannelPresentation *Ch = (SChannelPresentation*) userdata;
	if ( !Ch->visible || !gtk_expander_get_expanded( GTK_EXPANDER (Ch->expander)) )
		return TRUE;

	static GArray *lines = NULL;
	static PangoLayout *layout = NULL;

	gint ht, wd;
	gdk_drawable_get_size( wid->window,
			       &wd, &ht);
	if ( !lines ) {
		lines = g_array_sized_new( FALSE, FALSE, wd, sizeof(GdkPoint));
		layout = gtk_widget_create_pango_layout( wid, "");
	}
	if ( lines->len < wd )
		g_array_set_size( lines, wd);

#define L(x) g_array_index (lines, GdkPoint, x)
	guint i;
	for ( i = 0; i < wd; i++ ) {
		guint i_real = (gfloat)i / wd * Ch->power->len;
		L(i).x = i;
		L(i).y = - (g_array_index (Ch->power, double, i_real)
			    * agh_get_ppuV2())
			+ ht;
	}
#undef L

	gdk_draw_lines( wid->window, wid->style->fg_gc[GTK_STATE_NORMAL],
			(GdkPoint*)lines->data, lines->len);

	gdk_draw_line( wid->window, __gc_power,
		       10, 10,
		       10, 10 + agh_get_ppuV2()/10);
	snprintf( __label_text, 50, "<b><small>0.1 \302\265V\302\262</small></b>");
	pango_layout_set_markup( layout, __label_text, -1);
	gdk_draw_layout( wid->window, wid->style->fg_gc[GTK_STATE_NORMAL],
			 15, 15,
			 layout);

	snprintf( __label_text, 23, "<b>%g - %g</b> Hz", Ch->from, Ch->upto);
	pango_layout_set_markup( layout, __label_text, -1);
	gdk_draw_layout( wid->window, wid->style->fg_gc[GTK_STATE_NORMAL],
			 wd - 50, 10,
			 layout);

	return TRUE;
}






gboolean
daScoringFacProfileView_scroll_event_cb( GtkWidget *wid, GdkEventScroll *event, gpointer userdata)
{
	SChannelPresentation *Ch = (SChannelPresentation*) userdata;

	switch ( event->direction ) {
	case GDK_SCROLL_DOWN:
		if ( Ch->from > 0 )
			Ch->from--, Ch->upto--;
	    break;
	case GDK_SCROLL_UP:
		if ( Ch->upto < 10 )
			Ch->from++, Ch->upto++;
	    break;
	}

	agh_msmt_get_power( agh_j_name->str, agh_e_name->str, Ch->name,
			    Ch->power, Ch->track, Ch->from, Ch->upto, &Ch->max_value, FALSE);

	gtk_widget_queue_draw( wid);

	return TRUE;
}









gboolean
daScoringFacProfileView_button_press_event_cb( GtkWidget *wid, GdkEventButton *event, gpointer userdata)
{
	SChannelPresentation *Ch = (SChannelPresentation*) userdata;

	gint wd, ht;
	gdk_drawable_get_size( wid->window, &wd, &ht);

	switch ( event->button ) {
	case 1:
		__cur_page = (event->x / wd) * __total_pages;
		__cur_page_app = P2AP (__cur_page);
		gtk_spin_button_set_value( GTK_SPIN_BUTTON (eScoringFacCurrentPage), __cur_page_app+1);
	    break;
	case 3:
	{
		guint i_real = (guint)((gdouble)event->x / wd * Ch->power->len);
		agh_set_ppuV2( ht * 0.75 / g_array_index (Ch->power, double, i_real));
		gtk_widget_queue_draw( wid);
	}
	    break;
	}

	return TRUE;
}














// -------------------- Hypnogram

gboolean
daScoringFacHypnogram_expose_event_cb( GtkWidget *wid, GdkEventExpose *event, gpointer unused)
{
	static GArray *lines = NULL;
	if ( !lines )
		lines = g_array_new( FALSE, FALSE, sizeof(GdkPoint));

	gint ht, wd;
	gdk_drawable_get_size( wid->window,
			       &wd, &ht);

	guint i, ii;
	for ( i = 1; i < 8; i++ )
		gdk_draw_line( wid->window, __gc_score[i],
			       0,  __score_hypn_depth[i],
			       wd, __score_hypn_depth[i]);


#define X(x) g_array_index (__hypnogram, gchar, x)
	for ( i = 0; i < __hypnogram->len; i++ ) {
		gchar c;
		if ( (c = X(i)) ) {
			gint y = __score_hypn_depth[ SCOREID(c) ];
			gdk_draw_line( wid->window, __gc_score_line,
				       lroundf( (gfloat) i   /__hypnogram->len * wd), y,
				       lroundf( (gfloat)(i+1)/__hypnogram->len * wd), y);
//		gdk_draw_lines( wid->window, __gc_score_line,
//			       (GdkPoint*)lines->data, lines->len);
		}
	}
#undef X

	gdk_draw_rectangle( wid->window, __gc_score_cursor,
			    TRUE,
			    (float) __cur_page_app / P2AP (__total_pages) * wd,  0,
			    ceil( (gfloat)  1 / P2AP (__total_pages) * wd), ht);

	return TRUE;
}




gboolean
daScoringFacHypnogram_button_press_event_cb( GtkWidget *wid, GdkEventButton *event, gpointer unused)
{
	gint wd;
	gdk_drawable_get_size( wid->window, &wd, NULL);

	switch ( event->button ) {
	case 1:
		__cur_page = (event->x / wd) * __total_pages;
		__cur_page_app = P2AP (__cur_page);
		gtk_spin_button_set_value( GTK_SPIN_BUTTON (eScoringFacCurrentPage), __cur_page_app+1);
	    break;
	case 3:
	    break;
	}
	return TRUE;
}





static void
__repaint_score_stats()
{
	snprintf( __label_text, 29, "<b>%3.1f</b> %% scored", __percent_scored());
	gtk_label_set_markup( GTK_LABEL (lScoringFacPercentScored), __label_text);

	snprintf( __label_text, 29, "<small>%3.1f</small> %%", __percent_NREM());
	gtk_label_set_markup( GTK_LABEL (lScoreStatsNREMPercent), __label_text);

	snprintf( __label_text, 29, "<small>%3.1f</small> %%", __percent_REM());
	gtk_label_set_markup( GTK_LABEL (lScoreStatsREMPercent), __label_text);

	snprintf( __label_text, 29, "<small>%3.1f</small> %%", __percent_Wake());
	gtk_label_set_markup( GTK_LABEL (lScoreStatsWakePercent), __label_text);
}



static void
__do_score_forward( gchar score_ch)
{
	if ( __cur_page < __total_pages ) {
		g_array_index (__hypnogram, gchar, __cur_page) = score_ch;
		__cur_page++;
		__cur_page_app++; //  = P2AP (__cur_page);
		gtk_spin_button_set_value( GTK_SPIN_BUTTON (eScoringFacCurrentPage), __cur_page_app+1);
		__repaint_score_stats();
	}
}


void bScoreClear_clicked_cb()  { __do_score_forward( '\0'); }
void bScoreNREM1_clicked_cb()  { __do_score_forward(  '1'); }
void bScoreNREM2_clicked_cb()  { __do_score_forward(  '2'); }
void bScoreNREM3_clicked_cb()  { __do_score_forward(  '3'); }
void bScoreNREM4_clicked_cb()  { __do_score_forward(  '4'); }
void bScoreREM_clicked_cb()    { __do_score_forward(  '5'); }
void bScoreWake_clicked_cb()   { __do_score_forward(  '6'); }
void bScoreMVT_clicked_cb()    { __do_score_forward(  '7'); }





void
bScoringFacForward_clicked_cb()
{
	if ( __cur_page_app * APSZ < __signal_length ) {
		__cur_page_app++;
		__cur_page = AP2P (__cur_page_app);
		gtk_spin_button_set_value( GTK_SPIN_BUTTON (eScoringFacCurrentPage), __cur_page_app+1);
	}
}

void
bScoringFacBack_clicked_cb()
{
	if ( __cur_page_app ) {
		__cur_page_app--;
		__cur_page = AP2P (__cur_page_app);
		gtk_spin_button_set_value( GTK_SPIN_BUTTON (eScoringFacCurrentPage), __cur_page_app+1);
	}
}



void
eScoringFacPageSize_changed_cb()
{
	guint cur_pos = __cur_page_app * APSZ;
	__pagesize_item = gtk_combo_box_get_active( GTK_COMBO_BOX (eScoringFacPageSize));
	__cur_page_app = cur_pos / APSZ;
//	__cur_page = AP2P (__cur_page_app); // shouldn't change

	gtk_spin_button_set_value( GTK_SPIN_BUTTON (eScoringFacCurrentPage), __cur_page_app+1);
	gtk_spin_button_set_range( GTK_SPIN_BUTTON (eScoringFacCurrentPage), 1, P2AP (__total_pages));

	snprintf( __label_text, 19, "of %d", P2AP (__total_pages));
	gtk_label_set_text( GTK_LABEL (lScoringFacTotalPages), __label_text);

	gboolean pagesize_is_right = (APSZ == PSZ);
	gtk_widget_set_sensitive( bScoreClear, pagesize_is_right);
	gtk_widget_set_sensitive( bScoreNREM1, pagesize_is_right);
	gtk_widget_set_sensitive( bScoreNREM2, pagesize_is_right);
	gtk_widget_set_sensitive( bScoreNREM3, pagesize_is_right);
	gtk_widget_set_sensitive( bScoreNREM4, pagesize_is_right);
	gtk_widget_set_sensitive( bScoreREM,   pagesize_is_right);
	gtk_widget_set_sensitive( bScoreWake,  pagesize_is_right);
	gtk_widget_set_sensitive( bScoreMVT,   pagesize_is_right);

	if ( !pagesize_is_right ) {
		guint h;
		for ( h = 0; h < __channels->len; h++ ) {
			SChannelPresentation *Ch = &g_array_index( __channels, SChannelPresentation, h);
			if ( Ch->visible ) {
				g_signal_handler_block( Ch->page_view, Ch->expose_handler_id);
//				gtk_widget_modify_fg( Ch->page_view, GTK_STATE_NORMAL, &__fg_score[AGH_SCORE_NONE]);
				gtk_widget_modify_bg( Ch->page_view, GTK_STATE_NORMAL, &__bg_score[AGH_SCORE_NONE]);
				g_signal_handler_unblock( Ch->page_view, Ch->expose_handler_id);
			}
		}
	}
	g_signal_emit_by_name( eScoringFacCurrentPage, "value-changed");
}





void
eScoringFacCurrentPage_value_changed_cb()
{
	__cur_page_app = gtk_spin_button_get_value( GTK_SPIN_BUTTON (eScoringFacCurrentPage)) - 1;
//	CLAMP (__cur_page_app, 0, P2AP (__total_pages-1));
	guint __cur_pos = __cur_page_app * APSZ;
	__cur_pos_hr  =  __cur_pos / 3600;
	__cur_pos_min = (__cur_pos - __cur_pos_hr * 3600) / 60;
	__cur_pos_sec =  __cur_pos % 60;

	__cur_stage = SCOREID( g_array_index (__hypnogram, gchar,
					      __cur_pos / PSZ));

	guint h;
	for ( h = 0; h < __channels->len; h++ ) {
		SChannelPresentation *Ch = &g_array_index( __channels, SChannelPresentation, h);
		if ( Ch->visible && gtk_expander_get_expanded( GTK_EXPANDER (Ch->expander)) ) {
			if ( Ch->page_view ) {
				if ( agh_pagesize_item == __pagesize_item && Ch->page_view ) {
					g_signal_handler_block( Ch->page_view, Ch->expose_handler_id);
					gtk_widget_modify_fg( Ch->page_view, GTK_STATE_NORMAL, &__fg_score[__cur_stage]);
					gtk_widget_modify_bg( Ch->page_view, GTK_STATE_NORMAL, &__bg_score[__cur_stage]);
					g_signal_handler_unblock( Ch->page_view, Ch->expose_handler_id);
				}
				gtk_widget_queue_draw( Ch->page_view);
			}
		}
	}
	snprintf( __label_text, 29, "<b><big>%s</big></b>", __score_names[ __cur_stage ]);
	gtk_label_set_markup( GTK_LABEL (lScoringFacCurrentStage), __label_text);

	snprintf( __label_text, 39, "%2d:%02d:%02d", __cur_pos_hr, __cur_pos_min, __cur_pos_sec);
	gtk_label_set_text( GTK_LABEL (lScoringFacCurrentPos), __label_text);

	gtk_widget_queue_draw( daScoringFacHypnogram);
}










void
iScoreFileImport_activate_cb()
{
	GtkWidget *f_chooser = gtk_file_chooser_dialog_new( "Import Scores",
							    NULL,
							    GTK_FILE_CHOOSER_ACTION_OPEN,
							    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
							    GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
							    NULL);
	if ( gtk_dialog_run( GTK_DIALOG (f_chooser)) == GTK_RESPONSE_ACCEPT ) {
		gchar *fname = gtk_file_chooser_get_filename( GTK_FILE_CHOOSER (f_chooser));
		agh_scores_import( agh_j_name->str, agh_e_name->str, fname);
	}
	gtk_widget_destroy( f_chooser);
	agh_scores_get( agh_j_name->str, agh_e_name->str,
			__hypnogram, NULL);
	g_signal_emit_by_name( eScoringFacCurrentPage, "value-changed");
	__repaint_score_stats();
}

void
iScoreFileExport_activate_cb()
{
	agh_scores_put( agh_j_name->str, agh_e_name->str,
			__hypnogram, PSZ);
	GtkWidget *f_chooser = gtk_file_chooser_dialog_new( "Export Scores",
							    NULL,
							    GTK_FILE_CHOOSER_ACTION_SAVE,
							    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
							    GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
							    NULL);
	if ( gtk_dialog_run( GTK_DIALOG (f_chooser)) == GTK_RESPONSE_ACCEPT ) {
		gchar *fname = gtk_file_chooser_get_filename( GTK_FILE_CHOOSER (f_chooser));
		agh_scores_export( agh_j_name->str, agh_e_name->str, fname);
	}
	gtk_widget_destroy( f_chooser);
}




void
iScoreDoneAccept_activate_cb()
{
	gboolean need_artifact_save = FALSE;
	guint h;
	for ( h = 0; h < __channels->len; h++ ) {
		SChannelPresentation *Ch = &g_array_index (__channels, SChannelPresentation, h);
		if ( Ch->track && Ch->new_marks ) {
			need_artifact_save = TRUE;
			break;
		}
	}
	if ( need_artifact_save && agh_pop_question( "You have left some artifacts marked which have not been applied to the power profile.  "
						     "These will be saved now but not reflected in the profile.\n\n"
						     "Redo FFT with such markings now?") == GTK_RESPONSE_YES ) {
		agh_set_cursor_busy( TRUE, wScoringFacility);
		for ( h = 0; h < __channels->len; h++ ) {
			SChannelPresentation *Ch = &g_array_index (__channels, SChannelPresentation, h);
			if ( Ch->track ) {
				agh_msmt_update_track( agh_j_name->str, agh_e_name->str, Ch->name,
						       Ch->track);
				agh_msmt_get_power( agh_j_name->str, agh_e_name->str, Ch->name,
						    Ch->power, Ch->track, Ch->from, Ch->upto, &Ch->max_value,
						    TRUE);
			}
		}
		agh_set_cursor_busy( FALSE, wScoringFacility);
	}

	agh_scores_put( agh_j_name->str, agh_e_name->str,
			__hypnogram, PSZ);
	gtk_widget_hide( wScoringFacility);
}



void
iScoreDoneDiscard_activate_cb()
{
	gboolean need_artifact_save = FALSE;
	guint h;
	for ( h = 0; h < __channels->len; h++ ) {
		SChannelPresentation *Ch = &g_array_index (__channels, SChannelPresentation, h);
		if ( Ch->track && Ch->new_marks ) {
			need_artifact_save = TRUE;
			break;
		}
	}
	if ( need_artifact_save && agh_pop_question( "Also discard artifact markings?") != GTK_RESPONSE_YES ) {
		agh_set_cursor_busy( TRUE, wScoringFacility);
		for ( h = 0; h < __channels->len; h++ ) {
			SChannelPresentation *Ch = &g_array_index (__channels, SChannelPresentation, h);
			if ( Ch->track ) {
				agh_msmt_update_track( agh_j_name->str, agh_e_name->str, Ch->name,
						       Ch->track);
				if ( agh_pop_question( "Apply to power profile now?") == GTK_RESPONSE_YES )
					agh_msmt_get_power( agh_j_name->str, agh_e_name->str, Ch->name,
							    Ch->power, Ch->track, Ch->from, Ch->upto, &Ch->max_value,
							    TRUE);
			}
		}
		agh_set_cursor_busy( FALSE, wScoringFacility);
	}

	gtk_widget_hide( wScoringFacility);
}


void
iScoreDoneRevert_activate_cb()
{
	agh_scores_get( agh_j_name->str, agh_e_name->str, __hypnogram, NULL);

	g_signal_emit_by_name( eScoringFacCurrentPage, "value-changed");
}

void
iScoreDoneClearAll_activate_cb()
{
	g_array_set_size( __hypnogram, 0);
	g_array_set_size( __hypnogram, __signal_length / PSZ);

	g_signal_emit_by_name( eScoringFacCurrentPage, "value-changed");

	snprintf( __label_text, 29, "<b>%3.1f</b> %% scored", __percent_scored());
	gtk_label_set_markup( GTK_LABEL (lScoringFacPercentScored), __label_text);
}




// EOF
