// ;-*-C-*- *  Time-stamp: "2008-06-29 00:45:12 hmmr"

/***************************************************************************
 *            agh_ui-simulations.c
 *
 *  Copyright  2008  Andrei Zavada
 *  Email johnhommer@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <glade/glade.h>
#include "agh_iface.hh"
#include "agh_ui.h"



GtkTreeStore
	*agh_mSimulations;

GtkWidget
	*tvSimulations,
	*eSimulationsSession,
	*eSimulationsChannel;

static GtkWidget
	*eOperatingRangeLowerBound,
	*eOperatingRangeWidth,
	*lOperatingRange,

	*iBatchRunAllChannels,
	*iBatchRunAllSessions,
	*iBatchRunIterateRanges,
	*iBatchRunRedoOption[AGH_BATCHRUN_N_REDO_OPTIONS],

	*wBatchRunProgress,
	*rBatchRunPercentComplete,
	*lBatchRunStatus,
	*bBatchRunStop;






void eSimulationsSession_changed_cb();
void eSimulationsChannel_changed_cb();


static const gchar* const __agh_tunable_column_names[] = {
	"S\342\202\200", "S\342\210\236",
	"fc(REM)",	 "fc(Wake)",
	"t\342\206\227", "t\342\206\230",
	"rate const.",	 "rise rate",
	"gain const.",
	"gc2", "gc3", "gc4", "gc5", "gc6"
};

static gchar __label_text[60];


gint
agh_construct_Simulations( GladeXML *xml)
{
	GtkCellRenderer *renderer;

     // ------------- tvSimulations
	if ( !(tvSimulations = glade_xml_get_widget( xml, "tvSimulations")) )
		return -1;

	gtk_tree_view_set_model( GTK_TREE_VIEW (tvSimulations),
				 GTK_TREE_MODEL (agh_mSimulations));


	g_object_set( G_OBJECT (tvSimulations),
		      "expander-column", 0,
		      "enable-tree-lines", FALSE,
		      "headers-clickable", FALSE,
		      NULL);
	g_signal_connect( tvSimulations, "map", G_CALLBACK (gtk_tree_view_expand_all), NULL);

	renderer = gtk_cell_renderer_text_new();
	g_object_set( G_OBJECT (renderer),
		      "editable", FALSE,
		      NULL);
	gtk_tree_view_insert_column_with_attributes( GTK_TREE_VIEW (tvSimulations),
						     -1, "Subject", renderer,
						     "text", 0,
						     NULL);
	guint t;
	for ( t = 0; t < AGH_VNUM + AGH_EGC_MAX-1; t++ ) {
		renderer = gtk_cell_renderer_text_new();
		g_object_set( G_OBJECT (renderer),
			      "editable", FALSE,
			      "xalign", 1.,
			      NULL);
		g_object_set_data( G_OBJECT (renderer), "column", GINT_TO_POINTER (t+1));
		GtkTreeViewColumn *col =
			gtk_tree_view_column_new_with_attributes( __agh_tunable_column_names[t], renderer,
								  "text", t+1,
								  "visible", 10+AGH_EGC_MAX-1,
								  NULL);
		gtk_tree_view_column_set_alignment( col, .9);
		gtk_tree_view_column_set_expand( col, TRUE);
		gtk_tree_view_append_column( GTK_TREE_VIEW (tvSimulations), col);
	}

     // ------------- eOperatingRange*
	if ( !(eOperatingRangeLowerBound = glade_xml_get_widget( xml, "eOperatingRangeLowerBound")) ||
	     !(eOperatingRangeWidth      = glade_xml_get_widget( xml, "eOperatingRangeWidth"))      ||
	     !(lOperatingRange           = glade_xml_get_widget( xml, "lOperatingRange")) )
		return -1;


     // ------------- eSimulations{Session,Channel}
	if ( !(eSimulationsSession = glade_xml_get_widget( xml, "eSimulationsSession")) ||
	     !(eSimulationsChannel = glade_xml_get_widget( xml, "eSimulationsChannel")) )
		return -1;

	gtk_combo_box_set_model( GTK_COMBO_BOX (eSimulationsSession),
				 GTK_TREE_MODEL (agh_mDesignSetupSessions));
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (eSimulationsSession), renderer, FALSE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (eSimulationsSession), renderer,
					"text", 0,
					NULL);

	gtk_combo_box_set_model( GTK_COMBO_BOX (eSimulationsChannel),
				 GTK_TREE_MODEL (agh_mDesignSetupChannels));
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (eSimulationsChannel), renderer, FALSE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (eSimulationsChannel), renderer,
					"text", 0,
					NULL);

      // ---------- iBatch*
	if ( !(iBatchRunAllChannels	= glade_xml_get_widget( xml, "iBatchRunAllChannels")) ||
	     !(iBatchRunAllSessions	= glade_xml_get_widget( xml, "iBatchRunAllSessions")) ||
	     !(iBatchRunIterateRanges	= glade_xml_get_widget( xml, "iBatchRunIterateRanges")) ||
	     !(iBatchRunRedoOption[AGH_BATCHRUN_REDOSKIP]	= glade_xml_get_widget( xml, "iBatchRunRedoSkip")) ||
	     !(iBatchRunRedoOption[AGH_BATCHRUN_REDOFAILED]	= glade_xml_get_widget( xml, "iBatchRunRedoIfFailed")) ||
	     !(iBatchRunRedoOption[AGH_BATCHRUN_REDOSUCCEEDED]	= glade_xml_get_widget( xml, "iBatchRunRedoIfSucceeded")) ||
	     !(iBatchRunRedoOption[AGH_BATCHRUN_REDOALWAYS]	= glade_xml_get_widget( xml, "iBatchRunRedoAlways")) ||
	     !(wBatchRunProgress	= glade_xml_get_widget( xml, "wBatchRunProgress")) ||
	     !(lBatchRunStatus		= glade_xml_get_widget( xml, "lBatchRunStatus")) ||
	     !(rBatchRunPercentComplete	= glade_xml_get_widget( xml, "rBatchRunPercentComplete")) ||
	     !(bBatchRunStop		= glade_xml_get_widget( xml, "bBatchRunStop")) )
		return -1;

	return 0;
}



void
eSimulationsToken_changed_cb( GtkWidget *w, gpointer u)
{
}



void
eSimulationsSession_changed_cb()
{
	agh_current_session = gtk_combo_box_get_active( GTK_COMBO_BOX (eSimulationsSession));
	__agh_propagate_current_session_change();

	agh_populate_mSimulations();
}


void
eSimulationsChannel_changed_cb()
{
	agh_current_channel = gtk_combo_box_get_active( GTK_COMBO_BOX (eSimulationsChannel));
	__agh_propagate_current_channel_change();

	agh_populate_mSimulations();
}








void
cOperatingRange_map_cb()
{
	snprintf( __label_text, 59, "<b>%4.2f\342\200\223%4.2f</b> Hz",
		  agh_sim_get_operating_range_from(),
		  agh_sim_get_operating_range_upto());
	gtk_label_set_markup( GTK_LABEL (lOperatingRange), __label_text);

	gtk_spin_button_set_value( GTK_SPIN_BUTTON (eOperatingRangeLowerBound),
				   agh_sim_get_operating_range_low_bound());
	gtk_spin_button_set_value( GTK_SPIN_BUTTON (eOperatingRangeWidth),
				   agh_sim_get_operating_range_width());
}





// one and the same callback for either spinbutton
void
eOperatingRange_value_changed_cb( GtkSpinButton *unused, gpointer unused_either)
{
	gfloat from = gtk_spin_button_get_value( GTK_SPIN_BUTTON (eOperatingRangeLowerBound));
	agh_sim_set_operating_range_from( from);
	agh_sim_set_operating_range_upto( from
					  + gtk_spin_button_get_value( GTK_SPIN_BUTTON (eOperatingRangeWidth)));

	snprintf( __label_text, 59, "<b>%4.2f\342\200\223%4.2f</b> Hz",
		  from, agh_sim_get_operating_range_upto());
	gtk_label_set_markup( GTK_LABEL (lOperatingRange), __label_text);

	agh_populate_mSimulations();
}













void
pmRunBatch_map_cb()
{  gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM (iBatchRunAllChannels),   agh_sim_runbatch_include_all_channels);
   gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM (iBatchRunAllSessions),   agh_sim_runbatch_include_all_sessions);
   gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM (iBatchRunIterateRanges), agh_sim_runbatch_iterate_ranges);
   gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM (iBatchRunRedoOption[agh_sim_runbatch_redo_option]), TRUE);	}


void iBatchRunAllChannels_toggled_cb( GtkCheckMenuItem *item, gpointer unused)
{  agh_sim_runbatch_include_all_channels = gtk_check_menu_item_get_active( item);  }

void iBatchRunAllSessions_toggled_cb( GtkCheckMenuItem *item, gpointer unused)
{  agh_sim_runbatch_include_all_sessions = gtk_check_menu_item_get_active( item);  }

void iBatchRunIterateRanges_toggled_cb( GtkCheckMenuItem *item, gpointer unused)
{  agh_sim_runbatch_iterate_ranges = gtk_check_menu_item_get_active( item);  }


void iBatchRunRedoSkip_toggled_cb( GtkCheckMenuItem *item, gpointer unused)
{  if ( gtk_check_menu_item_get_active( item) )  agh_sim_runbatch_redo_option = AGH_BATCHRUN_REDOSKIP;  }

void iBatchRunRedoIfFailed_toggled_cb( GtkCheckMenuItem *item, gpointer unused)
{  if ( gtk_check_menu_item_get_active( item) )  agh_sim_runbatch_redo_option = AGH_BATCHRUN_REDOFAILED;  }

void iBatchRunRedoIfSucceeded_toggled_cb( GtkCheckMenuItem *item, gpointer unused)
{  if ( gtk_check_menu_item_get_active( item) )  agh_sim_runbatch_redo_option = AGH_BATCHRUN_REDOSUCCEEDED;  }

void iBatchRunRedoAlways_toggled_cb( GtkCheckMenuItem *item, gpointer unused)
{  if ( gtk_check_menu_item_get_active( item) )  agh_sim_runbatch_redo_option = AGH_BATCHRUN_REDOALWAYS;  }








void
bSimulationRun_clicked_cb()
{
	static GString *title = NULL;
	if ( !title )
		title = g_string_sized_new( 120);

	GtkTreeSelection *selection = gtk_tree_view_get_selection( GTK_TREE_VIEW (tvSimulations));
	GtkTreeModel *model;
	GList *paths = gtk_tree_selection_get_selected_rows( selection, &model);
	if ( !paths )
		return;
	GtkTreePath *path = (GtkTreePath*) g_list_nth_data( paths, 0);

	if ( gtk_tree_path_get_depth( path) > 1 ) {
		gchar *j_name;
		GtkTreeIter iter;
		gtk_tree_model_get_iter( GTK_TREE_MODEL (agh_mSimulations), &iter, path);
		gtk_tree_model_get( GTK_TREE_MODEL (agh_mSimulations), &iter,
				    0, &j_name,
				    -1);
		g_string_assign( agh_j_name, j_name);

		if ( __agh_prepare_model_run() ) {
			gtk_widget_show_all( wModelRun);
			g_string_printf( title, "Simulation: %s %s", agh_j_name->str, agh_get_current_channel_name());
			gtk_window_set_title( GTK_WINDOW (wModelRun),
					      title->str);
		}
		g_free( j_name);
	}

	gtk_tree_path_free( path);
	g_list_free( paths);
}





void
bSimulationExport_clicked_cb()
{
	GtkWidget *f_chooser = gtk_file_chooser_dialog_new( "Export Simulation Details",
							    NULL,
							    GTK_FILE_CHOOSER_ACTION_SAVE,
							    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
							    GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
							    NULL);
	if ( gtk_dialog_run( GTK_DIALOG (f_chooser)) == GTK_RESPONSE_ACCEPT ) {
		GString *fname = g_string_new( gtk_file_chooser_get_filename( GTK_FILE_CHOOSER (f_chooser)));
		if ( !g_str_has_suffix( fname->str, ".tsv") && !g_str_has_suffix( fname->str, ".TSV") )
			g_string_append_printf( fname, ".tsv");
		agh_sim_export_tsv( fname->str);
	}
	gtk_widget_destroy( f_chooser);
}






static gboolean __interrupt_batch;

void
bBatchRunStop_activate_cb()
{
	gtk_widget_set_sensitive( bBatchRunStop, FALSE);
	__interrupt_batch = TRUE;
}




void
iBatchRun_activate_cb()
{
	__interrupt_batch = FALSE;

	agh_set_cursor_busy( TRUE, wMainWindow);
	gtk_widget_show_all( wBatchRunProgress);

	void   *R;
	guint	__cur_session = agh_current_session,
		__cur_channel = agh_current_channel,
		d, h, j, g,
		total_runs, cur_run = 0;

	gchar	label_text[60];

	for ( g = 0, total_runs = 0; g < agh_get_n_groups(); g++ )
		total_runs += agh_get_n_subjects_in_group(g);
	total_runs *= (agh_get_n_sessions() * agh_get_n_channels());

	for ( g = 0; g < agh_get_n_groups(); g++ )
		for ( j = 0; j < agh_get_n_subjects_in_group(g); j++ ) {
			const gchar
				*j_name = agh_get_nth_subject_name_in_group(g,j),
				*g_name = agh_get_subject_group( j_name);

			for ( d = 0; d < agh_get_n_sessions(); d++ ) {
				agh_current_session = d;
				if ( !agh_sim_runbatch_include_all_sessions && d != __cur_session ) {
					cur_run += agh_get_n_subjects_in_group(g) * agh_get_n_channels();
					continue;
				}

				const gchar *d_name = agh_get_nth_session_name(d);

				for ( h = 0; h < agh_get_n_channels(); h++, cur_run++ ) {
					agh_current_channel = h;
					if ( !agh_sim_runbatch_include_all_channels && h != __cur_channel )
						continue;

					snprintf( label_text, 59, "<big>%s (%s) <b>%s %s</b></big>",
						  j_name, g_name,
						  agh_get_nth_session_name(d), agh_get_nth_channel_name(h));
					gtk_label_set_markup( GTK_LABEL (lBatchRunStatus), label_text);
					gtk_progress_bar_set_fraction( GTK_PROGRESS_BAR (rBatchRunPercentComplete),
								       (double)(cur_run) / total_runs);
					while ( gtk_events_pending() )
						gtk_main_iteration();

					if ( __interrupt_batch )
						goto out;

					if ( agh_sim_modelrun_setup( agh_get_nth_subject_name_in_group(g,j),
								     agh_get_current_channel_name(),
								     NULL, NULL, NULL, &R) != 0 )
						continue;

					SConsumerTunableSet t_set;
					agh_sim_modelrun_get_snapshot( R, -1, NULL, NULL, &t_set, NULL);

					if ( (agh_sim_runbatch_redo_option == AGH_BATCHRUN_REDOSKIP      && !(t_set.status & AGH_MODRUN_FITSUFFICIENT && t_set.status & AGH_MODRUN_ALLREQSTABLE)) ||
					     (agh_sim_runbatch_redo_option == AGH_BATCHRUN_REDOSUCCEEDED &&  (t_set.status & AGH_MODRUN_FITSUFFICIENT && t_set.status & AGH_MODRUN_ALLREQSTABLE)) ||
					     (agh_sim_runbatch_redo_option == AGH_BATCHRUN_REDOFAILED    &&  (t_set.status & AGH_MODRUN_FAILED)) ||
					     (agh_sim_runbatch_redo_option == AGH_BATCHRUN_REDOALWAYS) ) {

						agh_sim_modelrun_do_stride( R, 0, NULL);
						do {
							agh_sim_modelrun_do_stride( R, -1, NULL);
							agh_sim_modelrun_get_snapshot( R, -1, NULL, NULL, &t_set, NULL);
						} while ( !(t_set.status & AGH_MODRUN_FITSUFFICIENT && t_set.status & AGH_MODRUN_ALLREQSTABLE) &&
							  !(t_set.status & AGH_MODRUN_FAILED) );

						agh_sim_modelrun_save( R);
					}
				}
			}
		}

out:
	agh_current_channel = __cur_channel, agh_current_session = __cur_session;

	agh_populate_mSimulations();

	gtk_widget_hide( wBatchRunProgress);
	agh_set_cursor_busy( FALSE, wMainWindow);
}



// EOF
