// ;-*-C-*- *  Time-stamp: "2008-06-16 02:54:04 hmmr"

/***************************************************************************
 *            agh_ui-statusbar.c
 *
 *  Copyright  2008  Andrei Zavada
 *  Email johnhommer@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <gtk/gtk.h>
#include <glade/glade.h>
#include "agh_iface.hh"
#include "agh_ui.h"


GtkListStore	*agh_mExpDesignList;

gchar	*agh_last_expdesign_dir;
gint	 agh_last_expdesign;
static GString	*agh_expdesign_hist_filename;

gchar *agh_wMainWindow_title;

GtkWidget
	*sbMainStatusBar;

static GtkWidget
	*wAbout,
	*wExpDesignChooser,
	*tvExpDesignList,
	*bExpDesignSelect;

guint agh_sb_context_id_General;
guint agh_sb_context_id_DesignSetup;
guint agh_sb_context_id_DesignMeasurements;
guint agh_sb_context_id_Simulations;


static void
agh_desensitize_select()
{
	gtk_widget_set_sensitive( bExpDesignSelect, FALSE);
}

// -------------------------------

gint
agh_construct_StatusBar( GladeXML *xml)
{
	if ( !(sbMainStatusBar = glade_xml_get_widget( xml, "sbMainStatusBar")) ||
	     !(wAbout = glade_xml_get_widget( xml, "wAbout")) ||
	     !(wExpDesignChooser = glade_xml_get_widget( xml, "wExpDesignChooser")) ||
	     !(tvExpDesignList = glade_xml_get_widget( xml, "tvExpDesignList")) ||
	     !(bExpDesignSelect = glade_xml_get_widget( xml, "bExpDesignSelect")) )
		return -1;

	agh_sb_context_id_General = gtk_statusbar_get_context_id( GTK_STATUSBAR (sbMainStatusBar), "General context");
	agh_sb_context_id_DesignSetup = gtk_statusbar_get_context_id( GTK_STATUSBAR (sbMainStatusBar), "Design Setup");
	agh_sb_context_id_DesignMeasurements = gtk_statusbar_get_context_id( GTK_STATUSBAR (sbMainStatusBar), "Design Measurements");
	agh_sb_context_id_Simulations = gtk_statusbar_get_context_id( GTK_STATUSBAR (sbMainStatusBar), "Simulations");

	g_signal_connect( wExpDesignChooser, "delete-event", G_CALLBACK (gtk_main_quit), NULL);
	g_signal_connect( wExpDesignChooser, "show", G_CALLBACK (agh_desensitize_select), NULL);

	gtk_tree_view_set_model( GTK_TREE_VIEW (tvExpDesignList),
				 GTK_TREE_MODEL (agh_mExpDesignList));

	g_object_set( G_OBJECT (tvExpDesignList),
		      "headers-visible", FALSE,
		      NULL);

	GtkCellRenderer *renderer;
	renderer = gtk_cell_renderer_text_new();
	g_object_set( G_OBJECT (renderer), "editable", FALSE, NULL);
	g_object_set_data( G_OBJECT (renderer), "column", GINT_TO_POINTER (0));
	gtk_tree_view_insert_column_with_attributes( GTK_TREE_VIEW (tvExpDesignList),
						     -1, "ExpDesign", renderer,
						     "text", 0,
						     NULL);

	return 0;
}







void
agh_histfile_read()
{
	if ( !agh_expdesign_hist_filename ) {
		agh_expdesign_hist_filename = g_string_sized_new( 80);
		g_string_printf( agh_expdesign_hist_filename,
				 "%s/.config/aghermann/sessions", g_get_home_dir());
	}

	if ( agh_last_expdesign_dir )
		g_free( agh_last_expdesign_dir);

	GKeyFile *kf = g_key_file_new();
	GError *kf_error = NULL;

	gint last = -1;
	gchar **list = NULL;
	g_key_file_load_from_file( kf, agh_expdesign_hist_filename->str, G_KEY_FILE_KEEP_COMMENTS, &kf_error);
	if ( !kf_error )
		last = g_key_file_get_integer( kf, "Sessions", "Last", &kf_error);
	if ( !kf_error )
		list = g_key_file_get_string_list( kf, "Sessions", "List", NULL, &kf_error);

	GtkTreeIter iter;
	if ( last != -1 ) {
		agh_last_expdesign_dir = g_strdup( list[last]);
		agh_last_expdesign = last;

		guint i = 0;
		gchar *entry;
		if ( list ) {
			gtk_list_store_clear( agh_mExpDesignList);
			while ( entry = list[i++] ) {
				gtk_list_store_append( agh_mExpDesignList, &iter);
				gtk_list_store_set( agh_mExpDesignList, &iter,
						    0, entry,
						    -1);
			}
		}
	} else {
		gchar *cwd = g_get_current_dir();
		agh_last_expdesign_dir = g_build_filename( G_DIR_SEPARATOR_S,
							   cwd, "NewExperiment", NULL);
		g_free( cwd);

		agh_last_expdesign = 0;

		gtk_list_store_clear( agh_mExpDesignList);
		gtk_list_store_append( agh_mExpDesignList, &iter);
		gtk_list_store_set( agh_mExpDesignList, &iter,
				    0, agh_last_expdesign_dir,
				    -1);

		g_error_free( kf_error);
	}

	agh_wMainWindow_title = g_strdup_printf( "Aghermann: %s", agh_last_expdesign_dir);
	gtk_window_set_title( GTK_WINDOW (wMainWindow), agh_wMainWindow_title);

	if ( list )  g_strfreev( list);
	g_key_file_free( kf);
}





void
agh_histfile_write()
{
	GKeyFile *kf = g_key_file_new();
	GError *kf_error = NULL;

	gchar	 *entry;
	const gchar * const *list;
	GString *agg = g_string_sized_new( 400);
	GtkTreeIter iter;
	gsize n = 0;
	gtk_tree_model_get_iter_first( GTK_TREE_MODEL (agh_mExpDesignList), &iter);
	do {
		gtk_tree_model_get( GTK_TREE_MODEL (agh_mExpDesignList), &iter,  // at least one entry exists,
				    0, &entry,                                   // added by prec func
				    -1);
		g_string_append_printf( agg, "%s;", entry);
		g_free( entry);
		n++;
	} while ( gtk_tree_model_iter_next( GTK_TREE_MODEL (agh_mExpDesignList), &iter) );
	list = g_strsplit( agg->str, ";", -1);

	g_key_file_set_string_list( kf, "Sessions", "List", list, n);
	g_key_file_set_integer( kf, "Sessions", "Last", agh_last_expdesign);

	g_string_free( agg, TRUE);
	if ( list )  g_strfreev( list);

	gchar *dirname = g_path_get_dirname( agh_expdesign_hist_filename->str);
	g_mkdir_with_parents( dirname, 0755);
	g_free( dirname);

	gchar *towrite = g_key_file_to_data( kf, NULL, NULL);
	g_file_set_contents( agh_expdesign_hist_filename->str, towrite, -1, NULL);
	g_free( towrite);

	g_key_file_free( kf);
}








void
tvExpDesignList_cursor_changed_cb()
{
	GtkTreeSelection *selection = gtk_tree_view_get_selection( GTK_TREE_VIEW (tvExpDesignList));
	gtk_widget_set_sensitive( bExpDesignSelect, gtk_tree_selection_count_selected_rows( selection) > 0);
}




void
bExpDesignSelect_clicked_cb()
{
	GtkTreeSelection *selection = gtk_tree_view_get_selection( GTK_TREE_VIEW (tvExpDesignList));
	GtkTreeModel *model;
	GList *paths = gtk_tree_selection_get_selected_rows( selection, &model);
	GtkTreePath *path = (GtkTreePath*) g_list_nth_data( paths, 0);
	g_list_free( paths);

	GtkTreeIter iter;
	gtk_tree_model_get_iter( model, &iter, path);
	gtk_tree_model_get( model, &iter, 0, &agh_last_expdesign_dir, -1);
	gint selected = gtk_tree_path_get_indices( path)[0];

	gtk_tree_path_free( path);

	if ( selected != agh_last_expdesign ) {
		agh_last_expdesign = selected;

		agh_expdesign_shutdown();

		agh_expdesign_init();
//		agh_populate_models();

		if ( agh_wMainWindow_title )
			g_free( agh_wMainWindow_title);
		agh_wMainWindow_title = g_strdup_printf( "Aghermann: %s", agh_last_expdesign_dir);
		gtk_window_set_title( GTK_WINDOW (wMainWindow), agh_wMainWindow_title);
	}

	gtk_widget_hide( wExpDesignChooser);
	gtk_widget_show( wMainWindow);
}





void
bExpDesignQuit_clicked_cb()
{
	GtkTreeSelection *selection = gtk_tree_view_get_selection( GTK_TREE_VIEW (tvExpDesignList));
	GtkTreeModel *model;
	GList *paths = gtk_tree_selection_get_selected_rows( selection, &model);
	if ( paths ) {
		GtkTreePath *path = (GtkTreePath*) g_list_nth_data( paths, 0);
		g_list_free( paths);

		agh_last_expdesign = gtk_tree_path_get_indices( path)[0];

		gtk_tree_path_free( path);
	}

	gtk_main_quit();
}

void
bExpDesignCreateNew_clicked_cb()
{
	GtkWidget *dir_chooser = gtk_file_chooser_dialog_new( "Locate New Experiment Directory",
							      NULL,
							      GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
							      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
							      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
							      NULL);
	if ( gtk_dialog_run( GTK_DIALOG (dir_chooser)) == GTK_RESPONSE_ACCEPT ) {
		gchar *new_dir = gtk_file_chooser_get_filename( GTK_FILE_CHOOSER (dir_chooser));

		GtkTreeIter iter;
		gtk_list_store_append( agh_mExpDesignList, &iter);
		gtk_list_store_set( agh_mExpDesignList, &iter,
				    0, new_dir,
				    -1);
		g_free( new_dir);

		GtkTreePath *path = gtk_tree_model_get_path( GTK_TREE_MODEL (agh_mExpDesignList), &iter);
		gtk_tree_view_set_cursor( GTK_TREE_VIEW (tvDesignSetupGroupsSubjects),
					  path,
					  gtk_tree_view_get_column( GTK_TREE_VIEW (tvExpDesignList),
								    0),
					  FALSE);

		gtk_tree_path_free( path);
	}

	gtk_widget_destroy( dir_chooser);
}

void
bExpDesignRemove_clicked_cb()
{
	GtkTreeSelection *selection = gtk_tree_view_get_selection( GTK_TREE_VIEW (tvExpDesignList));
	GtkTreeModel *model;
	GList *paths = gtk_tree_selection_get_selected_rows( selection, &model);
	if ( !paths )
		return;
	GtkTreePath *path = (GtkTreePath*) g_list_nth_data( paths, 0);
	g_list_free( paths);

	GtkTreeIter iter;
	gtk_tree_model_get_iter( model, &iter, path);
	gtk_list_store_remove( agh_mExpDesignList, &iter);

	gtk_tree_path_free( path);
}










void
iExpChange_activate_cb()
{
	gtk_widget_hide( wMainWindow);
	gtk_widget_show( wExpDesignChooser);
}

void
iExpQuit_activate_cb()
{
	gtk_main_quit();
}





void
iAbout_activate_cb()
{
	gtk_widget_show( wAbout);
}



// ---------- misc
void
bDump_clicked_cb()
{
	agh_dump_all();
}



// EOF
