// ;-*-C-*- *  Time-stamp: "2008-06-24 16:05:45 hmmr"

/***************************************************************************
 *            agh_ui.c
 *
 *  Copyright  2008  Andrei Zavada
 *  Email johnhommer@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <unistd.h>
#include "agh_iface.hh"
#include "agh_ui.h"




GtkWidget *wMainWindow;




void
agh_pop_ok_message( const gchar *str)
{
	GtkWidget *msg = gtk_message_dialog_new( GTK_WINDOW (wMainWindow),
						 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
						 GTK_MESSAGE_INFO,
						 GTK_BUTTONS_OK,
						 str, NULL);
	gtk_dialog_run( GTK_DIALOG (msg));
	gtk_widget_destroy( msg);
}


gint
agh_pop_question( const gchar *str)
{
	GtkWidget *msg = gtk_message_dialog_new( GTK_WINDOW (wMainWindow),
						 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
						 GTK_MESSAGE_INFO,
						 GTK_BUTTONS_YES_NO,
						 str, NULL);
	gint response = gtk_dialog_run( GTK_DIALOG (msg));
	gtk_widget_destroy( msg);
	return response;
}




void
agh_set_cursor_busy( gboolean busy, GtkWidget *wid)
{
	static GdkCursor *cursor_busy   = NULL,
			 *cursor_normal = NULL;
	if ( !cursor_normal ) {
//		cursor_busy   = gdk_cursor_new( GDK_WATCH);
//		cursor_normal = gdk_cursor_new( GDK_LEFT_PTR);
		cursor_busy   = gdk_cursor_new_from_name( gdk_display_get_default(), "wait");
		cursor_normal = gdk_cursor_new_from_name( gdk_display_get_default(), "default");
	}
	if ( busy )
		gdk_window_set_cursor( wid->window, cursor_busy);
	else
		gdk_window_set_cursor( wid->window, cursor_normal);

	while ( gtk_events_pending () )
		gtk_main_iteration();
}







gint
agh_ui_init( char *argv0)
{
	gint retval = 0;
	GladeXML *xml = NULL;

	gchar	ui_file_relative[] = "ui/agh-ui.glade";
	GString *ui_file = g_string_sized_new( 180);
	g_string_printf( ui_file, "%s/%s", argv0, ui_file_relative);
//	printf( "..looking for %s\n", ui_file->str);
	if ( access( ui_file->str, R_OK) ) {
		g_string_printf( ui_file, "/usr/share/" PACKAGE "/%s", ui_file_relative);
//		printf( "..looking for %s\n", ui_file->str);
		if ( access( ui_file->str, R_OK) ) {
			g_string_printf( ui_file, "/usr/local/share/" PACKAGE "/%s", ui_file_relative);
//			printf( "..looking for %s\n", ui_file->str);
			if ( access( ui_file->str, R_OK) ) {
				printf( "UI Init: failed to find %s.\n", ui_file_relative);
				retval = -2;
				goto fail;
			}
		}
	}

	if ( !(xml = glade_xml_new( ui_file->str, NULL, NULL)) ||
	     !(wMainWindow = glade_xml_get_widget( xml, "wMainWindow")) ) {
		printf( "UI Init: Failed to construct ui from %s\n", ui_file->str);
		retval = -1;
		goto fail;
	}

	glade_xml_signal_autoconnect( xml);
	g_signal_connect( wMainWindow, "destroy", G_CALLBACK (gtk_main_quit), NULL);


	agh_create_models();

      // now construct treeviews which glade failed to
	if ( agh_construct_DesignSetup( xml)        ||
	     agh_construct_DesignMeasurements( xml) ||
	     agh_construct_Settings( xml)           ||
	     agh_construct_Simulations( xml)        ||
	     agh_construct_SimulationParams( xml)   ||
	     agh_construct_ScoringFacility( xml)    ||
	     agh_construct_ModelRun( xml)           ||
	     agh_construct_StatusBar( xml) ) {
		printf( "UI Init: Failed to construct (some) widgets\n");
		retval = -2;
		goto fail;
	}

fail:
	g_string_free( ui_file, TRUE);

	if ( xml )
		g_object_unref( xml);

	return retval;
}







gint
agh_ui_shutdown()
{
	return 0;
}





gint
agh_create_models()
{
	agh_mDesignSetupGroupsSubjects	= gtk_tree_store_new( 5,
							      G_TYPE_STRING,
							      G_TYPE_STRING,
							      G_TYPE_UINT,
							      G_TYPE_STRING,
							      G_TYPE_BOOLEAN);

	agh_mDesignSetupGroups		= gtk_list_store_new( 1,
							      G_TYPE_STRING);

	agh_mDesignSetupEpisodes	= gtk_list_store_new( 1,
							      G_TYPE_STRING);

	agh_mDesignSetupSessions	= gtk_list_store_new( 1,
							      G_TYPE_STRING);

	agh_mDesignSetupChannels	= gtk_list_store_new( 1,
							      G_TYPE_STRING);

	agh_mDesignSetupSources		= gtk_list_store_new( 7,
							      G_TYPE_STRING,
							      G_TYPE_STRING,
							      G_TYPE_STRING,
							      G_TYPE_STRING,
							      G_TYPE_STRING,
							      G_TYPE_STRING,
							      G_TYPE_STRING);


	agh_mDesignMeasurements		= gtk_tree_store_new( 1 + /* agh_n_current_episodes */ 16 + 1,
							      G_TYPE_STRING,
							      G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
							      G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
							      G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
							      G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
							      G_TYPE_BOOLEAN);

	agh_mSimulations = gtk_tree_store_new( 11 + AGH_EGC_MAX-1,
					       G_TYPE_STRING,
					       G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
					       G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
					       G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
					       G_TYPE_STRING, G_TYPE_STRING,
					       G_TYPE_STRING, G_TYPE_STRING,
					       G_TYPE_STRING,
					       G_TYPE_BOOLEAN);


	agh_mExpDesignList = gtk_list_store_new( 1,
						 G_TYPE_STRING);


	agh_mPageSize = gtk_list_store_new( 1,
					    G_TYPE_STRING);
	agh_mFFTParamsWindowType = gtk_list_store_new( 1,
						       G_TYPE_STRING);
	agh_mFFTParamsWindowSize = gtk_list_store_new( 1,
						       G_TYPE_STRING);

	return ( agh_mDesignSetupGroupsSubjects &&
		 agh_mDesignSetupGroups         &&
		 agh_mDesignSetupEpisodes       &&
		 agh_mDesignSetupSessions       &&
		 agh_mDesignSetupChannels       &&
		 agh_mDesignMeasurements        &&
		 agh_mSimulations               &&
		 agh_mPageSize			&&
		 agh_mFFTParamsWindowSize	&&
		 agh_mFFTParamsWindowType	&&
		 agh_mExpDesignList                  );

}







// EOF
