// ;-*-C++-*- *  Time-stamp: "2008-06-05 16:37:15 hmmr"

/***************************************************************************
 *            z_lists.cc
 *
 *  Copyright  2008  Andrei Zavada
 *  Email johnhommer@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <glib.h>

#include "z_lists.hh"





// a more complicated specialization: after deleting an element, we need
// to decrement _seq_no in all the elements following same in sequence
template<> gint
CList<COrderedName>::del( COrderedName *o)
{
	gint deleted_seq_no = o->_seq_no;

	GSList *node = list;
	while ( node )
		if ( *(COrderedName*) (node->data) == *o ) {
			delete (COrderedName*) (node->data);
			list = g_slist_delete_link( list, node);
			_n--;
			break;
		} else
			node = node->next;
	if ( !node )
		return 0;

      // walk list again to adjust higher-ranked entries
	node = list;
	while ( node ) {
		if ( ((COrderedName*)(node->data))->_seq_no > deleted_seq_no )
			((COrderedName*)(node->data))->_seq_no--;
		node = node->next;
	}
	return 1;
}




//
template<> COrderedName*
CList<COrderedName>::o_by_seq_no( gint iseq_no)
{
	GSList *node = list;
	while ( node )
		if ( ((COrderedName*) (node->data))->_seq_no == iseq_no )
			return (COrderedName*) (node->data);
		else
			node = node->next;
	return NULL;
}

template<> COrderedName*
CList<COrderedName>::o_by_name( const gchar *name)
{
	GSList *node = list;
	GQuark q = g_quark_from_string( name);
	while ( node )
		if ( ((COrderedName*) (node->data))->_q == q )
			return (COrderedName*) (node->data);
		else
			node = node->next;
	return NULL;
}



//
template<> gint
CList<COrderedName>::swap( gint seq_no1, gint seq_no2)
{
	GSList *node;
	COrderedName *o1 = NULL, *o2 = NULL;

	node = list;
	while ( node )
		if ( ((COrderedName*) (node->data))->_seq_no == seq_no1 ) {
			o1 = (COrderedName*) (node->data);
			break;
		} else
			node = node->next;
	node = list;
	while ( node )
		if ( ((COrderedName*) (node->data))->_seq_no == seq_no2 ) {
			o2 = (COrderedName*) (node->data);
			break;
		} else
			node = node->next;
	if ( o1 && o2 ) {
		gint fafa = o1->_seq_no;
		o1->_seq_no = o2->_seq_no;
		o2->_seq_no = fafa;
		return 1;
	} else
		return 0;
}

// EOF
